@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/29/99
 *
-* $Revision: 1.74 $
+* $Revision: 1.92 $
 *
 * File Description: 
 *
@@ -58,16 +58,10 @@ NLM_EXTERN void EntrezSetProgramName (
 )
 
 {
-  CharPtr  str;
-
-  str = (CharPtr) GetAppProperty (ENTREZ_TOOL_PROPERTY);
-  MemFree (str);
-
-  if (! StringHasNoText (progname)) {
-    SetAppProperty (ENTREZ_TOOL_PROPERTY, StringSave (progname));
-  } else {
-    SetAppProperty (ENTREZ_TOOL_PROPERTY, NULL);
-  }
+  MemFree (GetAppProperty (ENTREZ_TOOL_PROPERTY));
+  SetAppProperty (ENTREZ_TOOL_PROPERTY, (StringHasNoText (progname)
+                                         ? NULL
+                                         : StringSave (progname)));
 }
 
 static CharPtr EntrezGetProgramName (
@@ -91,40 +85,9 @@ static CharPtr EntrezGetProgramName (
   return ptr;
 }
 
-/* override URL paths or service name */
-#if 0
-static CharPtr  e2_host_machine = NULL;
-static Uint2    e2_host_port = 0;
-static CharPtr  e2_host_path = NULL;
-#endif
+/* override service name */
 static CharPtr  e2_service = NULL;
 
-/* to be phased out, currently overrides ncbi named service, goes directly to URL */
-
-NLM_EXTERN void EntrezSetServer (
-  CharPtr host_machine,
-  Uint2 host_port,
-  CharPtr host_path
-)
-
-{
-#if 0
-  if (! StringHasNoText (host_machine)) {
-    e2_host_machine = MemFree (e2_host_machine);
-    e2_host_machine = StringSaveNoNull (host_machine);
-  }
-  if (host_port != 0) {
-    e2_host_port = host_port;
-  }
-  if (! StringHasNoText (host_path)) {
-    e2_host_path = MemFree (e2_host_path);
-    e2_host_path = StringSaveNoNull (host_path);
-  }
-#else
-  /* define as NOP for now */
-#endif
-}
-
 /* use EntrezTest to override default Entrez ncbi named service */
 
 NLM_EXTERN void EntrezSetService (
@@ -132,90 +95,92 @@ NLM_EXTERN void EntrezSetService (
 )
 
 {
-  if (! StringHasNoText (service)) {
-    e2_service = MemFree (e2_service);
-    e2_service = StringSaveNoNull (service);
-  }
+  MemFree (e2_service);
+  e2_service = StringSaveNoNull (service);
 }
 
 /* low-level connection functions */
 
+static CharPtr GetDbFromE2Request (Entrez2RequestPtr e2rq)
+
+{
+  Entrez2BooleanExpPtr   e2be;
+  Entrez2EvalBooleanPtr  e2eb;
+  Entrez2HierQueryPtr    e2hq;
+  Entrez2IdPtr           e2id;
+  Entrez2IdListPtr       e2il;
+  Entrez2TermPosPtr      e2tp;
+  Entrez2TermQueryPtr    e2tq;
+  ValNodePtr             vnp;
+
+  if (e2rq == NULL) return NULL;
+
+  vnp = e2rq->request;
+  if (vnp == NULL) return NULL;
+
+  switch (vnp->choice) {
+    case E2Request_get_info :
+      break;
+    case E2Request_eval_boolean :
+      e2eb = (Entrez2EvalBooleanPtr) vnp->data.ptrvalue;
+      if (e2eb == NULL) return NULL;
+      e2be = e2eb->query;
+      if (e2be == NULL) return NULL;
+      if (StringDoesHaveText (e2be->db)) return e2be->db;
+      break;
+    case E2Request_get_docsum :
+      e2il = (Entrez2IdListPtr) vnp->data.ptrvalue;
+      if (e2il == NULL) return NULL;
+      if (StringDoesHaveText (e2il->db)) return e2il->db;
+      break;
+    case E2Request_get_term_pos :
+      e2tq = (Entrez2TermQueryPtr) vnp->data.ptrvalue;
+      if (e2tq == NULL) return NULL;
+      if (StringDoesHaveText (e2tq->db)) return e2tq->db;
+      break;
+    case E2Request_get_term_list :
+      e2tp = (Entrez2TermPosPtr) vnp->data.ptrvalue;
+      if (e2tp == NULL) return NULL;
+      if (StringDoesHaveText (e2tp->db)) return e2tp->db;
+      break;
+    case E2Request_get_term_hierarchy :
+      e2hq = (Entrez2HierQueryPtr) vnp->data.ptrvalue;
+      if (e2hq == NULL) return NULL;
+      if (StringDoesHaveText (e2hq->db)) return e2hq->db;
+      break;
+    case E2Request_get_links :
+      break;
+    case E2Request_get_linked :
+      break;
+    case E2Request_get_link_counts :
+      e2id = (Entrez2IdPtr) vnp->data.ptrvalue;
+      if (e2id == NULL) return NULL;
+      if (StringDoesHaveText (e2id->db)) return e2id->db;
+      break;
+    default :
+      break;
+  }
+
+  return NULL;
+}
+
 NLM_EXTERN CONN EntrezOpenConnection (
-  void
+  Entrez2RequestPtr e2rq
 )
 
 {
-#if 0
-  CharPtr  host_machine = e2_host_machine;
-  Uint2    host_port = e2_host_port;
-  CharPtr  host_path = e2_host_path;
-#endif
-  CharPtr  host_service = e2_service;
-#if 0
-#ifdef OS_UNIX
-  CharPtr  env_machine = NULL;
-  CharPtr  env_path = NULL;
-  CharPtr  env_port = NULL;
-  CharPtr  env_service = NULL;
-  int      val = 0;
-#endif
+  Char     arg [128];
+  CharPtr  db;
 
-#ifdef OS_UNIX
-  env_machine = (CharPtr) getenv ("NCBI_ENTREZ2_HOST_MACHINE");
-  if (! StringHasNoText (env_machine)) {
-    host_machine = env_machine;
-  }
-#endif
-  if (StringHasNoText (host_machine)) {
-    host_machine = "www.ncbi.nlm.nih.gov";
-  }
+  db = GetDbFromE2Request (e2rq);
+  if (StringDoesHaveText (db) && StringLen (db) < 100) {
+    StrCpy (arg,    "DB=");
+    StrCpy (arg + 3, db);
+  } else
+    *arg = '\0';
 
-#ifdef OS_UNIX
-  env_port = (CharPtr) getenv ("NCBI_ENTREZ2_HOST_PORT");
-  if (! StringHasNoText (env_port)) {
-    if (sscanf (env_port, "%d", &val) == 1) {
-      host_port = (Uint2) val;
-    }
-  }
-#endif
-  if (host_port == 0) {
-    host_port = 80;
-  }
-
-#ifdef OS_UNIX
-  env_path = (CharPtr) getenv ("NCBI_ENTREZ2_HOST_PATH");
-  if (! StringHasNoText (env_path)) {
-    host_path = env_path;
-  }
-#endif
-  if (StringHasNoText (host_path)) {
-    host_path = "/entrez/eutils/entrez2server.fcgi";
-  }
-
-#ifdef OS_UNIX
-  env_service = (CharPtr) getenv ("NCBI_ENTREZ2_HOST_SERVICE");
-  if (! StringHasNoText (env_service)) {
-    host_service = env_service;
-  }
-#endif
-#endif
-  if (StringHasNoText (host_service)) {
-    host_service = "Entrez2";
-  }
-#if 0
-  /* temporarily for direct access to URL, to be phased out */
-
-  if (e2_host_machine != NULL || e2_host_port != 0 || e2_host_path != NULL) {
-    return QUERY_OpenUrlQuery (host_machine, host_port, host_path,
-                               NULL, EntrezGetProgramName (),
-                               30, eMIME_T_NcbiData, eMIME_AsnBinary,
-                               eENCOD_None, 0);
-  }
-#endif
-
-  /* use new named service, Entrez or EntrezTest */
-
-  return QUERY_OpenServiceQuery (host_service, NULL, 30);
+  return QUERY_OpenServiceQueryEx
+    (StringHasNoText (e2_service) ? "Entrez2" : e2_service, NULL, 30, arg);
 }
 
 #ifdef OS_MAC
@@ -294,7 +259,7 @@ NLM_EXTERN Entrez2ReplyPtr EntrezSynchronousQuery (
   logtimes = (Boolean) ((getenv ("NCBI_LOG_SYNC_QUERY_TIMES")) != NULL);
 #endif
 
-  conn = EntrezOpenConnection ();
+  conn = EntrezOpenConnection (e2rq);
 
   if (conn == NULL) return NULL;
 
@@ -357,7 +322,7 @@ NLM_EXTERN Boolean EntrezAsynchronousQuery (
 
   if (e2rq == NULL) return FALSE;
 
-  conn = EntrezOpenConnection ();
+  conn = EntrezOpenConnection (e2rq);
 
   if (conn == NULL) return FALSE;
 
@@ -1079,9 +1044,13 @@ NLM_EXTERN Boolean ValidateEntrez2InfoPtrEx (
     }
 
     if (e2db->doc_count < 1) {
-      sprintf (buf, "Database %s has no documents", db);
-      ValNodeCopyStr (head, 0, buf);
-      rsult = FALSE;
+      if (StringICmp (db, "Nucleotide") == 0) {
+        /* now a virtual database consolidating NucCore, NucEst, NucGss */
+      } else {
+        sprintf (buf, "Database %s has no documents", db);
+        ValNodeCopyStr (head, 0, buf);
+        rsult = FALSE;
+      }
     }
     if (e2db->field_count < 1 || e2db->fields == NULL) {
       sprintf (buf, "Database %s has no fields", db);
@@ -1092,7 +1061,10 @@ NLM_EXTERN Boolean ValidateEntrez2InfoPtrEx (
       if (StringICmp (db, "books") != 0 &&
           StringICmp (db, "gensat") != 0 &&
           StringICmp (db, "mesh") != 0 &&
-          StringICmp (db, "nlmcatalog") != 0) {
+          StringICmp (db, "nlmcatalog") != 0 &&
+          StringICmp (db, "nuccore") != 0 &&
+          StringICmp (db, "nucgss") != 0 &&
+          StringICmp (db, "nucest") != 0) {
         sprintf (buf, "Database %s has no links", db);
         ValNodeCopyStr (head, 0, buf);
         rsult = FALSE;
@@ -1137,10 +1109,14 @@ NLM_EXTERN Boolean ValidateEntrez2InfoPtrEx (
         rsult = FALSE;
       } else if (StringCmp (fld, "ORGN") == 0) {
         if (e2fip->term_count == 0) {
-          sprintf (buf, "Database %s field %s term count is 0", db, fld);
-          ValNodeCopyStr (head, 0, buf);
-          rsult = FALSE;
-        }
+          if (StringICmp (db, "Nucleotide") == 0) {
+            /* now a virtual database consolidating NucCore, NucEst, NucGss */
+          } else {
+            sprintf (buf, "Database %s field %s term count is 0", db, fld);
+            ValNodeCopyStr (head, 0, buf);
+            rsult = FALSE;
+           }
+       }
       }
       if (StringLen (fld) > 4) {
         sprintf (buf, "Database %s field %s name is > 4 characters long", db, fld);
@@ -1325,6 +1301,17 @@ NLM_EXTERN Boolean ValidateEntrez2InfoPtrEx (
             } else if (StringICmp (last, "Disease") == 0 && StringICmp (str, "Disease-Stage") == 0) {
             } else if (StringICmp (last, "ActiveAid") == 0 && StringICmp (str, "ActiveAidCount") == 0) {
             } else if (StringICmp (last, "InactiveAid") == 0 && StringICmp (str, "InactiveAidCount") == 0) {
+            } else if (StringICmp (last, "Phenotype") == 0 && StringICmp (str, "Phenotype Ontology ID") == 0) {
+            } else if (StringICmp (last, "Title") == 0 && StringICmp (str, "Title Abbreviation") == 0) {
+            } else if (StringICmp (last, "Library") == 0 && StringICmp (str, "Library Class") == 0) {
+            } else if (StringICmp (last, "Sequence") == 0 && StringICmp (str, "Sequence Count") == 0) {
+            } else if (StringICmp (last, "Journal") == 0 && StringICmp (str, "Journal List Identifier") == 0) {
+            } else if (StringICmp (last, "CompoundID") == 0 && StringICmp (str, "CompoundIDActive") == 0) {
+            } else if (StringICmp (last, "MeSHDescription") == 0 && StringICmp (str, "MeSHDescriptionActive") == 0) {
+            } else if (StringICmp (last, "MeSHTerm") == 0 && StringICmp (str, "MeSHTermActive") == 0) {
+            } else if (StringICmp (last, "PharmAction") == 0 && StringICmp (str, "PharmActionActive") == 0) {
+            } else if (StringICmp (last, "SubstanceID") == 0 && StringICmp (str, "SubstanceIDActive") == 0) {
+            } else if (StringICmp (last, "Synonym") == 0 && StringICmp (str, "SynonymActive") == 0) {
             } else {
               sprintf (buf, "Menu names %s [%s] and %s [%s] may be unintended variants", last, dbnames [lastvnp->choice], str, dbnames [vnp->choice]);
               ValNodeCopyStr (head, 0, buf);
