@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   6/28/00
 *
-* $Revision: 1.16 $
+* $Revision: 1.32 $
 *
 * File Description: 
 *
@@ -37,6 +37,58 @@
 * --------------------------------------------------------------------------
 *
 * $Log: qblastapi.c,v $
+* Revision 1.32  2005/11/28 17:49:55  coulouri
+* correction to previous commit; define snprintf for windows
+*
+* Revision 1.31  2005/11/28 15:32:11  coulouri
+* do not overflow string buffer if BioseqGetTitle() returns NULL
+*
+* Revision 1.30  2004/02/23 15:30:02  lavr
+* New (last) parameter "how" added in CONN_Write() API call
+*
+* Revision 1.29  2004/02/20 17:13:01  madden
+* Doubled some timeouts
+*
+* Revision 1.28  2003/11/14 16:23:08  dondosha
+* If query id not local, append title to id to get defline in BLASTGetQuerySummary
+*
+* Revision 1.27  2003/11/12 21:19:03  kans
+* changed query_number to query_num - fixing typo
+*
+* Revision 1.26  2003/11/12 20:49:20  coulouri
+* Replace code accidentally removed between 1.24 and 1.25 in BlastGetQueryBioseqByRIDEx
+*
+* Revision 1.25  2003/11/03 20:50:53  madden
+* Fix problem with content length, remove call to QblastTestRID, add BLASTGetBOByRIDEx
+*
+* Revision 1.24  2003/01/17 20:40:29  madden
+* Put QUERY_NUMBER back in URL for fetching Seqannot
+*
+* Revision 1.23  2003/01/15 22:04:49  madden
+* Moved BLASTGetSeqAnnotByRIDEx from internal
+*
+* Revision 1.22  2003/01/02 15:06:48  boemker
+* Wrote BLASTGetQueryBioseqByRIDEx that extends behavior of
+* BLASTGetQueryBioseqByRID by returning the Bioseq for a particular query
+* (rather than always returning the Bioseq for the first query.  Modified
+* BLASTGetQueryBioseqByRID to use BLASTGetQueryBioseqByRIDEx.  Wrote
+* BLASTGetQuerySummary to return information about a query.
+*
+* Revision 1.21  2002/11/08 22:36:17  kans
+* in synchronous query, loop on == eIO_Timeout instead of != eIO_Success
+*
+* Revision 1.20  2002/08/07 18:45:15  lavr
+* Change from deprecated to current EIO_ReadMethod enums
+*
+* Revision 1.19  2002/07/24 21:09:56  kans
+* reverted ncbi URL
+*
+* Revision 1.18  2002/07/23 16:45:21  kans
+* changed www.ncbi.nlm.nih.gov to www.ncbi.nih.gov
+*
+* Revision 1.17  2002/04/12 14:27:35  madden
+* Check RID for internal jobs, send to appropriate place
+*
 * Revision 1.16  2001/02/25 21:50:41  kans
 * changes from Uint4 to size_t due to connection type changes
 *
@@ -72,6 +124,12 @@
 #include <Events.h>
 #endif
 
+#ifdef _MSC_VER
+#define strcasecmp _stricmp
+#define strdup _strdup
+#define snprintf _snprintf
+#endif
+
 static CharPtr ReadALine (CharPtr str, size_t size, FILE *fp)
 
 {
@@ -128,7 +186,7 @@ NLM_EXTERN EIO_Status QBlastWaitForReply (
 #endif
 
   starttime = GetSecs ();
-  while ((status = CONN_Wait (conn, eIO_Read, &timeout)) != eIO_Success && max < 300) {
+  while ((status = CONN_Wait (conn, eIO_Read, &timeout)) == eIO_Timeout && max < 300) {
     currtime = GetSecs ();
     max = currtime - starttime;
 #ifdef OS_MAC
@@ -500,7 +558,8 @@ NLM_EXTERN Boolean QBlastAsynchronousRequest (
   FileClose (fp);
 
   fp = FileOpen (path, "r");
-  CONN_Write (conn, (const void *) str, StringLen (str), &n_written);
+  CONN_Write (conn, (const void *) str, StringLen (str),
+              &n_written, eIO_WritePersist);
   QUERY_CopyFileToQuery (conn, fp);
   FileClose (fp);
 
@@ -605,7 +664,8 @@ NLM_EXTERN Int4 QBlastCheckQueue (
           sprintf (str, "RID=%s&CMD=Get\n", curr->rid);
           conn = QBlastOpenConnection ();
           if (conn != NULL) {
-            CONN_Write (conn, (const void *) str, StringLen (str), &n_written);
+            CONN_Write (conn, (const void *) str, StringLen (str),
+                        &n_written, eIO_WritePersist);
             QUERY_SendQuery (conn);
             QUERY_AddToQueue (&(curr->connqueue), conn, SecondQBlastCallback, (Pointer) curr, TRUE);
           } else {
@@ -622,7 +682,8 @@ NLM_EXTERN Int4 QBlastCheckQueue (
       sprintf (str, "CMD=Get&RID=%s&FORMAT_OBJECT=Alignment&FORMAT_TYPE=ASN.1\n", curr->rid);
       conn = QBlastOpenConnection ();
       if (conn != NULL) {
-        CONN_Write (conn, (const void *) str, StringLen (str), &n_written);
+        CONN_Write (conn, (const void *) str, StringLen (str),
+                    &n_written, eIO_WritePersist);
         QUERY_SendQuery (conn);
         QUERY_AddToQueue (&(curr->connqueue), conn, ThirdQBlastCallback, (Pointer) curr, TRUE);
       } else {
@@ -654,7 +715,8 @@ NLM_EXTERN Boolean QBlastCheckRequest (
   sprintf (str, "RID=%s&CMD=Get\n", rid);
   conn = QBlastOpenConnection ();
   if (conn == NULL) return FALSE;
-  CONN_Write (conn, (const void *) str, StringLen (str), &n_written);
+  CONN_Write (conn, (const void *) str, StringLen (str),
+              &n_written, eIO_WritePersist);
   QUERY_SendQuery (conn);
 
   QBlast_AddToQueue (queue, resultproc, announceproc, userdata, conn, NULL, rid);
@@ -731,6 +793,23 @@ static void LIBCALLBACK AsnIoErrorFunc(Int2 type, CharPtr message)
    finished and results are exists on the Qblast repository */
 
 NLM_EXTERN SeqAnnotPtr BLASTGetSeqAnnotByRID(CharPtr RID)
+{
+    return BLASTGetSeqAnnotByRIDEx(RID, NULL, 0, NULL, 0);
+}
+
+/*
+	Returns a SeqAnnot for a given RID.  
+	If host_machine and host_path are set to NULL and host_port to zero
+	Default servers will be used.
+
+	if query_number is set to zero all will be returned, otherwise the
+	one-based offset query will be returned.
+ */
+NLM_EXTERN SeqAnnotPtr BLASTGetSeqAnnotByRIDEx(CharPtr RID,
+					       Nlm_CharPtr host_machine, 
+					       Nlm_Uint2 host_port, 
+					       Nlm_CharPtr host_path,
+					       Int4 query_number)
 {
     Char         query_string[256];
     CONN         conn;
@@ -740,20 +819,21 @@ NLM_EXTERN SeqAnnotPtr BLASTGetSeqAnnotByRID(CharPtr RID)
     AsnIoConnPtr aicp;
     EIO_Status   status;
 
-    /* sprintf(query_string, "RID=%s&ALIGNMENT_VIEW=11", RID); */
-    
-    sprintf(query_string, "FORMAT_TYPE=ASN.1&CMD=Get&RID=%s&"
-            "FORMAT_OBJECT=Alignment", RID);
-    
-    /*    conn = QUERY_OpenUrlQuery ("www.ncbi.nlm.nih.gov", 80,  */
-    conn = QUERY_OpenUrlQuery ("www.ncbi.nlm.nih.gov", 80, 
-                               "/blast/Blast.cgi", NULL,  
-                               "BLASTGetSeqAnnotByRID()", 
-                               30, eMIME_T_Application, 
+    sprintf(query_string,
+            "FORMAT_TYPE=ASN.1&CMD=Get&RID=%s&QUERY_NUMBER=%ld&FORMAT_OBJECT=Alignment", 
+            RID,
+            (long) query_number);
+
+    conn = QUERY_OpenUrlQuery ((host_machine == NULL) ? "www.ncbi.nlm.nih.gov" : host_machine, 
+				(host_port <= 0) ? 80 : host_port, 
+                               	(host_path == NULL) ? "/blast/Blast.cgi" : host_path, 
+				NULL, "BLASTGetSeqAnnotByRIDEx()", 
+                               240, eMIME_T_Application, 
                                eMIME_WwwForm, eENCOD_Url, 0);
     
-    length = StringLen(query_string)+1;
-    status = CONN_Write (conn, query_string, length , &n_written);    
+    length = StringLen(query_string);
+    status = CONN_Write (conn, query_string, length,
+                         &n_written, eIO_WritePersist);
     QUERY_SendQuery (conn);
     
     aicp = QUERY_AsnIoConnOpen ("r", conn);
@@ -765,10 +845,16 @@ NLM_EXTERN SeqAnnotPtr BLASTGetSeqAnnotByRID(CharPtr RID)
     CONN_Close(conn);
     return annot;
 }
-/* Function to get SeqAnnot for RID. We suupose, that search already
+
+/* Function to get Bioseq for RID. We suupose, that search already
    finished and results are exists on the Qblast repository */
 
 NLM_EXTERN BioseqPtr BLASTGetQueryBioseqByRID(CharPtr RID)
+{
+	return BLASTGetQueryBioseqByRIDEx(RID, 0);
+}
+
+NLM_EXTERN BioseqPtr BLASTGetQueryBioseqByRIDEx(CharPtr RID, int query_num)
 {
     Char         query_string[256];
     CONN         conn;
@@ -777,22 +863,21 @@ NLM_EXTERN BioseqPtr BLASTGetQueryBioseqByRID(CharPtr RID)
     Int4         length;
     AsnIoConnPtr aicp;
     EIO_Status   status;
-    
-    /* sprintf(query_string, "RID=%s&ALIGNMENT_VIEW=11", RID); */
-    
-    sprintf(query_string, "CMD=Get&RID=%s&FORMAT_OBJECT=Bioseq&"
-            "FORMAT_TYPE=ASN.1", RID);
-    
-    /* conn = QUERY_OpenUrlQuery ("www.ncbi.nlm.nih.gov", 80,  */
+
+    sprintf(query_string,
+            "FORMAT_TYPE=ASN.1&CMD=Get&RID=%s&QUERY_NUMBER=%ld&FORMAT_OBJECT=Bioseq", 
+            RID,
+            (long) query_num);
     
     conn = QUERY_OpenUrlQuery ("www.ncbi.nlm.nih.gov", 80, 
                                "/blast/Blast.cgi", NULL, 
-                               "BLASTGetQueryBioseqByRID()", 
-                               30, eMIME_T_Application, 
+                               "BLASTGetQueryBioseqByRIDEx()", 
+                               60, eMIME_T_Application, 
                                eMIME_WwwForm, eENCOD_Url, 0);
 
     length = StringLen(query_string);
-    status = CONN_Write (conn, query_string, length , &n_written);    
+    status = CONN_Write (conn, query_string, length,
+                         &n_written, eIO_WritePersist);
     QUERY_SendQuery (conn);
     
     aicp = QUERY_AsnIoConnOpen ("r", conn);
@@ -805,10 +890,41 @@ NLM_EXTERN BioseqPtr BLASTGetQueryBioseqByRID(CharPtr RID)
     return bsp;
 }
 
+NLM_EXTERN Nlm_Boolean BLASTGetQuerySummary(CharPtr RID, Int4 query_number,
+    CharPtr *defline, Int4Ptr query_length)
+{
+    BioseqPtr bsp = BLASTGetQueryBioseqByRIDEx(RID, query_number);
+    if(!bsp)
+	return FALSE;
+    if (bsp->id->choice == SEQID_LOCAL) {
+       *defline = StringSave(BioseqGetTitle(bsp));
+    } else {
+#define SEQID_LENGTH 255
+       Int4 id_length;
+       Char tmp[SEQID_LENGTH+1];
+       Int4 title_length = StringLen(BioseqGetTitle(bsp));
+       SeqIdWrite(bsp->id, tmp, PRINTID_FASTA_LONG, SEQID_LENGTH);
+       id_length = StringLen(tmp);
+       title_length += id_length + 3;
+       *defline = (CharPtr) MemNew(title_length*sizeof(Char));
+       snprintf(*defline, title_length, "%s %s", tmp, BioseqGetTitle(bsp));
+    }
+    *query_length = BioseqGetLen(bsp);
+    BioseqFree(bsp);
+    return TRUE;
+}
+
+
 /* Function to get BlastObject for RID. We suupose, that search already
    finished and results are exists on the Qblast repository. Blast Object
    ASN.1 will be returned as CharPtr buffer*/
-NLM_EXTERN CharPtr BLASTGetBOByRID(CharPtr RID)
+/*
+ * retrive blast object
+ */
+NLM_EXTERN CharPtr BLASTGetBOByRIDEx(CharPtr RID,
+                                     Nlm_CharPtr host_machine,
+                                     Nlm_Uint2 host_port,
+                                     Nlm_CharPtr host_path)
 {
     Char         query_string[256];
     CONN         conn;
@@ -818,19 +934,19 @@ NLM_EXTERN CharPtr BLASTGetBOByRID(CharPtr RID)
     EIO_Status   status;
     CharPtr      in_buff;
     
-/*
-    sprintf(query_string, "RID=%s&ALIGNMENT_VIEW=13", RID);
-*/
     sprintf(query_string, "CMD=Get&RID=%s&FORMAT_OBJECT=BlastObject&FORMAT_TYPE=ASN.1", RID);
-    
-    conn = QUERY_OpenUrlQuery ("www.ncbi.nlm.nih.gov", 80, 
-                               "/blast/Blast.cgi", NULL, 
-                               "BLASTGetBOByRID()", 
-                               30, eMIME_T_Application, 
+
+    conn = QUERY_OpenUrlQuery ((host_machine == NULL) ? "www.ncbi.nlm.nih.gov" : host_machine,
+                                (host_port <= 0) ? 80 : host_port,
+                                (host_path == NULL) ? "/blast/Blast.cgi" : host_path,
+				NULL,
+                               "BLASTGetBOByRIDEx()", 
+                               60, eMIME_T_Application, 
                                eMIME_WwwForm, eENCOD_Url, 0);
 
-    length = StringLen(query_string) + 1;
-    status = CONN_Write (conn, query_string, length , &n_written);    
+    length = StringLen(query_string);
+    status = CONN_Write (conn, query_string, length,
+                         &n_written, eIO_WritePersist);
     QUERY_SendQuery (conn);
     
     new_size = 1024;   
@@ -841,8 +957,7 @@ NLM_EXTERN CharPtr BLASTGetBOByRID(CharPtr RID)
 
     buff_len = 0;
     while ((status = CONN_Read(conn, in_buff + buff_len, 
-                               1024, &bytes, eIO_Plain)) == eIO_Success) {
-        
+                               1024, &bytes, eIO_ReadPlain)) == eIO_Success) {
         if(bytes == 0) 
             break;
 
@@ -860,3 +975,8 @@ NLM_EXTERN CharPtr BLASTGetBOByRID(CharPtr RID)
     CONN_Close(conn);
     return in_buff;
 }
+
+NLM_EXTERN CharPtr BLASTGetBOByRID(CharPtr RID)
+{
+	return BLASTGetBOByRIDEx(RID, NULL, 0, NULL);
+}
