@@ -1,4 +1,4 @@
-/* $Id: blast_tabular.c,v 1.27 2005/04/19 13:56:48 kans Exp $
+/* $Id: blast_tabular.c,v 1.32 2006/04/25 17:59:02 papadopo Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -28,7 +28,7 @@
  * On-the-fly tabular formatting of BLAST results
  */
 
-static char const rcsid[] = "$Id: blast_tabular.c,v 1.27 2005/04/19 13:56:48 kans Exp $";
+static char const rcsid[] = "$Id: blast_tabular.c,v 1.32 2006/04/25 17:59:02 papadopo Exp $";
 
 #include <algo/blast/api/blast_tabular.h>
 #include <algo/blast/core/blast_util.h>
@@ -48,13 +48,15 @@ static char const rcsid[] = "$Id: blast_tabular.c,v 1.27 2005/04/19 13:56:48 kan
 
 BlastTabularFormatData*
 BlastTabularFormatDataNew(FILE* outfp, SeqLoc* query_seqloc,
-                          EBlastTabularFormatOptions format_option)
+                          EBlastTabularFormatOptions format_option,
+                          Boolean believe_query)
 {
    BlastTabularFormatData* tf_data = 
       (BlastTabularFormatData*) calloc(1, sizeof(BlastTabularFormatData));
    tf_data->outfp = outfp;
    tf_data->query_slp = query_seqloc;
    tf_data->format_options = format_option;
+   tf_data->believe_query = believe_query;
 
    return tf_data;
 }
@@ -161,7 +163,6 @@ FillNuclSequenceBuffers(EBlastProgramType program, BlastHSP* hsp,
    Int4* starts;
    Int4* lengths;
    Int4 offset;
-   GapEditScript* esp;
    Int4 start1, start2;
    char* buffer;
    Boolean reverse;
@@ -174,12 +175,11 @@ FillNuclSequenceBuffers(EBlastProgramType program, BlastHSP* hsp,
    reverse = (hsp->query.frame != hsp->subject.frame);
 
    /* Calculate number of segments. */
-   esp = hsp->gap_info;
-   for (numseg = 0; esp; esp = esp->next, ++numseg);
+   numseg = hsp->gap_info->size;
    /* Find the starts and lengths of each segment. */
    start1 = hsp->query.offset;
    start2 = hsp->subject.offset;
-   GapCollectDataForSeqalign(hsp, hsp->gap_info, numseg, query_length,
+   GapCollectDataForSeqalign(hsp, hsp->gap_info, 0, numseg, query_length,
                              subject_length, translate1, translate2,
                              &starts, &lengths, NULL, &start1, &start2);
 
@@ -278,6 +278,7 @@ void* Blast_TabularFormatThread(void* data)
    char* descr;
    Int4 num_queries;
    Int4* query_lengths;
+   Boolean sequence_in_use = FALSE;
  
    tf_data = (BlastTabularFormatData*) data;
    if (!tf_data || !tf_data->query_slp || !tf_data->hsp_stream ||
@@ -307,8 +308,10 @@ void* Blast_TabularFormatThread(void* data)
    query_lengths = (Int4*) malloc(num_queries*sizeof(Int4));
 
    for (index = 0, slp = tf_data->query_slp; slp; ++index, slp = slp->next) {
-      query_id_array[index] = SeqLocId(slp);
-      query_lengths[index] = SeqLocLen(slp);
+      BioseqPtr bsp = BioseqLockById(SeqLocId(slp));
+      query_id_array[index] = SeqIdSetDup(bsp->id);
+      query_lengths[index] = BioseqGetLen(bsp);
+      BioseqUnlockById(SeqLocId(slp));
    }
 
    one_seq_update_params = (BlastSeqSrcGetTotLen(seq_src) == 0);
@@ -331,6 +334,7 @@ void* Blast_TabularFormatThread(void* data)
          if (BlastSeqSrcGetSequence(seq_src, (void*) &seq_arg) < 0)
              continue;
          
+         sequence_in_use = TRUE;
          if (one_seq_update_params) {
             Int2 status;
             /* This is not a database search, so effective search spaces
@@ -343,6 +347,7 @@ void* Blast_TabularFormatThread(void* data)
                              eff_len_params)) != 0) {
                hsp_list = Blast_HSPListFree(hsp_list);
                BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+               sequence_in_use = FALSE;
                continue;
             }
          }
@@ -352,8 +357,10 @@ void* Blast_TabularFormatThread(void* data)
             ext_params->options, hit_params, gen_code_string);
          /* Return subject sequence unless it is needed for the sequence
             printout */
-         if (tf_data->format_options != eBlastTabularAddSequences)
+         if (tf_data->format_options != eBlastTabularAddSequences) {
             BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+            sequence_in_use = FALSE;
+         }
          /* Recalculate the bit scores, since they might have changed. */
          Blast_HSPListGetBitScores(hsp_list, 
             score_params->options->gapped_calculation, gap_align->sbp);
@@ -369,7 +376,8 @@ void* Blast_TabularFormatThread(void* data)
             description. */
          if (tf_data->show_gi || tf_data->show_accession) {
             Blast_SeqIdGetDefLine(subject_id, &subject_buffer, 
-                                  tf_data->show_gi, tf_data->show_accession); 
+                                  tf_data->show_gi, tf_data->show_accession,
+                                  TRUE); 
          } else {
             if ( !(subject_buffer = (char*) malloc(sizeof(char)*SEQIDLEN_MAX)))
                return NULL;
@@ -390,13 +398,14 @@ void* Blast_TabularFormatThread(void* data)
          subject_buffer = strtok(descr, " \t\n\r");
 
       /* Retrieve the subject sequence if it is needed and this has not 
-	 already been done. */ 
+         already been done. */ 
       if (tf_data->format_options == eBlastTabularAddSequences && 
           !tf_data->perform_traceback) {
           seq_arg.oid = hsp_list->oid;
-          seq_arg.encoding = BLASTNA_ENCODING;
+          seq_arg.encoding = eBlastEncodingNucleotide;
           if (BlastSeqSrcGetSequence(seq_src, (void*) &seq_arg) < 0)
               continue;
+          sequence_in_use = TRUE;
       }
 
       subject_length = BlastSeqSrcGetSeqLen(seq_src, (void*)&hsp_list->oid);
@@ -407,7 +416,8 @@ void* Blast_TabularFormatThread(void* data)
          query_index = 
             Blast_GetQueryIndexFromContext(hsp->context, program);
          Blast_SeqIdGetDefLine(query_id_array[query_index], &query_buffer, 
-                               tf_data->show_gi, tf_data->show_accession);
+                               tf_data->show_gi, tf_data->show_accession,
+                               tf_data->believe_query);
          
          eval_buff_ptr = eval_buff;
          ScoreAndEvalueToBuffers(hsp->bit_score, hsp->evalue, 
@@ -465,8 +475,10 @@ void* Blast_TabularFormatThread(void* data)
          sfree(query_buffer);
       }
 
-      /* Return the subject sequence */
-      BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+      /* Return the subject sequence, if it hasn't yet been done. */
+      if (sequence_in_use)
+          BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+
       fflush(tf_data->outfp);
       sfree(subject_buffer);
       hsp_list = Blast_HSPListFree(hsp_list);
@@ -474,6 +486,11 @@ void* Blast_TabularFormatThread(void* data)
 
    BlastSequenceBlkFree(seq_arg.seq);
 
+   for (index = 0; index<num_queries; ++index)
+   {
+        SeqIdSetFree(query_id_array[index]);
+        query_id_array[index] = NULL;
+   }
    sfree(query_lengths);
    sfree(query_id_array);
 
