@@ -1,4 +1,4 @@
-/* $Id: blast_hits.h,v 1.76 2005/04/28 14:05:55 dondosha Exp $
+/* $Id: blast_hits.h,v 1.92 2006/03/02 13:25:28 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,39 @@
 extern "C" {
 #endif
 
+/** Keeps prelim_hitlist_size and HitSavingOptions
+    together, mostly for use by hspstream. */
+typedef struct SBlastHitsParameters {
+   Int4 prelim_hitlist_size; /**< number of hits saved during preliminary 
+                           part of search. */
+   Int4 hsp_num_max; /**< number of HSPs to save per db sequence. */
+} SBlastHitsParameters; 
+
+/** Sets up small structures used by blast_hit.c and hspstream_collector.c
+ * for saving HSPs.
+ * @param hit_options field hitlist_size and hsp_num_max needed, a pointer to 
+ *      this structure will be stored on resulting structure.[in]
+ * @param ext_options field compositionBasedStats needed here. [in]
+ * @param scoring_options gapped_calculation needed here. [in]
+ * @param retval the allocated SBlastHitsParameters*
+ * @return zero on success, 1 on NULL parameter, 2 if calloc fails.
+ */
+NCBI_XBLAST_EXPORT
+Int2 SBlastHitsParametersNew(const BlastHitSavingOptions* hit_options,
+                             const BlastExtensionOptions* ext_options,
+                             const BlastScoringOptions* scoring_options,
+                             SBlastHitsParameters* *retval);
+
+/** Deallocated SBlastHitsParameters.
+ * @param param object to be freed.
+ * @return NULL pointer.
+ */
+NCBI_XBLAST_EXPORT
+SBlastHitsParameters* SBlastHitsParametersFree(SBlastHitsParameters* param);
+                   
+
+
+
 /** One sequence segment within an HSP */
 typedef struct BlastSeg {
    Int2 frame;  /**< Translation frame */
@@ -70,6 +103,9 @@ typedef struct BlastHSP {
                               statistics evaluation? If unset (0), this HSP is
                               not part of a linked set, i.e. value 0 is treated
                               the same way as 1. */
+   Int2		comp_adjustment_method;  /**< which mode of composition
+                                              adjustment was used; relevant
+                                              only for blastp and tblastn */
    SPHIHspInfo* pat_info; /**< In PHI BLAST, information about this pattern
                                  match. */
 } BlastHSP;
@@ -256,6 +292,13 @@ Blast_HSPTestIdentityAndLength(EBlastProgramType program_number,
                                const BlastScoringOptions* score_options,
                                const BlastHitSavingOptions* hit_options);
 
+/** Calculated the number of HSPs that should be saved.
+ * @param gapped_calculation ungapped if false [in]
+ * @param options HitSavingoptions object [in]
+ * @return number of HSPs to save. 
+ */
+Int4 BlastHspNumMax(Boolean gapped_calculation, const BlastHitSavingOptions* options);
+
 /** Calculate length of an HSP as length in query plus length of gaps in 
  * query. If gap information is unavailable, return maximum between length in
  * query and in subject.
@@ -404,6 +447,20 @@ NCBI_XBLAST_EXPORT
 Int2
 Blast_HSPListPurgeNullHSPs(BlastHSPList* hsp_list);
 
+/** Check for an overlap of two different alignments and remove redundant HSPs.
+ * A sufficient overlap is when two alignments have the same start or end values
+ * If an overlap is found the HSP with the lowest score is removed, if both scores
+ * are the same then the first is removed.
+ * @param program Type of BLAST program. For some programs (PHI BLAST), the
+ *                purge should not be performed. [in]
+ * @param hsp_list Contains array of pointers to HSPs to purge [in]
+ * @return The number of valid alignments remaining. 
+*/
+NCBI_XBLAST_EXPORT
+Int4
+Blast_HSPListPurgeHSPsWithCommonEndpoints(EBlastProgramType program, 
+                                          BlastHSPList* hsp_list);
+
 /** Reevaluate all HSPs in an HSP list, using ambiguity information. 
  * This is/can only done either for an ungapped search, or if traceback is 
  * already available.
@@ -436,16 +493,15 @@ Blast_HSPListReevaluateWithAmbiguities(EBlastProgramType program,
 
 /** Append one HSP list to the other. Discard lower scoring HSPs if there is
  * not enough space to keep all.
- * @param hsp_list New list of HSPs [in]
+ * @param old_hsp_list_ptr list of HSPs, will be NULLed out on return [in|out]
  * @param combined_hsp_list_ptr Pointer to the combined list of HSPs, possibly
  *                              containing previously saved HSPs [in] [out]
  * @param hsp_num_max Maximal allowed number of HSPs to save (unlimited if INT4_MAX) [in]
  * @return Status: 0 on success, -1 on failure.
  */ 
 NCBI_XBLAST_EXPORT
-Int2 Blast_HSPListAppend(BlastHSPList* hsp_list, 
-                         BlastHSPList** combined_hsp_list_ptr, 
-                         Int4 hsp_num_max);
+Int2 Blast_HSPListAppend(BlastHSPList** old_hsp_list_ptr,
+        BlastHSPList** combined_hsp_list_ptr, Int4 hsp_num_max);
 
 /** Merge an HSP list from a chunk of the subject sequence into a previously
  * computed HSP list.
@@ -457,7 +513,7 @@ Int2 Blast_HSPListAppend(BlastHSPList* hsp_list,
  * @return 0 if HSP lists have been merged successfully, -1 otherwise.
  */
 NCBI_XBLAST_EXPORT
-Int2 Blast_HSPListsMerge(BlastHSPList* hsp_list, 
+Int2 Blast_HSPListsMerge(BlastHSPList** hsp_list, 
                    BlastHSPList** combined_hsp_list_ptr, 
                    Int4 hsp_num_max, Int4 start, Boolean merge_hsps);
 
@@ -470,6 +526,17 @@ Int2 Blast_HSPListsMerge(BlastHSPList* hsp_list,
 NCBI_XBLAST_EXPORT
 void Blast_HSPListAdjustOffsets(BlastHSPList* hsp_list, Int4 offset);
 
+/** For nucleotide BLAST, if the match reward score is equal to 2, 
+ * random alignments are dominated by runs of exact matches, which all have even
+ * scores. This makes it impossible to estimate statistical parameters correctly
+ * for odd scores. Hence the raw score formula is adjusted - all scores are
+ * rounded down to the nearest even value in order to provide a conservative estimate.
+ * @param hsp_list HSP list structure to adjust scores for. [in] [out]
+ * @param gapped_calculation not an ungapped alignment [in]
+ * @param sbp used for round_down Boolean
+ */
+void Blast_HSPListAdjustOddBlastnScores(BlastHSPList* hsp_list, Boolean gapped_calculation, BlastScoreBlk* sbp);
+
 /** Check if HSP list is sorted by score.
  * @param hsp_list The list to check [in]
  * @return TRUE if sorted, FALSE if not.
@@ -587,11 +654,11 @@ Int2 Blast_HSPResultsPerformCulling(BlastHSPResults *results,
  * @param hsp_list The results for the current subject sequence; in case of 
  *                 multiple queries, offsets are still in the concatenated 
  *                 sequence coordinates [in]
- * @param hit_options The options related to saving hits [in]
+ * @param blasthit_params The parameters related to saving hits [in]
  */
 NCBI_XBLAST_EXPORT
 Int2 Blast_HSPResultsSaveRPSHSPList(EBlastProgramType program, BlastHSPResults* results, 
-        BlastHSPList* hsp_list, const BlastHitSavingOptions* hit_options);
+        BlastHSPList* hsp_list, const SBlastHitsParameters* blasthit_params);
 
 /** Blast_HSPResultsSaveHSPList
  *  Save the current HSP list to appropriate places in the results structure.
@@ -603,11 +670,11 @@ Int2 Blast_HSPResultsSaveRPSHSPList(EBlastProgramType program, BlastHSPResults*
  * @param hsp_list The results for the current subject sequence; in case of 
  *                 multiple queries, offsets are still in the concatenated 
  *                 sequence coordinates [in]
- * @param hit_options The options related to saving hits [in]
+ * @param blasthit_params The parameters related to saving hits [in]
  */
 NCBI_XBLAST_EXPORT
 Int2 Blast_HSPResultsSaveHSPList(EBlastProgramType program, BlastHSPResults* results, 
-        BlastHSPList* hsp_list, const BlastHitSavingOptions* hit_options);
+        BlastHSPList* hsp_list, const SBlastHitsParameters* blasthit_params);
 
 /** Blast_HSPResultsSaveHSPList
  * Insert an HSP list to the appropriate place in the results structure.
@@ -621,6 +688,39 @@ NCBI_XBLAST_EXPORT
 Int2 Blast_HSPResultsInsertHSPList(BlastHSPResults* results, 
         BlastHSPList* hsp_list, Int4 hitlist_size);
 
+/* Forward declaration */
+struct BlastHSPStream;
+
+BlastHSPResults*
+Blast_HSPResultsFromHSPStream(struct BlastHSPStream* hsp_stream, 
+                              size_t num_queries, 
+                              const BlastHitSavingOptions* hit_options, 
+                              const BlastExtensionOptions* ext_options, 
+                              const BlastScoringOptions* scoring_options);
+
+BlastHSPResults*
+Blast_HSPResultsFromHSPStreamWithLimit(struct BlastHSPStream* hsp_stream, 
+                                   Uint4 num_queries, 
+                                   const BlastHitSavingOptions* hit_options, 
+                                   const BlastExtensionOptions* ext_options, 
+                                   const BlastScoringOptions* scoring_options,
+                                   Uint4 max_num_hsps,
+                                   Boolean* removed_hsps);
+
+/** Splits the BlastHSPResults structure for a PHI BLAST search into an array of
+ * BlastHSPResults structures, corresponding to different pattern occurrences in
+ * query. All HSPs are copied, so it is safe to free the returned 
+ * BlastHSPResults structures independently of the input results structure.
+ * @param results All results from a PHI BLAST search, with HSPs for 
+ *                different query pattern occurrences mixed together. [in]
+ * @param pattern_info Information about pattern occurrences in query. [in]
+ * @return Array of pointers to BlastHSPResults structures, corresponding to 
+ *         different pattern occurrences.
+ */
+BlastHSPResults** 
+PHIBlast_HSPResultsSplit(const BlastHSPResults* results, 
+                         const SPHIQueryInfo* pattern_info);
+
 #ifdef __cplusplus
 }
 #endif
