@@ -1,4 +1,4 @@
-/* $Id: blast_options.c,v 1.163 2005/04/27 19:53:45 dondosha Exp $
+/* $Id: blast_options.c,v 1.183 2006/04/25 16:06:53 camacho Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -34,13 +34,15 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_options.c,v 1.163 2005/04/27 19:53:45 dondosha Exp $";
+    "$Id: blast_options.c,v 1.183 2006/04/25 16:06:53 camacho Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_options.h>
 #include <algo/blast/core/blast_filter.h>
 
-const int kUngappedHSPNumMax = 400;
+const int kUngappedHSPNumMax = 400;  /**< Suggested max. number of HSPs for an ungapped search. */
+
+const double kPSSM_NoImpalaScaling = 1.0;
 
 SDustOptions* SDustOptionsFree(SDustOptions* dust_options)
 {
@@ -172,7 +174,8 @@ Int2 SBlastFilterOptionsValidate(EBlastProgramType program_number, const SBlastF
 
        if (filter_options == NULL)
        {
-           Blast_MessageWrite(blast_message, BLAST_SEV_WARNING, 2, 1, "SBlastFilterOptionsValidate: NULL filter_options");
+           Blast_MessageWrite(blast_message, eBlastSevWarning, kBlastMessageNoContext, 
+              "SBlastFilterOptionsValidate: NULL filter_options");
            return 1;
        }
 
@@ -181,14 +184,15 @@ Int2 SBlastFilterOptionsValidate(EBlastProgramType program_number, const SBlastF
            if (program_number != eBlastTypeBlastn)
            {
                if (blast_message)
-                  Blast_MessageWrite(blast_message, BLAST_SEV_WARNING, 2, 1, 
+                  Blast_MessageWrite(blast_message, eBlastSevError, kBlastMessageNoContext,
                    "SBlastFilterOptionsValidate: Repeat filtering only supported with blastn");
                return 1;
            }
-           if (filter_options->repeatFilterOptions->database == NULL)
+           if (filter_options->repeatFilterOptions->database == NULL ||
+               strlen(filter_options->repeatFilterOptions->database) == 0)
            {
                if (blast_message)
-                  Blast_MessageWrite(blast_message, BLAST_SEV_WARNING, 2, 1, 
+                  Blast_MessageWrite(blast_message, eBlastSevError, kBlastMessageNoContext,
                    "SBlastFilterOptionsValidate: No repeat database specified for repeat filtering");
                return 1;
            }
@@ -199,7 +203,7 @@ Int2 SBlastFilterOptionsValidate(EBlastProgramType program_number, const SBlastF
            if (program_number != eBlastTypeBlastn)
            {
                if (blast_message)
-                  Blast_MessageWrite(blast_message, BLAST_SEV_WARNING, 2, 1, 
+                  Blast_MessageWrite(blast_message, eBlastSevError, kBlastMessageNoContext,
                    "SBlastFilterOptionsValidate: Dust filtering only supported with blastn");
                return 1;
            }
@@ -210,7 +214,7 @@ Int2 SBlastFilterOptionsValidate(EBlastProgramType program_number, const SBlastF
            if (program_number == eBlastTypeBlastn)
            {
                if (blast_message)
-                  Blast_MessageWrite(blast_message, BLAST_SEV_WARNING, 2, 1, 
+                  Blast_MessageWrite(blast_message, eBlastSevError, kBlastMessageNoContext,
                    "SBlastFilterOptionsValidate: SEG filtering is not supported with blastn");
                return 1;
            }
@@ -248,6 +252,7 @@ BlastQuerySetUpOptionsNew(QuerySetUpOptions* *options)
 
    (*options)->genetic_code = BLAST_GENETIC_CODE;
 
+   /** @todo the code below should be deprecated */
    status = SBlastFilterOptionsNew(&((*options)->filtering_options), eEmpty);
    
    return status;
@@ -312,6 +317,7 @@ BlastInitialWordOptionsNew(EBlastProgramType program,
    /* Except in one special case of greedy gapped extension, we always do 
       ungapped extension. Let the special case unset this option. */
    (*options)->ungapped_extension = TRUE;
+   (*options)->program_number = program;
 
    return 0;
 }
@@ -322,20 +328,18 @@ BlastInitialWordOptionsValidate(EBlastProgramType program_number,
    const BlastInitialWordOptions* options, 
    Blast_Message* *blast_msg)
 {
-   Int4 code=2;
-   Int4 subcode=1;
 
    ASSERT(options);
 
    /* For some blastn variants (i.e., megablast), and for PHI BLAST there is no
     * ungapped extension. */
    if (program_number != eBlastTypeBlastn  &&
-       program_number != eBlastTypePhiBlastn  &&
-       program_number != eBlastTypePhiBlastp && options->x_dropoff <= 0.0)
+       (!Blast_ProgramIsPhiBlast(program_number)) &&
+       options->x_dropoff <= 0.0)
    {
-      Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode,
+      Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                             "x_dropoff must be greater than zero");
-         return (Int2) code;
+         return (Int2) -1;
    }
    
    return 0;
@@ -397,12 +401,21 @@ BlastExtensionOptionsNew(EBlastProgramType program, BlastExtensionOptions* *opti
     (*options)->ePrelimGapExt = eDynProgExt;
     (*options)->eTbackExt = eDynProgTbck;
 
+    /** @todo how to determine this for PSI-BLAST bootstrap run (i.e. when
+     * program is blastp? */
+    if (program == eBlastTypePsiBlast) {
+        (*options)->compositionBasedStats = TRUE;
+    }
+
+    (*options)->program_number = program;
+
 	return 0;
 }
 
 Int2
 BLAST_FillExtensionOptions(BlastExtensionOptions* options, 
-   EBlastProgramType program, Int4 greedy, double x_dropoff, double x_dropoff_final)
+   EBlastProgramType program, Int4 greedy, double x_dropoff, 
+   double x_dropoff_final)
 {
    if (!options)
       return 1;
@@ -429,6 +442,10 @@ BLAST_FillExtensionOptions(BlastExtensionOptions* options,
       }
    }
 
+   if (program == eBlastTypePsiBlast) {
+       options->compositionBasedStats = TRUE;
+   }
+
    if (x_dropoff)
       options->gap_x_dropoff = x_dropoff;
    if (x_dropoff_final) {
@@ -457,11 +474,9 @@ BlastExtensionOptionsValidate(EBlastProgramType program_number,
           options->ePrelimGapExt == eGreedyExt ||
           options->eTbackExt == eGreedyTbck)
 		{
-			Int4 code=2;
-			Int4 subcode=1;
-			Blast_MessageWrite(blast_msg, BLAST_SEV_WARNING, code, subcode, 
+			Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
                             "Greedy extension only supported for BLASTN");
-			return (Int2) code;
+			return (Int2) 2;
 		}
 	}
 
@@ -506,7 +521,10 @@ BlastScoringOptionsNew(EBlastProgramType program_number, BlastScoringOptions* *o
       (*options)->gap_extend = BLAST_GAP_EXTN_NUCL;
    }
    (*options)->decline_align = INT2_MAX;
-   (*options)->gapped_calculation = TRUE;
+   if (program_number != eBlastTypeTblastx) {
+       (*options)->gapped_calculation = TRUE;
+   }
+   (*options)->program_number = program_number;
    
    return 0;
 }
@@ -539,11 +557,13 @@ BLAST_FillScoringOptions(BlastScoringOptions* options,
          options->gap_extend = BLAST_GAP_EXTN_NUCL;
       }
    }
-   if (gap_open)
+   if (gap_open >= 0)
       options->gap_open = gap_open;
-   if (gap_extend)
+   if (gap_extend >= 0)
       options->gap_extend = gap_extend;
 
+   options->program_number = program_number;
+
    return 0;
 }
 
@@ -558,11 +578,9 @@ BlastScoringOptionsValidate(EBlastProgramType program_number,
    if (program_number == eBlastTypeTblastx && 
               options->gapped_calculation)
    {
-		Int4 code=2;
-		Int4 subcode=1;
-      Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+      Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
          "Gapped search is not allowed for tblastx");
-		return (Int2) code;
+		return (Int2) 2;
    }
 
 	if (program_number == eBlastTypeBlastn ||
@@ -570,69 +588,57 @@ BlastScoringOptionsValidate(EBlastProgramType program_number,
 	{
 		if (options->penalty >= 0)
 		{
-			Int4 code=2;
-			Int4 subcode=1;
-			Blast_MessageWrite(blast_msg, BLAST_SEV_WARNING, code, subcode, 
+			Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
                             "BLASTN penalty must be negative");
-			return (Int2) code;
+			return (Int2) 2;
 		}
-                if (options->gap_open > 0 && options->gap_extend == 0) 
+                if (options->gapped_calculation && options->gap_open > 0 && options->gap_extend == 0) 
                 {
-                        Int4 code=2;
-                        Int4 subcode=1;
-                        Blast_MessageWrite(blast_msg, BLAST_SEV_WARNING, 
-                           code, subcode, 
+                        Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
                            "BLASTN gap extension penalty cannot be 0");
-                        return (Int2) code;
+                        return (Int2) 2;
                 }
 	}
 	else
 	{
-		Int2 status=0;
-
-		if ((status=Blast_KarlinBlkGappedLoadFromTables(NULL, options->gap_open, 
-                     options->gap_extend, options->decline_align, 
+                if (options->gapped_calculation && !Blast_ProgramIsRpsBlast(program_number))
+                {
+                    Int2 status=0;
+                    if ((status=Blast_KarlinBlkGappedLoadFromTables(NULL, options->gap_open,
+                     options->gap_extend, options->decline_align,
                      options->matrix)) != 0)
-		{
+                     {
 			if (status == 1)
 			{
 				char* buffer;
-				Int4 code=2;
-				Int4 subcode=1;
 
 				buffer = BLAST_PrintMatrixMessage(options->matrix); 
-            Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR,
-                               code, subcode, buffer);
+                                Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext, buffer);
 				sfree(buffer);
-				return (Int2) code;
+				return (Int2) 2;
 				
 			}
 			else if (status == 2)
 			{
 				char* buffer;
-				Int4 code=2;
-				Int4 subcode=1;
 
 				buffer = BLAST_PrintAllowedValues(options->matrix, 
                         options->gap_open, options->gap_extend, 
                         options->decline_align); 
-            Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
-                               buffer);
+                                Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext, buffer);
 				sfree(buffer);
-				return (Int2) code;
+				return (Int2) 2;
 			}
-		}
-		
+                    }
+	       }
 	}
 
 	if (program_number != eBlastTypeBlastx && 
        program_number != eBlastTypeTblastn && options->is_ooframe)
 	{
-      Int4 code=2;
-      Int4 subcode=1;
-      Blast_MessageWrite(blast_msg, BLAST_SEV_WARNING, code, subcode, 
-         "Out-of-frame only permitted for blastx and tblastn");
-      return (Int2) code;
+          Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
+             "Out-of-frame only permitted for blastx and tblastn");
+          return (Int2) 2;
 	}
 
 	return 0;
@@ -668,7 +674,7 @@ Int2 BlastScoringOptionsSetMatrix(BlastScoringOptions* opts,
         opts->matrix = strdup(matrix_name);
         /* Make it all upper case */
         for (i=0; i<strlen(opts->matrix); ++i)
-            opts->matrix[i] = toupper(opts->matrix[i]);
+            opts->matrix[i] = toupper((unsigned char) opts->matrix[i]);
     }
     return 0;
 }
@@ -720,6 +726,10 @@ LookupTableOptions*
 LookupTableOptionsFree(LookupTableOptions* options)
 
 {
+
+      if (options == NULL)
+          return NULL;
+
       sfree(options->phi_pattern);
    
 	sfree(options);
@@ -772,15 +782,16 @@ LookupTableOptionsNew(EBlastProgramType program_number, LookupTableOptions* *opt
            (*options)->threshold = BLAST_WORD_THRESHOLD_TBLASTX;
        break;
    }
-   
+
+   (*options)->program_number = program_number;
+
    return 0;
 }
 
 Int2 
 BLAST_FillLookupTableOptions(LookupTableOptions* options, 
-   EBlastProgramType program_number, Boolean is_megablast, Int4 threshold,
-   Int4 word_size, Boolean variable_wordsize,
-   Boolean use_pssm)
+   EBlastProgramType program_number, Boolean is_megablast, 
+   Int4 threshold, Int4 word_size)
 {
    if (!options)
       return 1;
@@ -802,25 +813,77 @@ BLAST_FillLookupTableOptions(LookupTableOptions* options,
    if (threshold == -1)
       options->threshold = 0;
 
-   /* if the supplied threshold is > 0, use it */
+   /* if the supplied threshold is > 0, use it otherwise, use the default */
    if (threshold > 0)
       options->threshold = threshold;
 
-   /* otherwise, use the default */
-
-   if (use_pssm)
-      options->use_pssm = use_pssm;
-   if (program_number == eBlastTypeRpsBlast ||
-       program_number == eBlastTypeRpsTblastn)
+   if (Blast_ProgramIsRpsBlast(program_number))
       options->lut_type = RPS_LOOKUP_TABLE;
    if (word_size)
       options->word_size = word_size;
-   if (program_number == eBlastTypeBlastn) {
-      options->variable_wordsize = variable_wordsize;
-   }
    return 0;
 }
 
+Int2 BLAST_GetSuggestedThreshold(EBlastProgramType program_number, const char* matrixName, Int4* threshold)
+{
+
+    const Int4 kB62_threshold = 11;
+
+    if (program_number == eBlastTypeBlastn)
+      return 0;
+
+    if (matrixName == NULL)
+      return -1;
+
+    if(strcasecmp(matrixName, "BLOSUM62") == 0)
+        *threshold = kB62_threshold;
+    else if(strcasecmp(matrixName, "BLOSUM45") == 0)
+        *threshold = 14;
+    else if(strcasecmp(matrixName, "BLOSUM62_20") == 0)
+        *threshold = 100;
+    else if(strcasecmp(matrixName, "BLOSUM80") == 0)
+        *threshold = 12;
+    else if(strcasecmp(matrixName, "PAM30") == 0)
+        *threshold = 16;
+    else if(strcasecmp(matrixName, "PAM70") == 0)
+        *threshold = 14;
+    else
+        *threshold = kB62_threshold;
+
+    if (Blast_SubjectIsTranslated(program_number) == TRUE)
+        *threshold += 2;  /* Covers tblastn, tblastx, psi-tblastn rpstblastn. */
+    else if (Blast_QueryIsTranslated(program_number) == TRUE)
+        *threshold += 1;
+
+    return 0;
+}
+
+Int2 BLAST_GetSuggestedWindowSize(EBlastProgramType program_number, const char* matrixName, Int4* window_size)
+{
+    const Int4 kB62_windowsize = 40;
+
+    if (program_number == eBlastTypeBlastn)
+      return 0;
+
+    if (matrixName == NULL)
+      return -1;
+
+    if(strcasecmp(matrixName, "BLOSUM62") == 0)
+        *window_size = kB62_windowsize;
+    else if(strcasecmp(matrixName, "BLOSUM45") == 0)
+        *window_size = 60;
+    else if(strcasecmp(matrixName, "BLOSUM80") == 0)
+        *window_size = 25;
+    else if(strcasecmp(matrixName, "PAM30") == 0)
+        *window_size = 15;
+    else if(strcasecmp(matrixName, "PAM70") == 0)
+        *window_size = 20;
+    else
+        *window_size = kB62_windowsize;
+
+    return 0;
+}
+
 /** Validate options for the discontiguous word megablast
  * Word size must be 11 or 12; template length 16, 18 or 21; 
  * template type 0, 1 or 2.
@@ -852,18 +915,15 @@ LookupTableOptionsValidate(EBlastProgramType program_number,
    const LookupTableOptions* options, Blast_Message* *blast_msg)
 
 {
-   Int4 code=2;
-   Int4 subcode=1;
-   const Boolean kPhiBlast = (program_number == eBlastTypePhiBlastn || 
-                              program_number == eBlastTypePhiBlastp);
+   const Boolean kPhiBlast = Blast_ProgramIsPhiBlast(program_number);
 
 	if (options == NULL)
 		return 1;
 
     if (options->phi_pattern && !kPhiBlast) {
-        Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+        Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
             "PHI pattern can be specified only for blastp and blastn");
-        return (Int2) code;
+        return (Int2) 2;
     }
 
     /* For PHI BLAST, the subsequent word size tests are not needed. */
@@ -871,74 +931,52 @@ LookupTableOptionsValidate(EBlastProgramType program_number,
         return 0;
 
 	if (program_number != eBlastTypeBlastn && 
-            program_number != eBlastTypeRpsBlast &&
-            program_number != eBlastTypeRpsTblastn &&
-            options->threshold <= 0)
+        (!Blast_ProgramIsRpsBlast(program_number)) &&
+        options->threshold <= 0)
 	{
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                          "Non-zero threshold required");
-		return (Int2) code;
+		return (Int2) 2;
 	}
 
 	if (options->word_size <= 0)
 	{
-                if (program_number != eBlastTypeRpsBlast &&
-                    program_number != eBlastTypeRpsTblastn) {
-        		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, 
-                                           code, subcode, 
-                                         "Word-size must be greater than zero");
-		        return (Int2) code;
-                }
+        if ( !Blast_ProgramIsRpsBlast(program_number)) {
+            Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
+                                     "Word-size must be greater than zero");
+            return (Int2) 2;
+        }
 	} else if (program_number == eBlastTypeBlastn && options->word_size < 4)
 	{
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
-                         "Word-size must be 4" 
-                         "or greater for nucleotide comparison");
-		return (Int2) code;
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext, 
+                  "Word-size must be 4 or greater for nucleotide comparison");
+		return (Int2) 2;
 	} else if (program_number != eBlastTypeBlastn && options->word_size > 5)
 	{
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                          "Word-size must be less"
                          "than 6 for protein comparison");
-		return (Int2) code;
+		return (Int2) 2;
 	}
 
-
-        /* FIXME: is this really needed?? */
-        if (options->variable_wordsize && 
-          ((options->word_size % 4) != 0) ) {
-         Blast_MessageWrite(blast_msg, BLAST_SEV_WARNING, code, subcode, 
-                            "Word size must be divisible by 4 if only full "
-                            "bytes of subject sequences are matched to query");
-         return (Int2) code;
-      }
-
 	if (program_number != eBlastTypeBlastn && 
        options->lut_type == MB_LOOKUP_TABLE)
 	{
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                          "Megablast lookup table only supported with blastn");
-		return (Int2) code;
+		return (Int2) 2;
 	}
 
-   if (options->lut_type == MB_LOOKUP_TABLE && options->word_size < 12 && 
-       options->mb_template_length == 0) {
-      Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
-                         "Word size must be 12 or greater with megablast"
-                         " lookup table");
-      return (Int2) code;
-   }
-
    if (program_number == eBlastTypeBlastn && options->mb_template_length > 0) {
       if (!s_DiscWordOptionsValidate(options->word_size,
               options->mb_template_length, options->mb_template_type)) {
-         Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+         Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                             "Invalid discontiguous template parameters");
-         return (Int2) code;
+         return (Int2) 2;
       } else if (options->lut_type != MB_LOOKUP_TABLE) {
-         Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+         Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
             "Invalid lookup table type for discontiguous Mega BLAST");
-         return (Int2) code;
+         return (Int2) 2;
       } 
    }
 	return 0;
@@ -962,8 +1000,8 @@ Int2 BlastHitSavingOptionsNew(EBlastProgramType program_number,
       return 1;
 
    (*options)->hitlist_size = BLAST_HITLIST_SIZE;
-   (*options)->prelim_hitlist_size = BLAST_PRELIM_HITLIST_SIZE;
    (*options)->expect_value = BLAST_EXPECT_VALUE;
+   (*options)->program_number = program_number;
 
    return 0;
 
@@ -972,15 +1010,18 @@ Int2 BlastHitSavingOptionsNew(EBlastProgramType program_number,
 Int2
 BLAST_FillHitSavingOptions(BlastHitSavingOptions* options, 
                            double evalue, Int4 hitlist_size,
-                           Boolean is_gapped, Int4 culling_limit)
+                           Boolean is_gapped, Int4 culling_limit,
+                           Int4 min_diag_separation)
 {
    if (!options)
       return 1;
 
    if (hitlist_size)
-      options->hitlist_size = options->prelim_hitlist_size = hitlist_size;
+      options->hitlist_size = hitlist_size;
    if (evalue)
       options->expect_value = evalue;
+   if (min_diag_separation)
+      options->min_diag_separation = min_diag_separation;
    if(!is_gapped)
      options->hsp_num_max = kUngappedHSPNumMax;
    options->culling_limit = culling_limit;
@@ -996,41 +1037,33 @@ BlastHitSavingOptionsValidate(EBlastProgramType program_number,
 	if (options == NULL)
 		return 1;
 
-	if (options->hitlist_size < 1 || options->prelim_hitlist_size < 1)
+	if (options->hitlist_size < 1)
 	{
-		Int4 code=1;
-		Int4 subcode=1;
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                          "No hits are being saved");
-		return (Int2) code;
+		return (Int2) 2;
 	}
 
 	if (options->expect_value <= 0.0 && options->cutoff_score <= 0)
 	{
-		Int4 code=2;
-		Int4 subcode=1;
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
          "expect value or cutoff score must be greater than zero");
-		return (Int2) code;
+		return (Int2) 2;
 	}	
 
    if (options->longest_intron != 0 && 
        program_number != eBlastTypeTblastn && 
        program_number != eBlastTypeBlastx) {
-		Int4 code=2;
-		Int4 subcode=1;
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
          "Uneven gap linking of HSPs is allowed for blastx and tblastn only");
-		return (Int2) code;
+		return (Int2) 2;
    }
 
 	if (options->culling_limit < 0)
 	{
-		Int4 code=2;
-		Int4 subcode=1;
-		Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, code, subcode, 
+		Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                     "culling limit must be greater than or equal to zero");
-		return (Int2) code;
+		return (Int2) 2;
 	}	
 
 	return 0;
@@ -1053,7 +1086,7 @@ Int2 PSIBlastOptionsNew(PSIBlastOptions** psi_options)
    options->use_best_alignment = TRUE;
 
    options->nsg_compatibility_mode = FALSE;
-   options->impala_scaling_factor = 1.0;    /* 1.0 indicates not to use this */
+   options->impala_scaling_factor = kPSSM_NoImpalaScaling;
    
    return 0;
 }
@@ -1068,13 +1101,13 @@ Int2 PSIBlastOptionsValidate(const PSIBlastOptions* psi_options,
     }
 
     if (psi_options->pseudo_count <= 0) {
-        Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, 0, 0,
+        Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                            "Pseudo count must be greater than 0");
         return retval;
     }
 
     if (psi_options->inclusion_ethresh <= 0.0) {
-        Blast_MessageWrite(blast_msg, BLAST_SEV_ERROR, 0, 0,
+        Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
                            "Inclusion threshold must be greater than 0");
         return retval;
     }
@@ -1111,6 +1144,10 @@ Int2 BlastDatabaseOptionsNew(BlastDatabaseOptions** db_options)
 BlastDatabaseOptions* 
 BlastDatabaseOptionsFree(BlastDatabaseOptions* db_options)
 {
+
+   if (db_options == NULL)
+      return NULL;
+
    sfree(db_options->gen_code_string);
    sfree(db_options);
    return NULL;
@@ -1160,6 +1197,40 @@ Int2 BLAST_InitDefaultOptions(EBlastProgramType program_number,
 
 }
 
+/**  Checks that the extension and scoring options are consistent with each other
+ * @param program_number identifies the program [in]
+ * @param ext_options the extension options [in]
+ * @param score_options the scoring options [in]
+ * @param blast_msg returns a message on errors. [in|out]
+ * @return zero on success, an error code otherwise. 
+ */
+static Int2 s_BlastExtensionScoringOptionsValidate(EBlastProgramType program_number,
+                           const BlastExtensionOptions* ext_options,
+                           const BlastScoringOptions* score_options, 
+                           Blast_Message* *blast_msg)
+{
+    if (ext_options == NULL || score_options == NULL)
+        return -1;
+
+    if (program_number == eBlastTypeBlastn)
+    {
+        if (score_options->gap_open == 0 && score_options->gap_extend == 0)
+        {
+	    if (ext_options->ePrelimGapExt != eGreedyWithTracebackExt && 
+                ext_options->ePrelimGapExt != eGreedyExt && 
+                ext_options->eTbackExt != eGreedyTbck)
+	    {
+			Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
+                            "Greedy extension must be used if gap existence and extension options are zero");
+			return (Int2) 2;
+	    }
+	}
+    }
+
+    return 0;
+}
+                   
+
 Int2 BLAST_ValidateOptions(EBlastProgramType program_number,
                            const BlastExtensionOptions* ext_options,
                            const BlastScoringOptions* score_options, 
@@ -1185,6 +1256,11 @@ Int2 BLAST_ValidateOptions(EBlastProgramType program_number,
    if ((status = BlastHitSavingOptionsValidate(program_number, hit_options,
                                                blast_msg)) != 0)
        return status;
+   if ((status = s_BlastExtensionScoringOptionsValidate(program_number, ext_options,
+                                               score_options, blast_msg)) != 0)
+       return status;
+
+   
 
    return status;
 }
@@ -1193,6 +1269,67 @@ Int2 BLAST_ValidateOptions(EBlastProgramType program_number,
  * ===========================================================================
  *
  * $Log: blast_options.c,v $
+ * Revision 1.183  2006/04/25 16:06:53  camacho
+ * tblastx scoring options must be set to false
+ *
+ * Revision 1.182  2006/04/20 19:28:30  madden
+ * Prototype change for Blast_MessageWrite
+ *
+ * Revision 1.181  2006/03/02 13:28:34  madden
+ * use BlastHspNumMax function, refactor how hsp_num_max is calculated
+ *
+ * Revision 1.180  2006/02/09 18:47:07  camacho
+ * Correct error reporting when there are errors in validation of filtering
+ * options.
+ *
+ * Revision 1.179  2006/01/23 16:29:51  papadopo
+ * allow the number of diagonals used in containment tests to be specified when initializing HitSavingOptions
+ *
+ * Revision 1.178  2005/12/22 14:07:48  papadopo
+ * remove variable wordsize, and change signature to BlastFillLookupTableOptions
+ *
+ * Revision 1.177  2005/12/19 16:11:12  papadopo
+ * no minimum value for megablast word size need be enforced; the engine will switch to a standard lookup table if the specified word size is too small
+ *
+ * Revision 1.176  2005/12/12 13:38:27  madden
+ * Add call to s_BlastExtensionScoringOptionsValidate to BLAST_ValidateOptions to check that scoring and extension options are consistent
+ *
+ * Revision 1.175  2005/11/16 14:27:03  madden
+ * Fix spelling in CRN
+ *
+ * Revision 1.174  2005/10/18 15:19:04  madden
+ * Exclude rpsblast from validation of gap parameters
+ *
+ * Revision 1.173  2005/10/17 14:03:34  madden
+ * Change convention for unset gap parameters from zero to negative number
+ *
+ * Revision 1.172  2005/08/29 13:51:44  madden
+ * Add functions BLAST_GetSuggestedThreshold and BLAST_GetSuggestedWindowSize
+ *
+ * Revision 1.171  2005/06/24 12:15:40  madden
+ * Add protection against NULL pointers in options free functons
+ *
+ * Revision 1.170  2005/06/20 13:09:36  madden
+ * Rename BlastSeverity enums in line with C++ tookit convention
+ *
+ * Revision 1.169  2005/06/03 16:22:22  lavr
+ * Explicit (unsigned char) casts in ctype routines
+ *
+ * Revision 1.168  2005/06/02 16:18:40  camacho
+ * Remove LookupTableOptions::use_pssm
+ *
+ * Revision 1.167  2005/05/20 18:26:54  camacho
+ * Deduce LookupTableOptions::use_pssm from program type
+ *
+ * Revision 1.166  2005/05/16 12:22:07  madden
+ * Removal of prelim_hitlist_size as an option
+ *
+ * Revision 1.165  2005/05/06 14:27:26  camacho
+ * + Blast_ProgramIs{Phi,Rps}Blast
+ *
+ * Revision 1.164  2005/05/02 19:38:47  camacho
+ * Introduced constant for IMPALA-style PSSM scaling
+ *
  * Revision 1.163  2005/04/27 19:53:45  dondosha
  * Added handling of PHI BLAST program enumeration values
  *
