@@ -1,4 +1,4 @@
-/*  $Id: blast_stat.h,v 1.67 2005/04/27 19:49:17 dondosha Exp $
+/*  $Id: blast_stat.h,v 1.78 2006/04/20 19:26:37 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #define __BLAST_STAT__
 
 #include <algo/blast/core/blast_def.h>
+#include <algo/blast/core/blast_query_info.h>
 #include <algo/blast/core/blast_message.h>
 
 #ifdef __cplusplus
@@ -113,6 +114,7 @@ typedef struct SPsiBlastScoreMatrix {
  * @return NULL in case of memory allocation failure, else new
  * SPsiBlastScoreMatrix structure
  */
+NCBI_XBLAST_EXPORT
 SPsiBlastScoreMatrix*
 SPsiBlastScoreMatrixNew(size_t ncols);
 
@@ -120,6 +122,7 @@ SPsiBlastScoreMatrixNew(size_t ncols);
  * @param matrix structure to deallocate [in]
  * @return NULL
  */
+NCBI_XBLAST_EXPORT
 SPsiBlastScoreMatrix*
 SPsiBlastScoreMatrixFree(SPsiBlastScoreMatrix* matrix);
 
@@ -160,6 +163,7 @@ protein alphabet (e.g., ncbistdaa etc.), FALSE for nt. alphabets. */
    Uint1*   ambiguous_res; /**< Array of ambiguous res. (e.g, 'X', 'N')*/
    Int2     ambig_size, /**< size of array above. FIXME: not needed here? */
          ambig_occupy;  /**< How many occupied? */
+   Boolean  round_down; /**< Score must be rounded down to nearest even score if odd. */
 } BlastScoreBlk;
 
 /** 
@@ -177,12 +181,14 @@ typedef struct Blast_ResFreq {
  * @param number_of_contexts how many strands or sequences [in]
  * @return BlastScoreBlk*
 */
+NCBI_XBLAST_EXPORT
 BlastScoreBlk* BlastScoreBlkNew (Uint1 alphabet, Int4 number_of_contexts);
 
 /** Deallocates BlastScoreBlk as well as all associated structures.
  * @param sbp BlastScoreBlk to be deallocated [in]
  * @return NULL pointer.
  */
+NCBI_XBLAST_EXPORT
 BlastScoreBlk* BlastScoreBlkFree (BlastScoreBlk* sbp);
 
 /** Set the ambiguous residue (e.g, 'N', 'X') in the BlastScoreBlk*.
@@ -192,23 +198,28 @@ BlastScoreBlk* BlastScoreBlkFree (BlastScoreBlk* sbp);
  * @param ambiguous_res the residue to be set on the BlastScoreBlk
  * @return zero on success, others on error
  */
+NCBI_XBLAST_EXPORT
 Int2 BLAST_ScoreSetAmbigRes (BlastScoreBlk* sbp, char ambiguous_res);
 
 
 /** Calculate and fill the ungapped Karlin-Altschul parameters in the
- * BlastScoreBlk structure.
+ * BlastScoreBlk structure (fields kbp_std, kbp_psi, and kbp of that structure).
  * @param program BLAST program type, needed to decide whether to substitute
  *                ideal values. [in]
  * @param sbp Scoring block to work with [in] [out]
  * @param query Buffer containing (concatenated) query sequence [in]
  * @param query_info Information about offsets of concatenated queries [in]
- * @return 0 if ungapped Karlin-Altschul parameters could be 
- *               calculated for at least one context; 1 otherwise.
+ * @param blast_message returns queries that could not be processed [out]
+ * @return 0 if ungapped Karlin-Altschul parameters could be calculated for 
+ *        all of the query sequence's contexts; 1 if any of the contexts 
+ *        failed (but all others will be populated).
  */
+NCBI_XBLAST_EXPORT
 Int2
 Blast_ScoreBlkKbpUngappedCalc(EBlastProgramType program, 
                               BlastScoreBlk* sbp, Uint1* query, 
-                              BlastQueryInfo* query_info);
+                              const BlastQueryInfo* query_info,
+                              Blast_Message* *blast_message);
 
 /** This function fills in the BlastScoreBlk structure.  
  * Tasks are:
@@ -217,11 +228,13 @@ Blast_ScoreBlkKbpUngappedCalc(EBlastProgramType program,
  * @param sbp Scoring block [in] [out]
  * @param matrix Full path to the matrix in the directory structure [in]
 */
+NCBI_XBLAST_EXPORT
 Int2 Blast_ScoreBlkMatrixFill (BlastScoreBlk* sbp, char* matrix);
  
 /** Callocs a Blast_KarlinBlk
  * @return pointer to the Blast_KarlinBlk
 */
+NCBI_XBLAST_EXPORT
 Blast_KarlinBlk* Blast_KarlinBlkNew (void);
 
 /** Copies contents of one Karlin block to another. Both must be allocated
@@ -230,6 +243,7 @@ Blast_KarlinBlk* Blast_KarlinBlkNew (void);
  * @param kbp_from Karlin block to copy values from [in]
  * @return 0 on success; -1 if either argument is NULL on input.
  */ 
+NCBI_XBLAST_EXPORT
 Int2 Blast_KarlinBlkCopy(Blast_KarlinBlk* kbp_to, Blast_KarlinBlk* kbp_from);
 
 
@@ -237,6 +251,7 @@ Int2 Blast_KarlinBlkCopy(Blast_KarlinBlk* kbp_to, Blast_KarlinBlk* kbp_from);
  * @param kbp KarlinBlk to be deallocated [in]
  * @return NULL
 */
+NCBI_XBLAST_EXPORT
 Blast_KarlinBlk* Blast_KarlinBlkFree(Blast_KarlinBlk* kbp);
 
 /** Fills in lambda, H, and K values, as calculated by Stephen Altschul 
@@ -249,10 +264,34 @@ Blast_KarlinBlk* Blast_KarlinBlkFree(Blast_KarlinBlk* kbp);
  * @param error_return filled in with error message if needed [out]
  * @return zero on success
  */
+NCBI_XBLAST_EXPORT
 Int2 Blast_KarlinBlkGappedCalc (Blast_KarlinBlk* kbp, Int4 gap_open, 
         Int4 gap_extend, Int4 decline_align, const char* matrix_name, 
         Blast_Message** error_return);
 
+/** Retrieves Karlin-Altschul parameters from precomputed tables, given the
+ * substitution and gap scores. Gap cost values greater than any of those 
+ * listed in the tables ("greater" meaning that both values are greater than or
+ * equal, and at least one is strictly greater), are treated as infinite, and 
+ * parameters values are copied from the ungapped Karlin block.
+ * @param kbp Allocated Karlin block to fill [in] [out]
+ * @param gap_open Gap openening (existence) cost [in]
+ * @param gap_extend Gap extension cost [in]
+ * @param reward Match reward score [in]
+ * @param penalty Mismatch penalty score [in]
+ * @param kbp_ungap Karlin block with ungapped Karlin-Altschul parameters [in]
+ * @param round_down specifies that the score should be rounded down to nearest even
+ *      score in some cases [in|out]
+ * @param error_return Pointer to error message. [in] [out]
+ */
+NCBI_XBLAST_EXPORT
+Int2
+Blast_KarlinBlkNuclGappedCalc(Blast_KarlinBlk* kbp, Int4 gap_open, 
+                              Int4 gap_extend, Int4 reward, Int4 penalty,
+                              Blast_KarlinBlk* kbp_ungap,
+                              Boolean* round_down,
+                              Blast_Message** error_return);
+
 
 /** Calculates the Karlin-Altschul parameters assuming standard residue
  * compositions for the query and subject sequences. It populates the kbp_ideal
@@ -262,6 +301,7 @@ Int2 Blast_KarlinBlkGappedCalc (Blast_KarlinBlk* kbp, Int4 gap_open,
  * @param sbp ScoreBlk used to calculate "ideal" values. [in|out]
  * @return 0 on success, 1 on failure
 */
+NCBI_XBLAST_EXPORT
 Int2 Blast_ScoreBlkKbpIdealCalc(BlastScoreBlk* sbp);
 
 /** Attempts to fill KarlinBlk for given gap opening, extensions etc.
@@ -275,12 +315,14 @@ Int2 Blast_ScoreBlkKbpIdealCalc(BlastScoreBlk* sbp);
  *          1 if matrix not found
  *           2 if matrix found, but open, extend etc. values not supported.
 */
+NCBI_XBLAST_EXPORT
 Int2 Blast_KarlinBlkGappedLoadFromTables(Blast_KarlinBlk* kbp, Int4 gap_open, Int4 gap_extend, Int4 decline_align, const char* matrix_name);
 
 /** Prints a messages about the allowed matrices, BlastKarlinBlkGappedFill should return 1 before this is called. 
  * @param matrix the matrix to print a message about [in]
  * @return the message
  */
+NCBI_XBLAST_EXPORT
 char* BLAST_PrintMatrixMessage(const char *matrix);
 
 /** Prints a messages about the allowed open etc values for the given matrix, 
@@ -291,9 +333,11 @@ char* BLAST_PrintMatrixMessage(const char *matrix);
  * @param decline_align cost of declining to align [in]
  * @return message
  */
+NCBI_XBLAST_EXPORT
 char* BLAST_PrintAllowedValues(const char *matrix, Int4 gap_open, Int4 gap_extend, Int4 decline_align);
 
 /** Calculates the parameter Lambda given an initial guess for its value */
+NCBI_XBLAST_EXPORT
 double
 Blast_KarlinLambdaNR(Blast_ScoreFreq* sfp, double initialLambdaGuess);
 
@@ -304,6 +348,7 @@ Blast_KarlinLambdaNR(Blast_ScoreFreq* sfp, double initialLambdaGuess);
  * @param searchsp total search space to be used [in]
  * @return the expect value
  */
+NCBI_XBLAST_EXPORT
 double BLAST_KarlinStoE_simple (Int4 S, Blast_KarlinBlk* kbp, Int8  searchsp);
 
 /** Compute a divisor used to weight the evalue of a collection of
@@ -322,6 +367,7 @@ double BLAST_KarlinStoE_simple (Int4 S, Blast_KarlinBlk* kbp, Int8  searchsp);
  * @param nsegs the number of HSPs in the sum group [in]
  * @return divisor used to compensate for multiple tests
  */
+NCBI_XBLAST_EXPORT
 double BLAST_GapDecayDivisor(double decayrate, unsigned nsegs );
 
 /** Calculate the cutoff score from the expected number of HSPs or vice versa.
@@ -332,6 +378,7 @@ double BLAST_GapDecayDivisor(double decayrate, unsigned nsegs );
  * @param dodecay Use gap decay feature? [in]
  * @param gap_decay_rate Gap decay rate to use, if dodecay is set [in]
  */
+NCBI_XBLAST_EXPORT
 Int2 BLAST_Cutoffs (Int4 *S, double* E, Blast_KarlinBlk* kbp, 
                     Int8 searchsp, Boolean dodecay, double gap_decay_rate);
 
@@ -350,6 +397,7 @@ Int2 BLAST_Cutoffs (Int4 *S, double* E, Blast_KarlinBlk* kbp,
  *    the calling routine [in]
  * @return the expect value 
  */
+NCBI_XBLAST_EXPORT
 double BLAST_SmallGapSumE (Int4 start_points, Int2 num,  double xsum,
                            Int4 query_length, Int4 subject_length,
                            Int8 searchsp_eff, double weight_divisor);
@@ -377,6 +425,7 @@ double BLAST_SmallGapSumE (Int4 start_points, Int2 num,  double xsum,
  *    the calling routine [in]
  * @return sum expect value.
  */
+NCBI_XBLAST_EXPORT
 double BLAST_UnevenGapSumE (Int4 query_start_points, Int4 subject_start_points,
                             Int2 num, double xsum,
                             Int4 query_length, Int4 subject_length,
@@ -395,10 +444,37 @@ double BLAST_UnevenGapSumE (Int4 query_start_points, Int4 subject_start_points,
  *    calling routine [in]
  * @return sum expect value.
  */
+NCBI_XBLAST_EXPORT
 double BLAST_LargeGapSumE (Int2 num,  double xsum,
                            Int4 query_length, Int4 subject_length,
                            Int8 searchsp_eff, double weight_divisor );
 
+/** Extract the recommended gap existence and extension values.
+ * Only to be used with protein matrices. 
+ * @param  matrixName name of the matrix [in]
+ * @param gap_existence returns recommended existence cost [in|out]
+ * @param gap_extension returns recommended extension cost [in|out]
+ * @return zero on success 
+ */
+NCBI_XBLAST_EXPORT
+Int2 BLAST_GetProteinGapExistenceExtendParams(const char* matrixName,
+                                       Int4* gap_existence,
+                                       Int4* gap_extension);
+
+/** Extract the recommended gap existence and extension values.
+ * Only to be used with blastn searches.
+ * @param reward match score [in]
+ * @param penalty mismatch score [in]
+ * @param gap_existence returns recommended existence cost [in|out]
+ * @param gap_extension returns recommended extension cost [in|out]
+ * @return zero on success 
+ */
+NCBI_XBLAST_EXPORT
+Int2 BLAST_GetNucleotideGapExistenceExtendParams(Int4 reward,
+                                       Int4 penalty,
+                                       Int4* gap_existence,
+                                       Int4* gap_extension);
+
 /** Extract the alpha and beta settings for this matrixName, and these
  *  gap open and gap extension costs
  * @param matrixName name of the matrix used [in]
@@ -407,10 +483,32 @@ double BLAST_LargeGapSumE (Int2 num,  double xsum,
  * @param gapped TRUE if a gapped search [in]
  * @param gap_open existence cost of a gap [in]
  * @param gap_extend extension cost of a gap [in]
+ * @param kbp_ungapped Karlin block with ungapped values of the parameters [in]
 */
+NCBI_XBLAST_EXPORT
 void BLAST_GetAlphaBeta (const char* matrixName, double *alpha,
-                    double *beta, Boolean gapped, Int4 gap_open, Int4 gap_extend);
+                         double *beta, Boolean gapped, Int4 gap_open, 
+                         Int4 gap_extend, const Blast_KarlinBlk* kbp_ungapped);
 
+/** Extract the alpha and beta settings for these substitution and gap scores. 
+ * If substitution or gap costs are not found in the tables, assume an ungapped
+ * search. Then alpha is computed using the formula Alpha = Lambda/H, and beta
+ * is equal to 0 except for some special cases.
+ * @param reward Match reward score [in]
+ * @param penalty Mismatch penalty score [in]
+ * @param gap_open Gap opening (existence) cost [in]
+ * @param gap_extend Gap extension cost [in]
+ * @param kbp Karlin block containing already computed Lambda, K and H 
+ *            parameters.
+ * @param gapped_calculation Is this a gapped search? [in]
+ * @param alpha Alpha parameter for this scoring system [out]
+ * @param beta Beta parameter for this scoring system [out]
+ */
+NCBI_XBLAST_EXPORT
+Int2 Blast_GetNuclAlphaBeta(Int4 reward, Int4 penalty, Int4 gap_open, 
+                            Int4 gap_extend, Blast_KarlinBlk* kbp,
+                            Boolean gapped_calculation,
+                            double *alpha, double *beta);
 
 /** Rescale the PSSM, using composition-based statistics, for use
  *  with RPS BLAST. This function produces a PSSM for a single RPS DB
@@ -428,6 +526,7 @@ void BLAST_GetAlphaBeta (const char* matrixName, double *alpha,
  * @param matrix_name Name of the score matrix underlying the RPS search [in]
  * @return rescaled pssm 
  */
+NCBI_XBLAST_EXPORT
 Int4 ** RPSRescalePssm(double scalingFactor, Int4 rps_query_length, 
                    const Uint1 * rps_query_seq, Int4 db_seq_length, 
                    Int4 **posMatrix, const char *matrix_name);
@@ -472,6 +571,7 @@ Int4 ** RPSRescalePssm(double scalingFactor, Int4 rps_query_length,
  * @return   0 if length_adjustment is known to be the largest integer less
  *           than the fixed point of f(ell); 1 otherwise.
  */
+NCBI_XBLAST_EXPORT
 Int4
 BLAST_ComputeLengthAdjustment(double K,
                               double logK,
@@ -487,11 +587,13 @@ BLAST_ComputeLengthAdjustment(double K,
     based upon the contents of sbp.
  * @param sbp The BlastScoreBlk* used to init prob [in]
 */
+NCBI_XBLAST_EXPORT
 Blast_ResFreq* Blast_ResFreqNew(const BlastScoreBlk* sbp);
 
 /** Deallocates Blast_ResFreq and prob0 element.
  * @param rfp the Blast_ResFreq to be deallocated.
 */
+NCBI_XBLAST_EXPORT
 Blast_ResFreq* Blast_ResFreqFree(Blast_ResFreq* rfp);
 
 
@@ -500,6 +602,7 @@ Blast_ResFreq* Blast_ResFreqFree(Blast_ResFreq* rfp);
  * @param rfp the prob element on this Blast_ResFreq is used.
  * @return zero on success
 */
+NCBI_XBLAST_EXPORT
 Int2 Blast_ResFreqStdComp(const BlastScoreBlk* sbp, Blast_ResFreq* rfp);
 
 /** Creates a new structure to keep track of score frequencies for a scoring
@@ -508,6 +611,7 @@ Int2 Blast_ResFreqStdComp(const BlastScoreBlk* sbp, Blast_ResFreq* rfp);
  * @param score_max Maximum score [in]
  * @return allocated and initialized pointer to Blast_ScoreFreq
  */
+NCBI_XBLAST_EXPORT
 Blast_ScoreFreq*
 Blast_ScoreFreqNew(Int4 score_min, Int4 score_max);
 
@@ -515,6 +619,7 @@ Blast_ScoreFreqNew(Int4 score_min, Int4 score_max);
  * @param sfp the structure to deallocate [in]
  * @return NULL
  */
+NCBI_XBLAST_EXPORT
 Blast_ScoreFreq*
 Blast_ScoreFreqFree(Blast_ScoreFreq* sfp);
 
@@ -526,6 +631,7 @@ Blast_ScoreFreqFree(Blast_ScoreFreq* sfp);
  * @param residue_size size of "residues" buffer [in]
  * @return Number of residues in alphabet or negative returns upon error.
  */
+NCBI_XBLAST_EXPORT
 Int2
 Blast_GetStdAlphabet(Uint1 alphabet_code, Uint1* residues, 
                      Uint4 residue_size);
@@ -537,6 +643,7 @@ Blast_GetStdAlphabet(Uint1 alphabet_code, Uint1* residues,
  * @param sfp array of probabilities for all scores [in]
  * @return zero on success, 1 on error.
  */
+NCBI_XBLAST_EXPORT
 Int2
 Blast_KarlinBlkUngappedCalc(Blast_KarlinBlk* kbp, Blast_ScoreFreq* sfp);
 
@@ -548,6 +655,7 @@ Blast_KarlinBlkUngappedCalc(Blast_KarlinBlk* kbp, Blast_ScoreFreq* sfp);
  * @param length the length of the sequence [in]
  * @param resProb the object to be filled in [in|out]
  */
+NCBI_XBLAST_EXPORT
 void
 Blast_FillResidueProbability(const Uint1* sequence, Int4 length, double * resProb);
 
@@ -560,6 +668,7 @@ Blast_FillResidueProbability(const Uint1* sequence, Int4 length, double * resPro
  [in|out]
  * @return zero on success.
 */
+NCBI_XBLAST_EXPORT
 Int2 BlastScoreBlkNuclMatrixCreate(BlastScoreBlk* sbp);
 
 #ifdef __cplusplus
