@@ -1,4 +1,4 @@
-/* $Id: blast_util.c,v 1.105 2005/11/16 14:27:04 madden Exp $
+/* $Id: blast_util.c,v 1.114 2006/05/05 17:46:02 camacho Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,17 +34,16 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_util.c,v 1.105 2005/11/16 14:27:04 madden Exp $";
+    "$Id: blast_util.c,v 1.114 2006/05/05 17:46:02 camacho Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_util.h>
 #include <algo/blast/core/blast_encoding.h>
 #include <algo/blast/core/blast_filter.h>
 #include <algo/blast/core/blast_stat.h>
-#include <algo/blast/core/pattern.h>
 
 Int2
-BlastSetUp_SeqBlkNew (const Uint1* buffer, Int4 length, Int4 context,
+BlastSetUp_SeqBlkNew (const Uint1* buffer, Int4 length,
    BLAST_SequenceBlk* *seq_blk, Boolean buffer_allocated)
 {
    /* Check if BLAST_SequenceBlk itself needs to be allocated here or not */
@@ -63,7 +62,6 @@ BlastSetUp_SeqBlkNew (const Uint1* buffer, Int4 length, Int4 context,
    }
    
    (*seq_blk)->length = length;
-   (*seq_blk)->context = context;
    
    return 0;
 }
@@ -687,111 +685,6 @@ Int1 BLAST_ContextToFrame(EBlastProgramType prog_number, Uint4 context_number)
    return frame;
 }
 
-Int4 
-Blast_GetQueryIndexFromContext(Int4 context, EBlastProgramType program)
-{
-   Int4 index = -1;     /* -1 is used to indicate error */
-   switch (program) {
-   case eBlastTypeBlastn:
-      index = context/NUM_STRANDS; 
-      break;
-   case eBlastTypeBlastp: 
-   case eBlastTypeTblastn: 
-   case eBlastTypeRpsBlast: 
-   case eBlastTypePsiBlast:
-   case eBlastTypeRpsTblastn:
-   case eBlastTypePhiBlastn:
-   case eBlastTypePhiBlastp:
-      index = context; 
-      break;
-   case eBlastTypeBlastx: 
-   case eBlastTypeTblastx:
-      index = context/NUM_FRAMES; 
-      break;
-   default:
-      break;
-   }
-   return index;
-}
-
-BlastQueryInfo* BlastQueryInfoFree(BlastQueryInfo* query_info)
-{
-    if (query_info) {
-        sfree(query_info->contexts);
-        query_info->pattern_info = 
-            SPHIQueryInfoFree(query_info->pattern_info);
-        sfree(query_info);
-    }
-    return NULL;
-}
-
-BlastQueryInfo* BlastQueryInfoDup(BlastQueryInfo* query_info)
-{
-   BlastQueryInfo* retval = BlastMemDup(query_info, sizeof(BlastQueryInfo));
-   Int4 num_contexts = query_info->last_context + 1;
-   
-   retval->contexts =
-       BlastMemDup(query_info->contexts, num_contexts * sizeof(BlastContextInfo));
-   
-   if (query_info->pattern_info) {
-       retval->pattern_info = 
-           SPHIQueryInfoCopy(query_info->pattern_info);
-   }
-
-   return retval;
-}
-
-/** Calculates length of the DNA query from the BlastQueryInfo structure that 
- * contains context information for translated frames for a set of queries.
- * @param query_info Query information containing data for all contexts [in]
- * @param query_index Which query to find DNA length for?
- * @return DNA length of the query, calculated as sum of 3 protein frame lengths, 
- *         plus 2, because 2 last nucleotide residues do not have a 
- *         corresponding codon.
- */
-static Int4 
-s_GetTranslatedQueryDNALength(const BlastQueryInfo* query_info, Int4 query_index)
-{
-    Int4 start_context = NUM_FRAMES*query_index;
-    Int4 dna_length = 2;
-    Int4 index;
- 
-    /* Make sure that query index is within appropriate range, and that this is
-       really a translated search */
-    ASSERT(query_index < query_info->num_queries);
-    ASSERT(start_context < query_info->last_context);
-
-    /* If only reverse strand is searched, then forward strand contexts don't 
-       have lengths information */
-    if (query_info->contexts[start_context].query_length == 0)
-        start_context += 3;
-
-    for (index = start_context; index < start_context + 3; ++index)
-        dna_length += query_info->contexts[index].query_length;
- 
-    return dna_length;
-}
-
-Int4 BlastQueryInfoGetQueryLength(const BlastQueryInfo* qinfo,
-                                  EBlastProgramType program,
-                                  Int4 query_index)
-{
-    const Uint4 kNumContexts = BLAST_GetNumberOfContexts(program);
-    ASSERT(query_index < qinfo->num_queries);
-
-    if (Blast_QueryIsTranslated(program)) {
-        return s_GetTranslatedQueryDNALength(qinfo, query_index);
-    } else if (program == eBlastTypeBlastn) {
-        Int4 retval = qinfo->contexts[query_index*kNumContexts].query_length;
-        if (retval <= 0) {
-            retval = qinfo->contexts[query_index*kNumContexts+1].query_length;
-        }
-        return retval;
-    } else {
-        return qinfo->contexts[query_index*kNumContexts].query_length;
-    }
-}
-
 Int2 BLAST_PackDNA(const Uint1* buffer, Int4 length, EBlastEncoding encoding, 
                           Uint1** packed_seq)
 {
@@ -844,18 +737,23 @@ Int2 BLAST_PackDNA(const Uint1* buffer, Int4 length, EBlastEncoding encoding,
    return 0;
 }
 
-Int2 BLAST_InitDNAPSequence(BLAST_SequenceBlk* query_blk, 
-                            const BlastQueryInfo* query_info)
+Int2 BLAST_CreateMixedFrameDNATranslation(BLAST_SequenceBlk* query_blk, 
+                                          const BlastQueryInfo* query_info)
 {
-   Uint1* buffer,* seq = NULL,* tmp_seq;
-   Int4 total_length, index, offset, i, context;
+   Uint1* buffer,* seq = NULL;
+   Int4 index;
    Int4 length[CODON_LENGTH];
-   
-   total_length = QueryInfo_GetSeqBufLen(query_info);
+   Int4 total_length = QueryInfo_GetSeqBufLen(query_info);
+
    /* Allocate 1 extra byte for a final sentinel. */ 
    buffer = (Uint1*) malloc(total_length+1);
 
    for (index = 0; index <= query_info->last_context; index += CODON_LENGTH) {
+      int i;
+
+      if (query_info->contexts[index].query_length == 0)  /* Indicates this context is not searched. */
+         continue;
+
       seq = &buffer[query_info->contexts[index].query_offset];
 
       for (i = 0; i < CODON_LENGTH; ++i) {
@@ -864,8 +762,9 @@ Int2 BLAST_InitDNAPSequence(BLAST_SequenceBlk* query_blk,
       }
       
       for (i = 0; ; ++i) {
-         context = i % 3;
-         offset = i / 3;
+         Uint1 *tmp_seq;
+         Int4 context = i % 3;
+         Int4 offset = i / 3;
          if (offset >= length[context]) {
             /* Once one frame is past its end, we are done */
             break;
@@ -1131,167 +1030,6 @@ Int4 BSearchInt4(Int4 n, Int4* A, Int4 size)
     return b;
 }
 
-Int4 BSearchContextInfo(Int4 n, BlastQueryInfo * A)
-{
-    Int4 m=0, b=0, e=0, size=0;
-    
-    size = A->last_context+1;
-    
-    b = 0;
-    e = size;
-    while (b < e - 1) {
-	m = (b + e) / 2;
-	if (A->contexts[m].query_offset > n)
-	    e = m;
-	else
-	    b = m;
-    }
-    return b;
-}
-
-Uint4
-QueryInfo_GetSeqBufLen(const BlastQueryInfo* qinfo)
-{
-    BlastContextInfo * cinfo = & qinfo->contexts[qinfo->last_context];
-    return cinfo->query_offset + cinfo->query_length + (cinfo->query_length ? 2 : 1);
-}
-
-Int4 *
-ContextOffsetsToOffsetArray(BlastQueryInfo* info)
-{
-    /* The Many Values of 'Length'
-     *
-     * 1. info->last_context: the index of the last query offset.
-     *
-     * 2. count: the number of query offsets.
-     *
-     * 3. count + 1: the size of the output array (has an 'extra'
-     *    member so as to communicate the last sequence length).
-     *
-     * 4. sz: the size of the context object array
-     */
-    
-    Uint4   count  = (info->last_context + 1);
-    Uint4   sz     = sizeof(Int4) * (count+1);
-    Uint4   frame  = 0;
-    Int4  * result = 0;
-    
-    ASSERT(info);
-    ASSERT(info->contexts);
-    
-    result = malloc(sz);
-    memset(result, 0, sz);
-    
-    for(frame = 0; frame < count; frame++) {
-        result[frame] = info->contexts[frame].query_offset;
-    }
-    
-    /* One more entry, provides length info for last element. */
-    
-    result[count] = info->contexts[count-1].query_offset;
-    
-    if (info->contexts[count-1].query_length) {
-        result[count] += info->contexts[count-1].query_length + 1;
-    }
-    
-    return result;
-}
-
-void
-OffsetArrayToContextOffsets(BlastQueryInfo    * info,
-                            Int4              * new_offsets,
-                            EBlastProgramType   prog)
-{
-    Uint4 count = (info->last_context + 1);
-    Uint4 i     = 0;
-    
-    ASSERT(info);
-    ASSERT(new_offsets);
-    
-    if (! info->contexts) {
-        info->contexts = calloc(count, sizeof(BlastContextInfo));
-    }
-    
-    for(i = 0; i < count; i++) {
-        Int4 distance = 0;
-        
-        info->contexts[i].query_offset = new_offsets[i];
-        
-        distance = new_offsets[i+1] - new_offsets[i];
-        info->contexts[i].query_length = distance ? distance-1 : 0;
-        
-        /* Set the frame and query index */
-        
-        info->contexts[i].frame =
-            BLAST_ContextToFrame(prog, i);
-        
-        info->contexts[i].query_index =
-            Blast_GetQueryIndexFromContext(i, prog);
-    }
-}
-
-Int2
-Blast_GetOneQueryStructs(BlastQueryInfo** one_query_info_ptr, 
-                         BLAST_SequenceBlk** one_query_ptr,
-                         const BlastQueryInfo* query_info, 
-                         BLAST_SequenceBlk* query, Int4 query_index)
-{
-    Int4 num_frames;
-    Int4 index;
-    Int4 first_context;
-    Int4 query_offset;
-    BlastQueryInfo* one_query_info = NULL;
-    BLAST_SequenceBlk* one_query = NULL;
-
-    if (!one_query_info_ptr || !one_query_ptr || !query_info || !query ||
-        query_index >= query_info->num_queries)
-        return -1;
-
-    num_frames = (query_info->last_context / query_info->num_queries) + 1;
-    first_context = query_index*num_frames;
-    query_offset = query_info->contexts[first_context].query_offset;
-
-    one_query_info = *one_query_info_ptr;
-    /* If this hasn't been already done, allocate new query information 
-       structure. */
-    if (!one_query_info) {
-        one_query_info = (BlastQueryInfo*) calloc(1, sizeof(BlastQueryInfo));
-        *one_query_info_ptr = one_query_info;
-        one_query_info->contexts = (BlastContextInfo*) calloc(num_frames, sizeof(BlastContextInfo));
-    }
-    one_query = *one_query_ptr;
-    /* If this hasn't been already done, allocate new sequence block. */
-    if (!one_query) {
-        one_query = (BLAST_SequenceBlk*) calloc(1, sizeof(BLAST_SequenceBlk));
-        *one_query_ptr = one_query;
-    }
-    if (!one_query_info || !one_query)
-        return -1;
-
-    one_query_info->num_queries = 1;
-    one_query_info->last_context = num_frames - 1;
-    
-    memcpy(one_query_info->contexts,
-           &query_info->contexts[first_context],
-           num_frames * sizeof(BlastContextInfo));
-    
-    /* Make context offsets relative to this query. */
-    for (index = 0; index < num_frames; ++index) {
-        one_query_info->contexts[index].query_offset -= query_offset;
-    }
-    
-    /* Fill the sequence block information for this one query. */
-    memset(one_query, 0, sizeof(BLAST_SequenceBlk));
-    one_query->sequence = &query->sequence[query_offset];
-    one_query->length =
-        one_query_info->contexts[num_frames-1].query_offset +
-        one_query_info->contexts[num_frames-1].query_length;
-    one_query->sequence_allocated = FALSE;
-    one_query->oid = query_index;
-
-    return 0;
-}
-
 Int2
 Blast_SetUpSubjectTranslation(BLAST_SequenceBlk* subject_blk, 
                               const Uint1* gen_code_string,
@@ -1411,3 +1149,31 @@ BLAST_GetNumberOfContexts(EBlastProgramType p)
 
     return retval;
 }
+
+SBlastProgress* SBlastProgressNew(void* user_data)
+{
+    SBlastProgress* retval = (SBlastProgress*)calloc(1, sizeof(SBlastProgress));
+    if ( !retval ) {
+        return NULL;
+    }
+    retval->user_data = user_data;
+    return retval;
+}
+
+SBlastProgress* SBlastProgressFree(SBlastProgress* progress_info)
+{
+    if ( !progress_info ) {
+        return NULL;
+    }
+    sfree(progress_info);
+    return NULL;
+}
+
+void SBlastProgressReset(SBlastProgress* progress_info)
+{
+    if ( !progress_info ) {
+        return;
+    }
+    progress_info->stage = ePrelimSearch;
+}
+
