@@ -1,4 +1,4 @@
-/* $Id: mb_lookup.c,v 1.57 2005/11/16 14:27:04 madden Exp $
+/* $Id: mb_lookup.c,v 1.60 2005/12/22 14:04:52 papadopo Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: mb_lookup.c,v 1.57 2005/11/16 14:27:04 madden Exp $";
+    "$Id: mb_lookup.c,v 1.60 2005/12/22 14:04:52 papadopo Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_options.h>
@@ -113,14 +113,13 @@ s_GetDiscTemplateType(Int4 weight, Uint1 length,
  * @param location locations on the query to be indexed in table [in]
  * @param mb_lt the (already allocated) megablast lookup 
  *              table structure [in|out]
- * @param width number of bytes in megablast word (2 or 3) [in]
  * @param lookup_options specifies the word_size and template options [in]
  * @return zero on success, negative number on failure. 
  */
 
 static Int2 
 s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
-        BlastMBLookupTable* mb_lt, Uint1 width,
+        BlastMBLookupTable* mb_lt,
         const LookupTableOptions* lookup_options)
 
 {
@@ -180,12 +179,8 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
    }
 
    mb_lt->discontiguous = TRUE;
-   mb_lt->compressed_wordsize = width + 1;
-   mb_lt->word_length = COMPRESSION_RATIO*(width+1);
    mb_lt->template_length = lookup_options->mb_template_length;
-   mb_lt->mask = (Int4) (-1);
    template_length = lookup_options->mb_template_length;
-
    pv_array = mb_lt->pv_array;
    pv_array_bts = mb_lt->pv_array_bts;
 
@@ -199,11 +194,11 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
       Uint1* seq;
       Uint1 val;
 
-      /* A word is added to the table after its last base is added.
-         At that point, the start offset of the word is
-         template_length - 1 positions behind. This index is
-         also incremented, because lookup table indices are 
-         1-based (offset 0 is reserved). */
+      /* A word is added to the table after the last base 
+         in the word is read in. At that point, the start 
+         offset of the word is template_length - 1 positions 
+         behind. This index is also incremented, because 
+         lookup table indices are 1-based (offset 0 is reserved). */
 
       from = loc->ssr->left - (template_length - 2);
       to = loc->ssr->right - (template_length - 2);
@@ -225,12 +220,17 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
          if (seq < pos)
             continue;
 
+#ifdef LOOKUP_VERBOSE
+         mb_lt->num_words_added++;
+#endif
          /* compute the hashtable index for the first template
             and add 'index' at that position */
 
          ecode1 = ComputeDiscontiguousIndex(accum, template_type);
          if (mb_lt->hashtable[ecode1] == 0) {
+#ifdef LOOKUP_VERBOSE
             mb_lt->num_unique_pos_added++;
+#endif
             pv_array[ecode1>>pv_array_bts] |= 
                          (((PV_ARRAY_TYPE) 1)<<(ecode1&PV_ARRAY_MASK));
          }
@@ -247,7 +247,9 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
          
          ecode2 = ComputeDiscontiguousIndex(accum, second_template_type);
          if (mb_lt->hashtable2[ecode2] == 0) {
+#ifdef LOOKUP_VERBOSE
             mb_lt->num_unique_pos_added++;
+#endif
             pv_array[ecode2>>pv_array_bts] |= 
                          (((PV_ARRAY_TYPE) 1)<<(ecode2&PV_ARRAY_MASK));
          }
@@ -281,29 +283,29 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
  * @param query the query sequence [in]
  * @param location locations on the query to be indexed in table [in]
  * @param mb_lt the (already allocated) megablast lookup table structure [in|out]
- * @param width number of bytes in megablast word (2 or 3) [in]
  * @param lookup_options specifies the word_size [in]
  * @return zero on success, negative number on failure. 
  */
 
 static Int2 
-s_FillContigMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
-        BlastMBLookupTable* mb_lt, Uint1 width,
+s_FillContigMBTable(BLAST_SequenceBlk* query, 
+        BlastSeqLoc* location,
+        BlastMBLookupTable* mb_lt, 
         const LookupTableOptions* lookup_options)
 
 {
    BlastSeqLoc* loc;
    const Uint1 kNucMask = 0xfc;
    /* 12-mers (or perhaps 8-mers) are used to build the lookup table 
-      and this is what kWordLength specifies. */
-   const Int4 kWordLength = COMPRESSION_RATIO*width;   
+      and this is what kLutWordLength specifies. */
+   const Int4 kLutWordLength = mb_lt->lut_word_length;
+   const Int4 kLutMask = mb_lt->hashsize - 1;
    /* The user probably specified a much larger word size (like 28) 
       and this is what full_word_size is. */
-   Int4 full_word_size;
+   Int4 full_word_size = mb_lt->word_length;
    Int4 index;
-   PV_ARRAY_TYPE *pv_array=NULL;
+   PV_ARRAY_TYPE *pv_array;
    Int4 pv_array_bts;
-   const Int4 kMask = (1 << (8*width - 2)) - 1;
    /* The calculation of the longest chain can be cpu intensive for 
       long queries or sets of queries. So we use a helper_array to 
       keep track of this, but compress it by kCompressionFactor so 
@@ -317,15 +319,10 @@ s_FillContigMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
 
    ASSERT(mb_lt);
 
-   /* The next_pos array begins small and will grow dynamically */
    mb_lt->next_pos = (Int4 *)calloc(query->length + 1, sizeof(Int4));
    if (mb_lt->next_pos == NULL)
       return -1;
 
-   mb_lt->compressed_wordsize = width;
-   full_word_size = mb_lt->word_length = lookup_options->word_size;
-   mb_lt->mask = mb_lt->hashsize - 1;
-
    pv_array = mb_lt->pv_array;
    pv_array_bts = mb_lt->pv_array_bts;
 
@@ -339,41 +336,48 @@ s_FillContigMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
          Since sequence pointer points to the end of the word, subtract
          word length from the loop boundaries.  */
       Int4 from = loc->ssr->left;
-      Int4 to = loc->ssr->right - kWordLength;
+      Int4 to = loc->ssr->right - kLutWordLength;
       Int4 ecode = 0;
       Int4 last_offset;
       Uint1* pos;
       Uint1* seq;
       Uint1 val;
 
-     /* case of unmasked region >=  kWordLength but < full_word_size,
+     /* case of unmasked region >=  kLutWordLength but < full_word_size,
         so no hits should be generated. */
       if (full_word_size > (loc->ssr->right - loc->ssr->left + 1))
          continue; 
 
       seq = query->sequence_start + from;
-      pos = seq + kWordLength;
+      pos = seq + kLutWordLength;
       
       /* Also add 1 to all indices, because lookup table indices count 
          from 1. */
-      from -= kWordLength - 2;
+      from -= kLutWordLength - 2;
       last_offset = to + 2;
 
       for (index = from; index <= last_offset; index++) {
          val = *++seq;
-         if ((val & kNucMask) != 0) { /* ambiguity or gap */
+         /* if an ambiguity is encountered, do not add
+            any words that would contain it */
+         if ((val & kNucMask) != 0) {
             ecode = 0;
-            pos = seq + kWordLength;
+            pos = seq + kLutWordLength;
             continue;
          }
 
          /* get next base */
-         ecode = ((ecode & kMask) << 2) + val;
+         ecode = ((ecode << 2) & kLutMask) + val;
          if (seq < pos) 
             continue;
 
+#ifdef LOOKUP_VERBOSE
+         mb_lt->num_words_added++;
+#endif
          if (mb_lt->hashtable[ecode] == 0) {
+#ifdef LOOKUP_VERBOSE
             mb_lt->num_unique_pos_added++;
+#endif
             pv_array[ecode>>pv_array_bts] |= 
                          (((PV_ARRAY_TYPE) 1)<<(ecode&PV_ARRAY_MASK));
          }
@@ -398,13 +402,14 @@ s_FillContigMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
 /* Documentation in mb_lookup.h */
 Int2 MB_LookupTableNew(BLAST_SequenceBlk* query, BlastSeqLoc* location,
         BlastMBLookupTable** mb_lt_ptr,
-        const LookupTableOptions* lookup_options)
+        const LookupTableOptions* lookup_options,
+        Int4 approx_table_entries)
 {
-   Int4 pv_shift, pv_size;
-   Int2 status=0;
-   Int2 bytes_in_word;
-   Uint1 width;
+   Int4 pv_size;
+   Int2 status = 0;
    BlastMBLookupTable* mb_lt;
+   const Int4 kAlphabetSize = BLAST2NA_SIZE;
+   const Int4 kTargetPVSize = 131072;
    const Int4 kSmallQueryCutoff = 15000;
    const Int4 kLargeQueryCutoff = 800000;
    
@@ -415,97 +420,114 @@ Int2 MB_LookupTableNew(BLAST_SequenceBlk* query, BlastSeqLoc* location,
      return -1;
    }
 
-   if ((mb_lt = (BlastMBLookupTable*) calloc(1, sizeof(BlastMBLookupTable))) == NULL)
-   {
+   mb_lt = (BlastMBLookupTable*)calloc(1, sizeof(BlastMBLookupTable));
+   if (mb_lt == NULL) {
 	return -1;
    }
     
-   bytes_in_word = (lookup_options->word_size + 1)/ 4;
-   if (bytes_in_word < 3) 
-      width = 2;
-   else
-      width = 3;
+   /* first determine how 'wide' the lookup table is,
+      i.e. how many nucleotide letters are in a single entry */
 
+   if (lookup_options->mb_template_length > 0) {
 
-   if (width == 2) {
-      mb_lt->hashsize = 1 << 16;
-      pv_shift = 0;
+      /* for discontiguous megablast the choice is fixed */
+      if (lookup_options->word_size == 11)
+         mb_lt->lut_word_length = 11;
+      else
+         mb_lt->lut_word_length = 12;
    }
    else {
-      Int4 from, to;
-      BlastSeqLoc* loc;
-      Int4 table_entries=0;
-      /* determine the approximate number of hashtable entries */
-      for (loc = location; loc; loc = loc->next) {
-         from = loc->ssr->left;
-         to = loc->ssr->right;
-         table_entries += (to - from);
+       /* Choose the smallest lookup table that will not
+          be congested; this is a compromise between good
+          cache performance and few table accesses. The 
+          number of entries where one table width becomes
+          better than another is probably machine-dependent */
+
+      ASSERT(lookup_options->word_size >= 10);
+
+      switch(lookup_options->word_size) {
+      case 10:
+         if (approx_table_entries < 18000)
+            mb_lt->lut_word_length = 9;
+         else
+            mb_lt->lut_word_length = 10;
+         break;
+      case 11:
+         if (approx_table_entries < 18000)
+            mb_lt->lut_word_length = 9;
+         else if (approx_table_entries < 180000)
+            mb_lt->lut_word_length = 10;
+         else
+            mb_lt->lut_word_length = 11;
+         break;
+      case 12:
+         if (approx_table_entries < 18000)
+            mb_lt->lut_word_length = 9;
+         else if (approx_table_entries < 60000)
+            mb_lt->lut_word_length = 10;
+         else if (approx_table_entries < 900000)
+            mb_lt->lut_word_length = 11;
+         else
+            mb_lt->lut_word_length = 12;
+         break;
+      default:
+         if (approx_table_entries < 300000)
+            mb_lt->lut_word_length = 11;
+         else
+            mb_lt->lut_word_length = 12;
+         break;
       }
 
-      /* To fit in the external cache of latter-day micro-
-         processors, the PV array must be compressed. pv_shift
-	 below is the power of two that the array size is
-	 divided by. The target PV array size is 128 kBytes.
-
-	 If the query is too small or too large, the compression 
-	 should be higher. Small queries don't reuse the PV array,
-	 and large queries saturate it. In either case, cache
-	 is better used on other data. */
-
-      if (lookup_options->word_size == 11 && lookup_options->mb_template_length > 0) {
-         mb_lt->hashsize = 1 << 22;
-         if( table_entries <= kSmallQueryCutoff ||
-	     table_entries >= kLargeQueryCutoff )
-            pv_shift = 3;
-	 else
-            pv_shift = 2;
-      }
-      else {
-         mb_lt->hashsize = 1 << 24;
-         if( table_entries <= kSmallQueryCutoff ||
-	     table_entries >= kLargeQueryCutoff )
-            pv_shift = 5;
-	 else
-            pv_shift = 4;
-      }
+      mb_lt->lut_word_length = MIN(lookup_options->word_size,
+                                   mb_lt->lut_word_length);
    }
 
-   if (lookup_options->mb_template_length)
-   {
-        mb_lt->full_byte_scan = lookup_options->full_byte_scan;  /* Only applies to discontiguous megablast. */
-        /* The next two are not allowed in discontiguous megablast. */
-        mb_lt->ag_scanning_mode = FALSE;
-        mb_lt->variable_wordsize = FALSE; 
-        mb_lt->scan_step = 0;
-   }
-   else
-   {
-        mb_lt->ag_scanning_mode = TRUE; /* FIXME, are there wordsize where this should be FALSE? */
-        mb_lt->scan_step = CalculateBestStride(lookup_options->word_size, lookup_options->variable_wordsize, 
-                  lookup_options->lut_type);
-        mb_lt->variable_wordsize = lookup_options->variable_wordsize;
-   }
-
-   if ((mb_lt->hashtable = (Int4*) 
-        calloc(mb_lt->hashsize, sizeof(Int4))) == NULL) {
+   mb_lt->word_length = lookup_options->word_size;
+   mb_lt->hashsize = iexp(kAlphabetSize, mb_lt->lut_word_length);
+   mb_lt->hashtable = (Int4*)calloc(mb_lt->hashsize, sizeof(Int4));
+   if (mb_lt->hashtable == NULL) {
       MBLookupTableDestruct(mb_lt);
       return -1;
    }
 
-   mb_lt->pv_array_bts = PV_ARRAY_BTS + pv_shift; 
+   /* Allocate the PV array. To fit in the external cache of 
+      latter-day microprocessors, the PV array cannot have one
+      bit for for every lookup table entry. Instead we choose
+      a size that should fit in cache and make a single bit
+      of the PV array handle multiple hashtable entries if
+      necessary.
 
-   pv_size = mb_lt->hashsize>>(mb_lt->pv_array_bts);
-   /* Size measured in PV_ARRAY_TYPE's */
-   if ((mb_lt->pv_array = calloc(PV_ARRAY_BYTES, pv_size)) == NULL)
-   {
+      If the query is too small or too large, the compression 
+      should be higher. Small queries don't reuse the PV array,
+      and large queries saturate it. In either case, cache
+      is better used on something else */
+
+   if (mb_lt->hashsize <= 8 * kTargetPVSize)
+      pv_size = mb_lt->hashsize >> PV_ARRAY_BTS;
+   else
+      pv_size = kTargetPVSize / PV_ARRAY_BYTES;
+
+   if(approx_table_entries <= kSmallQueryCutoff ||
+      approx_table_entries >= kLargeQueryCutoff) {
+         pv_size = pv_size / 2;
+   }
+   mb_lt->pv_array_bts = ilog2(mb_lt->hashsize / pv_size);
+   mb_lt->pv_array = calloc(PV_ARRAY_BYTES, pv_size);
+   if (mb_lt->pv_array == NULL) {
       MBLookupTableDestruct(mb_lt);
       return -1;
    }
 
-   if (lookup_options->mb_template_length)
-     status = s_FillDiscMBTable(query, location, mb_lt, width, lookup_options);
-   else
-     status = s_FillContigMBTable(query, location, mb_lt, width, lookup_options);
+   if (lookup_options->mb_template_length > 0) {
+        /* discontiguous megablast */
+        mb_lt->full_byte_scan = lookup_options->full_byte_scan; 
+        status = s_FillDiscMBTable(query, location, mb_lt, lookup_options);
+   }
+   else {
+        /* contiguous megablast */
+        mb_lt->scan_step = mb_lt->word_length - mb_lt->lut_word_length + 1;
+        status = s_FillContigMBTable(query, location, mb_lt, lookup_options);
+   }
 
    if (status > 0) {
       MBLookupTableDestruct(mb_lt);
@@ -513,6 +535,18 @@ Int2 MB_LookupTableNew(BLAST_SequenceBlk* query, BlastSeqLoc* location,
    }
 
    *mb_lt_ptr = mb_lt;
+
+#ifdef LOOKUP_VERBOSE
+   printf("lookup table size: %d (%d letters)\n", mb_lt->hashsize,
+                                        mb_lt->lut_word_length);
+   printf("words in table: %d\n", mb_lt->num_words_added);
+   printf("filled entries: %d (%f%%)\n", mb_lt->num_unique_pos_added,
+                        100.0 * mb_lt->num_unique_pos_added / mb_lt->hashsize);
+   printf("PV array size: %d bytes (%d table entries/bit)\n",
+                                 pv_size * PV_ARRAY_BYTES, 
+                                 mb_lt->hashsize / (pv_size << PV_ARRAY_BTS));
+   printf("longest chain: %d\n", mb_lt->longest_chain);
+#endif
    return 0;
 }
 
@@ -524,12 +558,14 @@ Int4 MB_AG_ScanSubject(const LookupTableWrap* lookup_wrap,
    BlastMBLookupTable* mb_lt;
    Uint1* s;
    Uint1* abs_start;
-   Int4  index=0, s_off;
-   
-   Int4 q_off;
+   Int4 q_off, s_off;
+   Int4 last_offset;
+   Int4 index;
+   Int4 mask;
    PV_ARRAY_TYPE *pv_array;
    Int4 total_hits = 0;
-   Int4 compressed_wordsize, compressed_scan_step, word_size;
+   Int4 lut_word_length;
+   Int4 scan_step;
    Uint1 pv_array_bts;
    
    ASSERT(lookup_wrap->lut_type == MB_LOOKUP_TABLE);
@@ -543,51 +579,103 @@ Int4 MB_AG_ScanSubject(const LookupTableWrap* lookup_wrap,
    max_hits -= mb_lt->longest_chain;
 
    abs_start = subject->sequence;
-   s = abs_start + start_offset/COMPRESSION_RATIO;
-   compressed_scan_step = mb_lt->scan_step / COMPRESSION_RATIO;
-   compressed_wordsize = mb_lt->compressed_wordsize;
-   word_size = compressed_wordsize*COMPRESSION_RATIO;
+   lut_word_length = mb_lt->lut_word_length;
+   mask = mb_lt->hashsize - 1;
+   scan_step = mb_lt->scan_step;
+   last_offset = subject->length - lut_word_length;
+   ASSERT(lut_word_length == 9 || 
+          lut_word_length == 10 ||
+          lut_word_length == 11 ||
+          lut_word_length == 12);
 
-   index = 0;
+   if (lut_word_length > 9) {
+      /* perform scanning for lookup tables of width 10, 11, and 12.
+         These widths require three bytes of the compressed subject
+         sequence, and possibly a fourth if the word is not known
+         to be aligned on a 4-base boundary */
+
+      if (scan_step % COMPRESSION_RATIO == 0) {
+
+         /* for strides that are a multiple of 4, words are
+            always aligned and three bytes of the subject sequence 
+            will always hold a complete word (plus possible extra bases 
+            that must be shifted away). s_end below points to either
+            the second to last or third to last byte of the subject 
+            sequence; all subject sequences must therefore have a 
+            sentinel byte */
+
+         Uint1* s_end = abs_start + (subject->length - lut_word_length) /
+                                                   COMPRESSION_RATIO;
+         Int4 shift = 2 * (12 - lut_word_length);
+         s = abs_start + start_offset / COMPRESSION_RATIO;
+         scan_step = scan_step / COMPRESSION_RATIO;
+   
+         for ( ; s <= s_end; s += scan_step) {
+            
+            index = s[0] << 16 | s[1] << 8 | s[2];
+            index = index >> shift;
+      
+            if (NA_PV_TEST(pv_array, index, pv_array_bts)) {
+               if (total_hits >= max_hits)
+                  break;
+               q_off = mb_lt->hashtable[index];
+               s_off = (s - abs_start)*COMPRESSION_RATIO;
+               while (q_off) {
+                  offset_pairs[total_hits].qs_offsets.q_off = q_off - 1;
+                  offset_pairs[total_hits++].qs_offsets.s_off = s_off;
+                  q_off = mb_lt->next_pos[q_off];
+               }
+            }
+         }
+         *end_offset = (s - abs_start)*COMPRESSION_RATIO;
+      }
+      else {
+         /* when the stride is not a multiple of 4, extra bases
+            may occur both before and after every word read from
+            the subject sequence. The portion of each 16-base
+            region that contains the actual word depends on the
+            offset of the word and the lookup table width, and
+            must be recalculated for each 16-base region */
+
+         for (s_off = start_offset; s_off <= last_offset; s_off += scan_step) {
+      
+            Int4 shift = 2*(16 - (s_off % COMPRESSION_RATIO + lut_word_length));
+            s = abs_start + (s_off / COMPRESSION_RATIO);
+      
+            index = s[0] << 24 | s[1] << 16 | s[2] << 8 | s[3];
+            index = (index >> shift) & mask;
+      
+            if (NA_PV_TEST(pv_array, index, pv_array_bts)) {
+               if (total_hits >= max_hits)
+                  break;
+               q_off = mb_lt->hashtable[index];
+               while (q_off) {
+                  offset_pairs[total_hits].qs_offsets.q_off = q_off - 1;
+                  offset_pairs[total_hits++].qs_offsets.s_off = s_off;
+                  q_off = mb_lt->next_pos[q_off];
+               }
+            }
+         }
+         *end_offset = s_off;
+      }
+   }
+   else {
+      /* perform scanning for a lookup tables of width 9
+         Here the stride will never be a multiple of 4 
+         (this table is only used for very small word sizes) */
+
+      for (s_off = start_offset; s_off <= last_offset; s_off += scan_step) {
+   
+         Int4 shift = 2*(12 - (s_off % COMPRESSION_RATIO + lut_word_length));
+         s = abs_start + (s_off / COMPRESSION_RATIO);
+   
+         index = s[0] << 16 | s[1] << 8 | s[2];
+         index = (index >> shift) & mask;
    
-   /* NB: s in this function always points to the start of the word!
-    */
-   if (mb_lt->scan_step % COMPRESSION_RATIO == 0) {
-      Uint1* s_end = abs_start + subject->length/COMPRESSION_RATIO - 
-         compressed_wordsize;
-      for ( ; s <= s_end; s += compressed_scan_step) {
-         BlastNaLookupInitIndex(compressed_wordsize, s, &index);
-         
          if (NA_PV_TEST(pv_array, index, pv_array_bts)) {
             if (total_hits >= max_hits)
                break;
             q_off = mb_lt->hashtable[index];
-            s_off = (s - abs_start)*COMPRESSION_RATIO;
-            while (q_off) {
-               offset_pairs[total_hits].qs_offsets.q_off = q_off - 1;
-               offset_pairs[total_hits++].qs_offsets.s_off = s_off;
-               q_off = mb_lt->next_pos[q_off];
-            }
-         }
-      }
-      *end_offset = (s - abs_start)*COMPRESSION_RATIO;
-   } else {
-      Int4 last_offset = subject->length - word_size;
-      Uint1 bit;
-      Int4 adjusted_index;
-
-      for (s_off = start_offset; s_off <= last_offset; 
-           s_off += mb_lt->scan_step) {
-         s = abs_start + (s_off / COMPRESSION_RATIO);
-         bit = 2*(s_off % COMPRESSION_RATIO);
-         /* Compute index for a word made of full bytes */
-         s = BlastNaLookupInitIndex(compressed_wordsize, s, &index);
-         /* Adjust the word index by the base within a byte */
-         adjusted_index = BlastNaLookupAdjustIndex(s, index, mb_lt->mask, bit);
-         if (NA_PV_TEST(pv_array, adjusted_index, pv_array_bts)) {
-            if (total_hits >= max_hits)
-               break;
-            q_off = mb_lt->hashtable[adjusted_index];
             while (q_off) {
                offset_pairs[total_hits].qs_offsets.q_off = q_off - 1;
                offset_pairs[total_hits++].qs_offsets.s_off = s_off;
@@ -601,65 +689,6 @@ Int4 MB_AG_ScanSubject(const LookupTableWrap* lookup_wrap,
    return total_hits;
 }
 
-Int4 MB_ScanSubject(const LookupTableWrap* lookup,
-       const BLAST_SequenceBlk* subject, Int4 start_offset, 
-       BlastOffsetPair* NCBI_RESTRICT offset_pairs, Int4 max_hits,
-       Int4* end_offset) 
-{
-   Uint1* abs_start,* s_end;
-   Uint1* s;
-   Int4 total_hits = 0;
-   Uint4 q_off, s_off;
-   Int4 index;
-   BlastMBLookupTable* mb_lt;
-   PV_ARRAY_TYPE *pv_array;
-   Uint1 pv_array_bts;
-   Int4 compressed_wordsize;
-
-   ASSERT(lookup->lut_type == MB_LOOKUP_TABLE);
-   mb_lt = (BlastMBLookupTable*) lookup->lut;
-
-   pv_array = mb_lt->pv_array;
-   pv_array_bts = mb_lt->pv_array_bts;
-   compressed_wordsize = mb_lt->compressed_wordsize;
-
-   /* Since the test for number of hits here is done after adding them, 
-      subtract the longest chain length from the allowed offset array size.
-   */
-   max_hits -= mb_lt->longest_chain;
-
-   abs_start = subject->sequence;
-   s = abs_start + start_offset/COMPRESSION_RATIO;
-   s_end = abs_start + (*end_offset)/COMPRESSION_RATIO;
-
-   s = BlastNaLookupInitIndex(mb_lt->compressed_wordsize, s, &index);
-
-   /* In the following code, s points to the byte right after the end of 
-      the word. */
-   while (s <= s_end) {
-      if (NA_PV_TEST(pv_array, index, pv_array_bts)) {
-         if (total_hits >= max_hits)
-            break;
-         q_off = mb_lt->hashtable[index];
-         s_off = ((s - abs_start) - compressed_wordsize)*COMPRESSION_RATIO;
-         while (q_off) {
-            offset_pairs[total_hits].qs_offsets.q_off = q_off - 1;
-            offset_pairs[total_hits++].qs_offsets.s_off = s_off;
-            q_off = mb_lt->next_pos[q_off];
-         }
-      }
-      /* Compute the next value of the lookup index 
-         (shifting sequence by a full byte) */
-      index = BlastNaLookupComputeIndex(FULL_BYTE_SHIFT, mb_lt->mask, 
-                                        s++, index);
-   }
-
-   *end_offset = 
-     ((s - abs_start) - compressed_wordsize)*COMPRESSION_RATIO;
-
-   return total_hits;
-}
-
 Int4 MB_DiscWordScanSubject(const LookupTableWrap* lookup, 
        const BLAST_SequenceBlk* subject, Int4 start_offset,
        BlastOffsetPair* NCBI_RESTRICT offset_pairs, Int4 max_hits, 
