@@ -1,4 +1,4 @@
-/* $Id: phi_lookup.h,v 1.7 2004/09/13 12:39:50 madden Exp $
+/* $Id: phi_lookup.h,v 1.11 2006/02/01 14:58:45 ivanov Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -37,50 +37,34 @@
 #include <algo/blast/core/pattern.h>
 #include <algo/blast/core/lookup_wrap.h>
 
-#ifndef PHI_LOOKUP__H
-#define PHI_LOOKUP__H
+#ifndef ALGO_BLAST_CORE__PHI_LOOKUP_H
+#define ALGO_BLAST_CORE__PHI_LOOKUP_H
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/** The following 3 flags define 3 options for running the program */
-#define SEED_FLAG 1         
-#define PATTERN_FLAG 2
-#define PAT_SEED_FLAG 3
-#define PAT_MATCH_FLAG 4
-#define PATTERN_TOO_LONG  2
-
-/** Original allocation size for the starts and lengths arrays for pattern 
- * occurrencies in the query.
+/** Initialize the pattern items structure, serving as a "pseudo" lookup table
+ * in a PHI BLAST search.
+ * @param pattern String describing the pattern to search for [in]
+ * @param is_dna boolean describing whether the strings are DNA or protein [in]
+ * @param sbp Scoring block with statistical parameters [in]
+ * @param pattern_blk The initialized structure [out]
+ * @param error_msg Error message, if any.
+ * @return 0 on success, -1 on failure.
  */
-#define MIN_PHI_LOOKUP_SIZE 1000
+NCBI_XBLAST_EXPORT
+Int2 SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp, 
+                            SPHIPatternSearchBlk* *pattern_blk, 
+                            Blast_Message* *error_msg);
 
-/** Pseudo lookup table structure for PHI-BLAST. Contains starting and ending
- * offsets of pattern occurrences in the query sequence. 
+/** Deallocate memory for the PHI BLAST lookup table.
+ * @param pattern_blk The structure to deallocate [in]
+ * @return NULL.
  */
-typedef struct BlastPHILookupTable {
-   patternSearchItems* pattern_info;
-   Boolean is_dna;
-   Int4 num_matches;
-   Int4 allocated_size;
-   Int4* start_offsets;
-   Int4* lengths;
-} BlastPHILookupTable;
-
-/** Initialize the pseudo lookup table for PHI BLAST */
-Int2 PHILookupTableNew(const LookupTableOptions* opt, 
-                       BlastPHILookupTable* * lut,
-                       Boolean is_dna, BlastScoreBlk* sbp);
-
-/** Deallocate memory for the PHI BLAST lookup table */
-BlastPHILookupTable* PHILookupTableDestruct(BlastPHILookupTable* lut);
-
-/** Find all occurrencies of a pattern in query, and save starts/stops in the
- * PHILookupTable structure.
- */
-Int4 PHIBlastIndexQuery(BlastPHILookupTable* lookup,
-        BLAST_SequenceBlk* query, BlastSeqLoc* location, Boolean is_dna);
+NCBI_XBLAST_EXPORT
+SPHIPatternSearchBlk* 
+SPHIPatternSearchBlkFree(SPHIPatternSearchBlk* pattern_blk);
 
 
 /**
@@ -91,22 +75,22 @@ Int4 PHIBlastIndexQuery(BlastPHILookupTable* lookup,
  * "offset". 
  *
  * @param lookup_wrap contains the pseudo lookup table with offsets of pattern
- *                    occurrencies in query [in]
+ *                    occurrences in query [in]
  * @param query_blk the query sequence [in]
  * @param subject the subject sequence [in]
  * @param offset the offset in the subject at which to begin scanning [in/out]
- * @param query_offsets array to which hits will be copied [out]
- * @param subject_offsets array to which hits will be copied [out]
+ * @param offset_pairs Array of start and end positions of pattern in subject [out]
  * @param array_size length of the offset arrays [in]
  * @return The number of hits found.
  */
+NCBI_XBLAST_EXPORT
 Int4 PHIBlastScanSubject(const LookupTableWrap* lookup_wrap,
         const BLAST_SequenceBlk *query_blk, const BLAST_SequenceBlk *subject, 
-        Int4* offset, Uint4 * query_offsets, Uint4 * subject_offsets, 
+        Int4* offset, BlastOffsetPair* NCBI_RESTRICT offset_pairs,
         Int4 array_size);
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* PHI_LOOKUP__H */
+#endif /* ALGO_BLAST_CORE__PHI_LOOKUP_H */
