@@ -3,6 +3,27 @@
 *   -- GenBank Feature table define file
 *
 * $Log: gbftdef.h,v $
+* Revision 6.25  2005/10/19 16:56:17  kans
+* added new source qualifiers, expanded array of optional qualifiers
+*
+* Revision 6.24  2005/09/22 20:50:07  kans
+* changed /experimental to /experiment - typo
+*
+* Revision 6.23  2005/09/19 17:17:31  kans
+* removed /hydrogenosome as a separate qualifier - it is really /organelle=hydrogenosome
+*
+* Revision 6.22  2005/09/16 19:56:37  kans
+* added hydrogenosome
+*
+* Revision 6.21  2005/06/15 13:47:21  kans
+* corrected optional qualifier numbers, added Validate_ParFlat_GBFeat function
+*
+* Revision 6.20  2005/06/14 16:06:44  kans
+* initial support for ribosomal_slippage, trans_splicing qualifiers derived from /exception
+*
+* Revision 6.19  2005/06/10 17:25:26  kans
+* added space for new qualifiers experimental, inference, rpt_unit_seq, rpt_unit_range
+*
 * Revision 6.18  2004/08/17 15:50:52  kans
 * added GBQUAL_old_locus_tag and GBQUAL_compare
 *
@@ -194,8 +215,19 @@
 #define GBQUAL_operon            86
 #define GBQUAL_old_locus_tag     87
 #define GBQUAL_compare           88
+#define GBQUAL_experiment        89
+#define GBQUAL_inference         90
+#define GBQUAL_rpt_unit_seq      91
+#define GBQUAL_rpt_unit_range    92
+#define GBQUAL_ribosomal_slippage 93
+#define GBQUAL_trans_splicing    94
+#define GBQUAL_collected_by      95
+#define GBQUAL_collection_date   96
+#define GBQUAL_identified_by     97
+#define GBQUAL_lat_lon           98
+#define GBQUAL_PCR_primers       99
 
-#define ParFlat_TOTAL_GBQUAL     89
+#define ParFlat_TOTAL_GBQUAL    100
 #define ParFlat_TOTAL_IntOr       3
 #define ParFlat_TOTAL_LRB         3
 #define ParFlat_TOTAL_Exp         2
@@ -233,7 +265,7 @@ typedef struct sematic_gbfeature {
     Int2     mand_num;
     Int2     mand_qual[5];
     Int2     opt_num;
-    Int2     opt_qual[55];
+    Int2     opt_qual[65];
 } SematicFeat, PNTR SematicFeatPtr;
 
 typedef struct gbfeat_name {
@@ -254,6 +286,8 @@ NLM_EXTERN GbFeatNamePtr x_ParFlat_GBQual_names PROTO((void));
 NLM_EXTERN SematicFeatPtr x_ParFlat_GBFeat PROTO((void));
 #define ParFlat_GBFeat x_ParFlat_GBFeat()
 
+NLM_EXTERN ValNodePtr Validate_ParFlat_GBFeat (void);
+
 extern CharPtr     ParFlat_IntOrString [ParFlat_TOTAL_IntOr];
 extern CharPtr     ParFlat_LRBString   [ParFlat_TOTAL_LRB];
 extern CharPtr     ParFlat_ExpString   [ParFlat_TOTAL_Exp];
