@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 7/13/91
 *
-* $Revision: 6.49 $
+* $Revision: 6.51 $
 *
 * File Description:  Ports onto Bioseqs
 *
@@ -40,6 +40,12 @@
 *
 *
 * $Log: seqport.h,v $
+* Revision 6.51  2005/08/24 15:14:31  kans
+* modified MolWtForLoc to use StreamCache, added MolWtForBsp and MolWtForStr
+*
+* Revision 6.50  2005/06/01 20:27:06  kans
+* added MapNa4ByteToIUPACplusGapString
+*
 * Revision 6.49  2005/03/15 14:35:44  kans
 * seqport stream gap control flags (2-bit set) are STREAM_EXPAND_GAPS, GAP_TO_SINGLE_DASH, and EXPAND_GAPS_TO_DASHES
 *
@@ -461,6 +467,7 @@ Uint2 or Uint4 for fast integer copying.
 
 NLM_EXTERN Uint4Ptr LIBCALL MapNa2ByteToIUPACString PROTO((Uint1Ptr bytep, Uint4Ptr buf, Int4 total));
 NLM_EXTERN Uint2Ptr LIBCALL MapNa4ByteToIUPACString PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
+NLM_EXTERN Uint2Ptr LIBCALL MapNa4ByteToIUPACplusGapString PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
 NLM_EXTERN Uint2Ptr LIBCALL MapNa2ByteToNa4String PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
 NLM_EXTERN Uint4Ptr LIBCALL MapNa2ByteTo4BitString PROTO((Uint1Ptr bytep, Uint4Ptr buf, Int4 total));
 NLM_EXTERN Uint2Ptr LIBCALL MapNa4ByteTo4BitString PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
@@ -944,11 +951,15 @@ NLM_EXTERN void ConvertNsToGaps (
 *
 *  Returns a protein molecular weight for a SeqLoc
 *    If it cannot calculate the value it returns -1.0
-*    If sequence contains X,B,U,*,orZ it fails
+*    If sequence contains X, J, or O it fails
 *
 ***************************************************************/
 NLM_EXTERN FloatHi MolWtForLoc (SeqLocPtr slp);
 
+NLM_EXTERN FloatHi MolWtForBsp (BioseqPtr bsp);
+
+NLM_EXTERN FloatHi MolWtForStr (CharPtr str);
+
 
 #ifdef __cplusplus
 }
