@@ -53,6 +53,13 @@
 #define ERR_SEQ_INST_BadHTGSeq  1,49
 #define ERR_SEQ_INST_GapInProtein  1,50
 #define ERR_SEQ_INST_BadProteinStart  1,51
+#define ERR_SEQ_INST_TerminalGap  1,52
+#define ERR_SEQ_INST_OverlappingDeltaRange  1,53
+#define ERR_SEQ_INST_LeadingX  1,54
+#define ERR_SEQ_INST_InternalNsInSeqRaw  1,55
+#define ERR_SEQ_INST_InternalNsAdjacentToGap  1,56
+#define ERR_SEQ_INST_CaseDifferenceInSeqID  1,57
+#define ERR_SEQ_INST_DeltaComponentIsGi0  1,58
 #define ERR_SEQ_DESCR  2,0
 #define ERR_SEQ_DESCR_BioSourceMissing  2,1
 #define ERR_SEQ_DESCR_InvalidForType  2,2
@@ -86,6 +93,11 @@
 #define ERR_SEQ_DESCR_MultipleTitles  2,30
 #define ERR_SEQ_DESCR_RefGeneTrackingOnNonRefSeq  2,31
 #define ERR_SEQ_DESCR_BioSourceInconsistency  2,32
+#define ERR_SEQ_DESCR_FastaBracketTitle  2,33
+#define ERR_SEQ_DESCR_MissingText  2,34
+#define ERR_SEQ_DESCR_BadCollectionDate  2,35
+#define ERR_SEQ_DESCR_BadPCRPrimerSequence  2,36
+#define ERR_SEQ_DESCR_BadPunctuation  2,37
 #define ERR_GENERIC  3,0
 #define ERR_GENERIC_NonAsciiAsn  3,1
 #define ERR_GENERIC_Spell  3,2
@@ -94,6 +106,10 @@
 #define ERR_GENERIC_UnnecessaryPubEquiv  3,5
 #define ERR_GENERIC_BadPageNumbering  3,6
 #define ERR_GENERIC_MedlineEntryPub  3,7
+#define ERR_GENERIC_BadDate  3,8
+#define ERR_GENERIC_StructuredCitGenCit  3,9
+#define ERR_GENERIC_CollidingSerialNumbers  3,10
+#define ERR_GENERIC_EmbeddedScript  3,11
 #define ERR_SEQ_PKG  4,0
 #define ERR_SEQ_PKG_NoCdRegionPtr  4,1
 #define ERR_SEQ_PKG_NucProtProblem  4,2
@@ -205,6 +221,33 @@
 #define ERR_SEQ_FEAT_PolyATail  5,92
 #define ERR_SEQ_FEAT_ProteinNameEndsInBracket  5,93
 #define ERR_SEQ_FEAT_CDSwithMultipleMRNAs  5,94
+#define ERR_SEQ_FEAT_MultipleEquivBioSources  5,95
+#define ERR_SEQ_FEAT_MultipleEquivPublications  5,96
+#define ERR_SEQ_FEAT_BadFullLengthFeature  5,97
+#define ERR_SEQ_FEAT_RedundantFields  5,98
+#define ERR_SEQ_FEAT_CDSwithNoMRNAOverlap  5,99
+#define ERR_SEQ_FEAT_FeatureProductInconsistency  5,100
+#define ERR_SEQ_FEAT_ImproperBondLocation  5,101
+#define ERR_SEQ_FEAT_GeneXrefWithoutGene  5,102
+#define ERR_SEQ_FEAT_SeqFeatXrefProblem  5,103
+#define ERR_SEQ_FEAT_ProductFetchFailure  5,104
+#define ERR_SEQ_FEAT_SuspiciousGeneXref  5,105
+#define ERR_SEQ_FEAT_MissingTrnaAA  5,106
+#define ERR_SEQ_FEAT_CollidingFeatureIDs  5,107
+#define ERR_SEQ_FEAT_ExceptionProblem  5,108
+#define ERR_SEQ_FEAT_PolyAsignalNotRange  5,109
+#define ERR_SEQ_FEAT_OldLocusTagMismtach  5,110
+#define ERR_SEQ_FEAT_DuplicateGeneOntologyTerm  5,111
+#define ERR_SEQ_FEAT_InvalidInferenceValue  5,112
+#define ERR_SEQ_FEAT_HpotheticalProteinMismatch  5,113
+#define ERR_SEQ_FEAT_FeatureRefersToAccession  5,114
+#define ERR_SEQ_FEAT_SelfReferentialProduct  5,115
+#define ERR_SEQ_FEAT_ITSdoesNotAbutRRNA  5,116
+#define ERR_SEQ_FEAT_FeatureSeqIDCaseDifference  5,117
+#define ERR_SEQ_FEAT_FeatureLocationIsGi0  5,118
+#define ERR_SEQ_FEAT_GapFeatureProblem  5,119
+#define ERR_SEQ_FEAT_PseudoCdsHasProtXref  5,120
+#define ERR_SEQ_FEAT_ErroneousException  5,121
 #define ERR_SEQ_ALIGN  6,0
 #define ERR_SEQ_ALIGN_SeqIdProblem  6,1
 #define ERR_SEQ_ALIGN_StrandRev  6,2
