@@ -1,4 +1,4 @@
-/*  $Id: ncbi_connutil.c,v 6.72 2005/04/20 15:50:30 lavr Exp $
+/*  $Id: ncbi_connutil.c,v 6.108 2006/04/21 14:38:12 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -26,7 +26,7 @@
  * Author:  Denis Vakatov, Anton Lavrentiev
  *
  * File Description:
- *   Auxiliary API, mostly CONN-, URL-, and MIME-related
+ *   Auxiliary API, mostly CONN-, URL-/BASE64-, and MIME-related
  *   (see in "ncbi_connutil.h" for more details).
  *
  */
@@ -37,69 +37,94 @@
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
+#if defined(NCBI_OS_UNIX)
+#  ifndef NCBI_OS_SOLARIS
+#    include <limits.h>
+#  endif
+#  if defined(HAVE_GETPWUID)  ||  defined(HAVE_GETPWUID_R)
+#    include <pwd.h>
+#  endif
+#  include <unistd.h>
+#elif defined(NCBI_OS_MSWIN)
+#  if defined(_MSC_VER)  &&  (_MSC_VER > 1200)
+#    define WIN32_LEAN_AND_MEAN
+#  endif
+#  include <windows.h>
+#endif
 
 
-static const char* s_GetValue(const char* service, const char* param,
-                              char* value, size_t value_size,
-                              const char* def_value)
+extern const char* ConnNetInfo_GetValue(const char* service, const char* param,
+                                        char* value, size_t value_size,
+                                        const char* def_value)
 {
-    char        key[256];
-    char*       sec;
+    char        buf[256];
     const char* val;
+    char*       s;
 
-    if (!param  ||  !value  ||  value_size <= 0)
+    if (!value  ||  value_size <= 0)
         return 0;
     *value = '\0';
 
+    if (!param  ||  !*param)
+        return 0;
+
     if (service  &&  *service) {
         /* Service-specific inquiry */
-        if (strlen(service) + 1 + sizeof(DEF_CONN_REG_SECTION) +
-            strlen(param) + 1 > sizeof(key))
+        size_t slen = strlen(service);
+        size_t plen = strlen(param) + 1;
+        if (slen + 1 + sizeof(DEF_CONN_REG_SECTION) + plen > sizeof(buf))
             return 0;
+
         /* First, environment search for 'service_CONN_param' */
-        sprintf(key, "%s_" DEF_CONN_REG_SECTION "_%s", service, param);
-        strupr(key);
-        if ((val = getenv(key)) != 0)
+        s = (char*) memcpy(buf, service, slen) + slen;
+        *s++ = '_';
+        memcpy(s, DEF_CONN_REG_SECTION, sizeof(DEF_CONN_REG_SECTION) - 1);
+        s += sizeof(DEF_CONN_REG_SECTION) - 1;
+        *s++ = '_';
+        memcpy(s, param, plen);
+        if ((val = getenv(strupr(buf))) != 0)
             return strncpy0(value, val, value_size - 1);
+
         /* Next, search for 'CONN_param' in '[service]' registry section */
-        sprintf(key, DEF_CONN_REG_SECTION "_%s", param);
-        sec = key + strlen(key) + 1;
-        strupr(key);
-        strcpy(sec, service);
-        strupr(sec);
-        CORE_REG_GET(sec, key, value, value_size, 0);
+        buf[slen++] = '\0';
+        s = buf + slen;
+        CORE_REG_GET(buf, s, value, value_size, 0);
         if (*value)
             return value;
     } else {
         /* Common case. Form 'CONN_param' */
-        if (sizeof(DEF_CONN_REG_SECTION) + strlen(param) + 1 > sizeof(key))
+        size_t plen = strlen(param) + 1;
+        if (sizeof(DEF_CONN_REG_SECTION) + plen > sizeof(buf))
             return 0;
-        sprintf(key, DEF_CONN_REG_SECTION "_%s", param);
-        strupr(key);
+        s = buf;
+        memcpy(s, DEF_CONN_REG_SECTION, sizeof(DEF_CONN_REG_SECTION) - 1);
+        s += sizeof(DEF_CONN_REG_SECTION) - 1;
+        *s++ = '_';
+        memcpy(s, param, plen);
+        s = strupr(buf);
     }
 
     /* Environment search for 'CONN_param' */
-    if ((val = getenv(key)) != 0)
+    if ((val = getenv(s)) != 0)
         return strncpy0(value, val, value_size - 1);
 
     /* Last resort: Search for 'param' in default registry section */
-    strcpy(key, param);
-    strupr(key);
-    CORE_REG_GET(DEF_CONN_REG_SECTION, key, value, value_size, def_value);
-
+    s += sizeof(DEF_CONN_REG_SECTION);
+    CORE_REG_GET(DEF_CONN_REG_SECTION, s, value, value_size, def_value);
     return value;
 }
 
 
-/***********************************************************************
- *  EXTERNAL
- ***********************************************************************/
+
+/****************************************************************************
+ * ConnNetInfo API
+ */
 
 
 extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 {
 #define REG_VALUE(name, value, def_value) \
-    s_GetValue(service, name, value, sizeof(value), def_value)
+    ConnNetInfo_GetValue(service, name, value, sizeof(value), def_value)
 
     SConnNetInfo* info = (SConnNetInfo*) malloc(sizeof(*info) +
                                                 (service  &&  *service
@@ -117,6 +142,11 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     if (!SOCK_gethostbyaddr(0, info->client_host, sizeof(info->client_host)))
         SOCK_gethostname(info->client_host, sizeof(info->client_host));
 
+    /* Future extensions, clear up for now */
+    info->scheme  = eURL_Unspec;
+    info->user[0] = '\0';
+    info->pass[0] = '\0';
+
     /* dispatcher host name */
     REG_VALUE(REG_CONN_HOST, info->host, DEF_CONN_HOST);
 
@@ -179,6 +209,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
         (strcmp(str, "1") == 0  ||
          strcasecmp(str, "true") == 0  ||
          strcasecmp(str, "yes" ) == 0  ||
+         strcasecmp(str, "on"  ) == 0  ||
          strcasecmp(str, "some") == 0)) {
         info->debug_printout = eDebugPrintout_Some;
     } else if (*str  &&
@@ -193,21 +224,24 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     info->stateless = (*str  &&
                        (strcmp(str, "1") == 0  ||
                         strcasecmp(str, "true") == 0  ||
-                        strcasecmp(str, "yes" ) == 0));
+                        strcasecmp(str, "yes" ) == 0  ||
+                        strcasecmp(str, "on"  ) == 0));
 
     /* firewall mode? */
     REG_VALUE(REG_CONN_FIREWALL, str, DEF_CONN_FIREWALL);
     info->firewall = (*str  &&
                       (strcmp(str, "1") == 0  ||
                        strcasecmp(str, "true") == 0  ||
-                       strcasecmp(str, "yes" ) == 0));
+                       strcasecmp(str, "yes" ) == 0  ||
+                       strcasecmp(str, "on"  ) == 0));
 
     /* prohibit the use of local load balancer? */
     REG_VALUE(REG_CONN_LB_DISABLE, str, DEF_CONN_LB_DISABLE);
     info->lb_disable = (*str  &&
                         (strcmp(str, "1") == 0  ||
                          strcasecmp(str, "true") == 0  ||
-                         strcasecmp(str, "yes" ) == 0));
+                         strcasecmp(str, "yes" ) == 0  ||
+                         strcasecmp(str, "on"  ) == 0));
 
     /* user header (with optional '\r\n' added automagically) */
     REG_VALUE(REG_CONN_HTTP_USER_HEADER, str, DEF_CONN_HTTP_USER_HEADER);
@@ -240,10 +274,10 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 
 extern int/*bool*/ ConnNetInfo_AdjustForHttpProxy(SConnNetInfo* info)
 {
-    if (info->http_proxy_adjusted  ||  !*info->http_proxy_host)
+    if (!info  ||  info->http_proxy_adjusted  ||  !*info->http_proxy_host)
         return 0/*false*/;
 
-    if (strlen(info->host) + strlen(info->path) + 16 > sizeof(info->path)) {
+    if (strlen(info->host) + 16 + strlen(info->path) >= sizeof(info->path)) {
         CORE_LOG(eLOG_Error,
                  "[ConnNetInfo_AdjustForHttpProxy]  Adjusted path too long");
         assert(0);
@@ -251,11 +285,11 @@ extern int/*bool*/ ConnNetInfo_AdjustForHttpProxy(SConnNetInfo* info)
     }
 
     {{
-        char x_path[sizeof(info->path)];
+        char x_path[sizeof(info->host) + 16 + sizeof(info->path)];
         sprintf(x_path, "http://%s:%hu%s%s", info->host, info->port,
                 *info->path == '/' ? "" : "/", info->path);
         assert(strlen(x_path) < sizeof(x_path));
-        strcpy(info->path, x_path);
+        strncpy0(info->path, x_path, sizeof(info->path) - 1);
     }}
 
     assert(sizeof(info->host) >= sizeof(info->http_proxy_host));
@@ -301,8 +335,10 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
             if (sscanf(a, ":%hu%n", &port, &n) < 1 || a + n != s)
                 return 0/*failure*/;
             info->port = port;
-        } else
+        } else {
             a = s;
+            info->port = DEF_CONN_PORT;
+        }
         if ((size_t)(a - h) < sizeof(info->host)) {
             memcpy(info->host, h, (size_t)(a - h));
             info->host[(size_t)(a - h)] = '\0';
@@ -551,21 +587,27 @@ extern int/*bool*/ ConnNetInfo_AppendArg(SConnNetInfo* info,
                                          const char*   arg,
                                          const char*   val)
 {
-    const char* amp = "&";
+    size_t len, used;
 
     if (!arg || !*arg)
         return 1/*success*/;
-    if (!info->args[0])
-        amp = "";
-    if (strlen(info->args) + strlen(amp) + strlen(arg) +
-        (val ? 1 + strlen(val) : 0) >= sizeof(info->args))
+
+    used = strlen(info->args);
+    len  = strlen(arg);
+    
+    if (used + (used ? 1/*&*/ : 0) + len +
+        (val && *val ? 1/*=*/ + strlen(val) : 0) >= sizeof(info->args)) {
         return 0/*failure*/;
-    strcat(info->args, amp);
-    strcat(info->args, arg);
-    if (!val || !*val)
-        return 1/*success*/;
-    strcat(info->args, "=");
-    strcat(info->args, val);
+    }
+
+    if (used)
+        info->args[used++] = '&';
+    strcpy(info->args + used, arg);
+    if (val && *val) {
+        used += len;
+        info->args[used++] = '=';
+        strcpy(info->args + used, val);
+    }
     return 1/*success*/;
 }
 
@@ -574,25 +616,27 @@ extern int/*bool*/ ConnNetInfo_PrependArg(SConnNetInfo* info,
                                           const char*   arg,
                                           const char*   val)
 {
-    char amp = '&';
-    size_t off;
+    size_t len, off, used;
 
     if (!arg || !*arg)
         return 1/*success*/;
-    if (!info->args[0])
-        amp = '\0';
-    off = strlen(arg) + (val && *val ? 1 + strlen(val) : 0) + (amp ? 1 : 0);
-    if (off + strlen(info->args) >= sizeof(info->args))
+
+    used = strlen(info->args);
+    len  = strlen(arg);
+    off  = len + (val && *val ? 1/*=*/ + strlen(val) : 0) + (used? 1/*&*/ : 0);
+
+    if (off + used >= sizeof(info->args))
         return 0/*failure*/;
-    if (amp)
-        memmove(&info->args[off], info->args, strlen(info->args) + 1);
+
+    if (used)
+        memmove(info->args + off, info->args, used + 1);
     strcpy(info->args, arg);
     if (val && *val) {
-        strcat(info->args, "=");
-        strcat(info->args, val);
+        info->args[len++] = '=';
+        strcpy(info->args + len, val);
     }
-    if (amp)
-        info->args[off - 1] = amp;
+    if (used)
+        info->args[off - 1] = '&';
     return 1/*success*/;
 }
 
@@ -610,7 +654,7 @@ extern void ConnNetInfo_DeleteArg(SConnNetInfo* info,
         if (*a == '&')
             a++;
         arglen = strcspn(a, "&");
-        if (arglen < argnamelen || strncmp(a, arg, argnamelen) != 0 ||
+        if (arglen < argnamelen || strncasecmp(a, arg, argnamelen) != 0 ||
             (a[argnamelen] && a[argnamelen] != '=' && a[argnamelen] != '&'))
             continue;
         if (a[arglen]) {
@@ -626,13 +670,37 @@ extern void ConnNetInfo_DeleteArg(SConnNetInfo* info,
 }
 
 
+extern void ConnNetInfo_DeleteAllArgs(SConnNetInfo* info,
+                                      const char*   args)
+{
+    char* temp;
+    char* arg;
+    if (!args || !*args || !(temp = strdup(args))) {
+        if (args && *args)
+            *info->args = '\0';
+        return;
+    }
+    arg = temp;
+    while (*arg) {
+        char* end = strchr(arg, '&');
+        if (!end)
+            end = arg + strlen(arg);
+        else
+            *end++ = '\0';
+        ConnNetInfo_DeleteArg(info, arg);
+        arg = end;
+    }
+    free(temp);
+}
+
+
 extern int/*bool*/ ConnNetInfo_PreOverrideArg(SConnNetInfo* info,
                                               const char*   arg,
                                               const char*   val)
 {
     if (!arg || !*arg)
         return 1/*success*/;
-    ConnNetInfo_DeleteArg(info, arg);
+    ConnNetInfo_DeleteAllArgs(info, arg);
     return ConnNetInfo_PrependArg(info, arg, val);
 }
 
@@ -643,11 +711,94 @@ extern int/*bool*/ ConnNetInfo_PostOverrideArg(SConnNetInfo* info,
 {
     if (!arg || !*arg)
         return 1/*success*/;
-    ConnNetInfo_DeleteArg(info, arg);
+    ConnNetInfo_DeleteAllArgs(info, arg);
     return ConnNetInfo_AppendArg(info, arg, val);
 }
 
 
+static int/*bool*/ s_IsSufficientAddress(const char* addr)
+{
+    size_t i, len = strlen(addr);
+    int dots = 0, isip = 1;
+    const char* dot = 0;
+
+    for (i = 0; i < len; i++) {
+        if (addr[i] == '.') {
+            ++dots;
+            if (i  &&  dots <= 3) {
+                if (isip) {
+                    size_t n = (size_t)(&addr[i] - (dot ? dot : addr - 1));
+                    if (n <= 1  ||  n > 4)
+                        isip = 0;
+                }
+            } else
+                isip = 0;
+            dot = &addr[i];
+        } else if (!isdigit((unsigned char) addr[i]))
+            isip = 0;
+    }
+    if (dots < 3)
+        isip = 0;
+    if (dot == &addr[len - 1])
+        --dots;
+    return isip ? 1 : dots < 2 ? 0 : 1;
+}
+
+
+static const char* s_ClientAddress(const char* client_host)
+{
+    unsigned int ip;
+    char addr[64];
+    char* s;
+
+    if (!client_host  ||  s_IsSufficientAddress(client_host)        ||
+        !(ip = SOCK_gethostbyname(*client_host ? client_host : 0))  ||
+        SOCK_ntoa(ip, addr, sizeof(addr)) != 0                      ||
+        !(s = (char*) malloc(strlen(client_host) + strlen(addr) + 3))) {
+        return client_host;
+    }
+    sprintf(s, "%s(%s)", client_host, addr);
+    return s;
+}
+
+
+extern int/*bool*/ ConnNetInfo_SetupStandardArgs(SConnNetInfo* info)
+{
+    static const char service[]  = "service";
+    static const char address[]  = "address";
+    static const char platform[] = "platform";
+    const char* arch;
+    const char* addr;
+
+    if (!info)
+        return 0/*failed*/;
+    if (!info->service  ||  !*info->service) {
+        assert(0);
+        return 0/*failed*/;
+    }
+    /* Dispatcher CGI arguments (sacrifice some if they all do not fit) */
+    if (!(arch = CORE_GetPlatform())  ||  !*arch)
+        ConnNetInfo_DeleteArg(info, platform);
+    else
+        ConnNetInfo_PreOverrideArg(info, platform, arch);
+    if (!(addr = s_ClientAddress(info->client_host))  ||  !*addr)
+        ConnNetInfo_DeleteArg(info, address);
+    else
+        ConnNetInfo_PreOverrideArg(info, address, addr);
+    if (addr != info->client_host)
+        free((void*) addr);
+    if (!ConnNetInfo_PreOverrideArg(info, service, info->service)) {
+        ConnNetInfo_DeleteArg(info, platform);
+        if (!ConnNetInfo_PreOverrideArg(info, service, info->service)) {
+            ConnNetInfo_DeleteArg(info, address);
+            if (!ConnNetInfo_PreOverrideArg(info, service, info->service))
+                return 0/*failed*/;
+        }
+    }
+    return 1/*succeeded*/;
+}
+
+
 extern SConnNetInfo* ConnNetInfo_Clone(const SConnNetInfo* info)
 {
     SConnNetInfo* x_info;
@@ -759,6 +910,12 @@ extern void ConnNetInfo_Destroy(SConnNetInfo* info)
 }
 
 
+
+/****************************************************************************
+ * URL_Connect
+ */
+
+
 extern SOCK URL_Connect
 (const char*     host,
  unsigned short  port,
@@ -783,11 +940,12 @@ extern SOCK URL_Connect
     char        buffer[80];
     size_t      headersize;
     const char* x_args = 0;
+    size_t      user_hdr_len = user_hdr  &&  *user_hdr ? strlen(user_hdr) : 0;
     const char* x_req_r; /* "POST "/"GET " */
 
     /* check the args */
-    if (!host  ||  !*host  ||  !port  ||  !path  ||  !*path  ||
-        (user_hdr  &&  *user_hdr  &&  user_hdr[strlen(user_hdr)-1] != '\n')) {
+    if (!host  ||  !*host  ||  !port  ||
+        (user_hdr  &&  *user_hdr  &&  user_hdr[user_hdr_len - 1] != '\n')) {
         CORE_LOG(eLOG_Error, "[URL_Connect]  Bad arguments");
         assert(0);
         return 0/*error*/;
@@ -839,7 +997,7 @@ extern SOCK URL_Connect
     if (/* {POST|GET} <path>?<args> HTTP/1.0\r\n */
         !BUF_Write(&buf, x_req_r, strlen(x_req_r))            ||
         !BUF_Write(&buf, path,    strlen(path))               ||
-        (x_args
+        (x_args  &&  *x_args
          &&  (!BUF_Write(&buf, X_REQ_Q, sizeof(X_REQ_Q) - 1)  ||
               !BUF_Write(&buf, x_args,  strlen(x_args))))     ||
         !BUF_Write(&buf,       X_REQ_E, sizeof(X_REQ_E) - 1)  ||
@@ -850,8 +1008,8 @@ extern SOCK URL_Connect
         !BUF_Write(&buf, "\r\n", 2)                           ||
 
         /* <user_header> */
-        (user_hdr
-         &&  !BUF_Write(&buf, user_hdr, strlen(user_hdr)))    ||
+        (user_hdr  &&  *user_hdr
+         &&  !BUF_Write(&buf, user_hdr, user_hdr_len))        ||
 
         /* Content-Length: <content_length>\r\n\r\n */
         (req_method != eReqMethod_Get
@@ -905,8 +1063,12 @@ extern SOCK URL_Connect
 }
 
 
-/* Code for the "*_StripToPattern()" functions
+
+/****************************************************************************
+ * StripToPattern()
  */
+
+
 typedef EIO_Status (*FDoIO)
      (void*     stream,
       void*     buf,
@@ -1077,14 +1239,15 @@ static EIO_Status s_BUF_IO
  size_t*   n_read,
  EIO_Event what)
 {
+    BUF b;
     switch (what) {
     case eIO_Read:
         *n_read = BUF_Read((BUF) stream, buf, size);
         return *n_read ? eIO_Success : eIO_Closed;
     case eIO_Write:
         assert(stream);
-        return BUF_PushBack((BUF*) &stream, buf, size)
-            ? eIO_Success : eIO_Unknown;
+        b = (BUF) stream;
+        return BUF_PushBack(&b, buf, size) ? eIO_Success : eIO_Unknown;
     default:
         break;
     }
@@ -1103,6 +1266,13 @@ extern EIO_Status BUF_StripToPattern
 }
 
 
+
+
+/****************************************************************************
+ * URL- and BASE64- Encoding/Decoding
+ */
+
+
 /* Return integer (0..15) corresponding to the "ch" as a hex digit
  * Return -1 on error
  */
@@ -1117,6 +1287,7 @@ static int s_HexChar(char ch)
     return -1;
 }
 
+
 /* The URL-encoding table
  */
 static const char s_Encode[256][4] = {
@@ -1153,6 +1324,7 @@ static const char s_Encode[256][4] = {
     "%F0", "%F1", "%F2", "%F3", "%F4", "%F5", "%F6", "%F7",
     "%F8", "%F9", "%FA", "%FB", "%FC", "%FD", "%FE", "%FF"
 };
+
 #define VALID_URL_SYMBOL(ch)  (s_Encode[(unsigned char)ch][0] != '%')
 
 
@@ -1259,7 +1431,6 @@ extern void URL_Encode
 }
 
 
-
 extern void BASE64_Encode
 (const void* src_buf,
  size_t      src_size,
@@ -1279,9 +1450,9 @@ extern void BASE64_Encode
     unsigned char* src = (unsigned char*) src_buf;
     unsigned char* dst = (unsigned char*) dst_buf;
     size_t len = 0, i = 0, j = 0;
+    unsigned char temp = 0, c;
     unsigned char shift = 2;
-    unsigned char temp = 0;
-    if (!max_src) {
+    if (!max_src  ||  !src_size) {
         *src_read    = 0;
         *dst_written = 0;
         if (dst_size > 0) {
@@ -1292,33 +1463,37 @@ extern void BASE64_Encode
     if (src_size > max_src) {
         src_size = max_src;
     }
+    c = src[0];
     for (;;) {
-        unsigned char c = i < src_size ? src[i] : 0;
         unsigned char bits = (c >> shift) & 0x3F;
-        assert((temp | bits) < sizeof(syms) - 1);
-        dst[j++] = syms[temp | bits];
-        if (max_len  &&  ++len >= max_len) {
+        if (max_len  &&  len >= max_len) {
             dst[j++] = '\n';
             len = 0;
         }
+        assert((size_t)(temp | bits) < sizeof(syms) - 1);
+        dst[j++] = syms[temp | bits];
+        len++;
         if (i >= src_size) {
             break;
         }
         shift += 2;
         shift &= 7;
+        temp = (c << (8 - shift)) & 0x3F;
         if (shift) {
+            c = ++i < src_size ? src[i] : 0;
+        } else if (i + 1 == src_size) {
             i++;
         }
-        temp = (c << (8 - shift)) & 0x3F;
     }
     assert(j <= dst_size);
     *src_read = i;
     for (i = 0; i < (3 - src_size % 3) % 3; i++) {
-        dst[j++] = '=';
-        if (max_len  &&  ++len >= max_len) {
+        if (max_len  &&  len >= max_len) {
             dst[j++] = '\n';
             len = 0;
         }
+        dst[j++] = '=';
+        len++;
     }
     assert(j <= dst_size);
     *dst_written = j;
@@ -1346,7 +1521,8 @@ extern int/*bool*/ BASE64_Decode
         return 0/*false*/;
     }
     for (;;) {
-        unsigned char c = i < src_size ? src[i++] : '=';
+        int/*bool*/  ok = i < src_size ? 1/*true*/ : 0/*false*/;
+        unsigned char c = ok ? src[i++] : '=';
         if (c == '=') {
             c  = 64; /*end*/
         } else if (c >= 'A'  &&  c <= 'Z') {
@@ -1367,7 +1543,10 @@ extern int/*bool*/ BASE64_Decode
         if (!(++k & 3)  ||  c == 64) {
             if (c == 64) {
                 if (k < 2) {
-                    --i;
+                    if (ok) {
+                        /* pushback leading '=' */
+                        --i;
+                    }
                     break;
                 }
                 switch (k) {
@@ -1381,12 +1560,19 @@ extern int/*bool*/ BASE64_Decode
                     temp >>= 8;
                     break;
                 default:
+                    assert(0);
                     break;
                 }
-                for (l = k; l < 4; l++) {
-                    if (i < src_size  &&  src[i] == '=') {
-                        i++;
-                    }
+                l = 4 - k;
+                while (l > 0) {
+                    /* eat up '='-padding */
+                    if (i >= src_size)
+                        break;
+                    if (src[i] == '=')
+                        l--;
+                    else if (src[i] != '\r'  &&  src[i] != '\n')
+                        break;
+                    i++;
                 }
             } else {
                 k = 0;
@@ -1412,7 +1598,7 @@ extern int/*bool*/ BASE64_Decode
     }
     *src_read    = i;
     *dst_written = j;
-    return i ? 1/*true*/ : 0/*false*/;
+    return i  &&  j ? 1/*true*/ : 0/*false*/;
 }
 
 
@@ -1421,6 +1607,7 @@ extern int/*bool*/ BASE64_Decode
  * NCBI-specific MIME content type and sub-types
  */
 
+
 static const char* s_MIME_Type[eMIME_T_Unknown+1] = {
     "x-ncbi-data",
     "text",
@@ -1589,81 +1776,346 @@ extern int/*bool*/ MIME_ParseContentType
 
 
 /****************************************************************************
- * Reading and writing [host][:port] addresses
+ * Reading and writing [host][:port] addresses:  Deprecated here, use upcalls.
  */
 
 extern const char* StringToHostPort(const char*     str,
                                     unsigned int*   host,
                                     unsigned short* port)
 {
-    unsigned short p;
-    unsigned int h;
-    char abuf[256];
-    const char* s;
-    size_t alen;
-    int n = 0;
-
-    if (host)
-        *host = 0;
-    if (port)
-        *port = 0;
-    for (s = str; *s; s++) {
-        if (isspace((unsigned char)(*s)) || *s == ':')
-            break;
-    }
-    if ((alen = (size_t)(s - str)) > sizeof(abuf) - 1)
-        return str;
-    if (alen) {
-        strncpy0(abuf, str, alen);
-        if (!(h = SOCK_gethostbyname(abuf)))
-            return str;
-    } else
-        h = 0;
-    if (*s == ':') {
-        if (sscanf(++s, "%hu%n", &p, &n) < 1 ||
-            (s[n] && !isspace((unsigned char) s[n])))
-            return alen ? 0 : str;
-    } else
-        p = 0;
-    if (host)
-        *host = h;
-    if (port)
-        *port = p;
-    return s + n;
+    return SOCK_StringToHostPort(str, host, port);
 }
 
-
 extern size_t HostPortToString(unsigned int   host,
                                unsigned short port,
                                char*          buf,
                                size_t         buflen)
 {
-    char   x_buf[16/*sizeof("255.255.255.255")*/ + 8/*:port*/];
-    size_t n;
+    return SOCK_HostPortToString(host, port, buf, buflen);
+}
 
-    if (!buf || !buflen)
-        return 0;
-    if (!host)
-        *x_buf = 0;
-    else if (SOCK_ntoa(host, x_buf, sizeof(x_buf)) != 0) {
-        *buf = 0;
+
+
+/****************************************************************************
+ * CRC32
+ */
+
+
+/* Standard Ethernet/ZIP polynomial */
+#define CRC32_POLY 0x04C11DB7UL
+
+
+static unsigned int s_CRC32[256];
+
+static void s_CRC32_Init(void)
+{
+    size_t i;
+    if (s_CRC32[255])
+        return;
+    for (i = 0;  i < 256;  i++) {
+        unsigned int byteCRC = (unsigned int) i << 24;
+        int j;
+        for (j = 0;  j < 8;  j++) {
+            if (byteCRC & 0x80000000UL)
+                byteCRC = (byteCRC << 1) ^ CRC32_POLY;
+            else
+                byteCRC = (byteCRC << 1);
+        }
+        s_CRC32[i] = byteCRC;
+    }
+}
+
+
+extern unsigned int CRC32_Update(unsigned int checksum,
+                                 const void *ptr, size_t count)
+{
+    const unsigned char* str = (const unsigned char*) ptr;
+    size_t j;
+
+    s_CRC32_Init();
+    for (j = 0;  j < count;  j++) {
+        size_t i = ((checksum >> 24) ^ *str++) & 0xFF;
+        checksum <<= 8;
+        checksum  ^= s_CRC32[i];
+    }
+    return checksum;
+}
+
+
+
+/****************************************************************************
+ * CONNUTIL_GetUsername
+ */
+
+
+extern const char* CONNUTIL_GetUsername(char* buf, size_t bufsize)
+{
+#if defined(NCBI_OS_UNIX)
+#  if !defined(NCBI_OS_SOLARIS)  &&  defined(HAVE_GETLOGIN_R)
+#    ifndef LOGIN_NAME_MAX
+#      ifdef _POSIX_LOGIN_NAME_MAX
+#        define LOGIN_NAME_MAX _POSIX_LOGIN_NAME_MAX
+#      else
+#        define LOGIN_NAME_MAX 256
+#      endif
+#    endif
+    char loginbuf[LOGIN_NAME_MAX + 1];
+#  endif
+    struct passwd* pw;
+#  if !defined(NCBI_OS_SOLARIS)  &&  defined(HAVE_GETPWUID_R)
+    struct passwd pwd;
+    char pwdbuf[256];
+#  endif
+#elif defined(NCBI_OS_MSWIN)
+    char  loginbuf[256 + 1];
+    DWORD loginbufsize = sizeof(loginbuf) - 1;
+#endif
+    const char* login;
+
+    assert(buf  &&  bufsize);
+
+#ifndef NCBI_OS_UNIX
+
+#  ifdef NCBI_OS_MSWIN
+    if (GetUserName(loginbuf, &loginbufsize)) {
+        assert(loginbufsize < sizeof(loginbuf));
+        loginbuf[loginbufsize] = '\0';
+        strncpy0(buf, loginbuf, bufsize - 1);
+        return buf;
+    }
+    if ((login = getenv("USERNAME")) != 0) {
+        strncpy0(buf, login, bufsize - 1);
+        return buf;
+    }
+#  endif
+
+#else /*!NCBI_OS_UNIX*/
+
+#  if defined(NCBI_OS_SOLARIS)  ||  !defined(HAVE_GETLOGIN_R)
+    /* NB:  getlogin() is MT-safe on Solaris, yet getlogin_r() comes in two
+     * flavors that differ only in return type, so to make things simpler,
+     * use plain getlogin() here */
+#    ifndef NCBI_OS_SOLARIS
+    CORE_LOCK_WRITE;
+#    endif
+    if ((login = getlogin()) != 0)
+        strncpy0(buf, login, bufsize - 1);
+#    ifndef NCBI_OS_SOLARIS
+    CORE_UNLOCK;
+#    endif
+    if (login)
+        return buf;
+#  else
+    if (getlogin_r(loginbuf, sizeof(loginbuf) - 1) == 0) {
+        loginbuf[sizeof(loginbuf) - 1] = '\0';
+        strncpy0(buf, loginbuf, bufsize - 1);
+        return buf;
+    }
+#  endif
+
+#  if defined(NCBI_OS_SOLARIS)  ||  \
+    (!defined(HAVE_GETPWUID_R)  &&  defined(HAVE_GETPWUID))
+    /* NB:  getpwuid() is MT-safe on Solaris, so use it here, if available */
+#  ifndef NCBI_OS_SOLARIS
+    CORE_LOCK_WRITE;
+#  endif
+    if ((pw = getpwuid(getuid())) != 0  &&  pw->pw_name)
+        strncpy0(buf, pw->pw_name, bufsize - 1);
+#  ifndef NCBI_OS_SOLARIS
+    CORE_UNLOCK;
+#  endif
+    if (pw  &&  pw->pw_name)
+        return buf;
+#  elif defined(HAVE_GETPWUID_R)
+#    if   HAVE_GETPWUID_R == 4
+    /* obsolete but still existent */
+    pw = getpwuid_r(getuid(), &pwd, pwdbuf, sizeof(pwdbuf));
+#    elif HAVE_GETPWUID_R == 5
+    /* POSIX-conforming */
+    if (getpwuid_r(getuid(), &pwd, pwdbuf, sizeof(pwdbuf), &pw) != 0)
+        pw = 0;
+#    else
+#      error "Unknown value of HAVE_GETPWUID_R, 4 or 5 expected."
+#    endif
+    if (pw  &&  pw->pw_name) {
+        assert(pw == &pwd);
+        strncpy0(buf, pw->pw_name, bufsize - 1);
+        return buf;
+    }
+#  endif /*HAVE_GETPWUID_R*/
+
+#endif /*!NCBI_OS_UNIX*/
+
+    /* last resort */
+    if (!(login = getenv("USER"))  &&  !(login = getenv("LOGNAME"))) {
+        buf[0] = '\0';
         return 0;
     }
-    n = strlen(x_buf);
-    if (port || !host)
-        n += sprintf(x_buf + n, ":%hu", port);
-    assert(n < sizeof(x_buf));
-    if (n >= buflen)
-        n = buflen - 1;
-    memcpy(buf, x_buf, n);
-    buf[n] = 0;
-    return n;
+    strncpy0(buf, login, bufsize - 1);
+    return buf;
+}
+
+
+
+/****************************************************************************
+ * Page size granularity
+ * See also at corelib's ncbi_system.cpp::GetVirtualMemoryPageSize().
+ */
+
+size_t CONNUTIL_GetVMPageSize(void)
+{
+    static size_t ps = 0;
+
+    if (!ps) {
+#if defined(NCBI_OS_MSWIN)
+        SYSTEM_INFO si;
+        GetSystemInfo(&si); 
+        ps = (size_t) si.dwAllocationGranularity;
+#elif defined(NCBI_OS_UNIX) 
+#  if   defined(_SC_PAGESIZE)
+#    define NCBI_SC_PAGESIZE _SC_PAGESIZE
+#  elif defined(_SC_PAGE_SIZE)
+#    define NCBI_SC_PAGESIZE _SC_PAGE_SIZE
+#  elif defined(NCBI_SC_PAGESIZE)
+#    undef  NCBI_SC_PAGESIZE
+#  endif
+#  ifndef   NCBI_SC_PAGESIZE
+        long x = 0;
+#  else
+        long x = sysconf(NCBI_SC_PAGESIZE);
+#    undef  NCBI_SC_PAGESIZE
+#  endif
+        if (x <= 0) {
+#  ifdef HAVE_GETPAGESIZE
+            if ((x = getpagesize()) <= 0)
+                return 0;
+#  else
+            return 0;
+#  endif
+        }
+        ps = (size_t) x;
+#endif /*OS_TYPE*/
+    }
+    return ps;
 }
 
 
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_connutil.c,v $
+ * Revision 6.108  2006/04/21 14:38:12  lavr
+ * ConnNetInfo_GetValue() simplified and sped up considerably
+ *
+ * Revision 6.107  2006/04/21 01:37:17  lavr
+ * SConnNetInfo::lb_disable reinstated along with LB_DISABLE reg/env key
+ *
+ * Revision 6.105  2006/04/20 13:58:32  lavr
+ * Registry keys for new switching scheme for service mappers;
+ * Registry keys for LOCAL service mappers;
+ * ConnNetInfo_GetValue() exported
+ *
+ * Revision 6.104  2006/04/19 02:12:06  lavr
+ * Call DeleteAllArgs when doing argument overrides
+ *
+ * Revision 6.103  2006/04/19 01:38:48  lavr
+ * ConnNetInfo_{Append|Prepend}Arg sped-up considerably
+ *
+ * Revision 6.102  2006/04/11 13:51:23  lavr
+ * URL_Connect():  Relax on parameter checking
+ *
+ * Revision 6.101  2006/03/19 01:40:17  lavr
+ * Yet another fix for s_IsSufficientAddress()
+ *
+ * Revision 6.100  2006/03/17 16:42:33  lavr
+ * BUF_StripToPattern(): Avoid type-punned ptr (and keep off GCC warning)
+ *
+ * Revision 6.99  2006/03/16 20:04:02  lavr
+ * s_IsSufficientAddress():  IP recognition bug fixed
+ *
+ * Revision 6.98  2006/03/07 15:37:19  lavr
+ * CONNUTIL_GetUsername(): Do not define loginbuf, if it's going to be unused
+ *
+ * Revision 6.97  2006/02/23 17:42:36  lavr
+ * CONNUTIL_GetVMPageSize(): use sysctl() first (fallback to getpagesize())
+ *
+ * Revision 6.96  2006/02/23 15:23:03  lavr
+ * +CONNUTIL_GetVMPageSize() [merely copy of what corelib/ncbi_system.cpp has]
+ *
+ * Revision 6.95  2006/02/14 15:50:22  lavr
+ * Introduce and use CONN_TRACE (via CORE_TRACE) -- NOP in Release mode
+ *
+ * Revision 6.94  2006/02/01 16:24:24  lavr
+ * CONNUTIL_GetUsername(): '\0'-terminate returned result of getlogin_r()
+ *
+ * Revision 6.93  2006/02/01 00:54:45  ucko
+ * One more fix to CONNUTIL_GetUsername: favor LOGIN_NAME_MAX over
+ * _POSIX_LOGIN_NAME_MAX, which tends to be smaller, and may not be
+ * defined at all on some BSDish systems, and fall back to 256 if neither
+ * is defined.
+ *
+ * Revision 6.92  2006/01/31 20:38:12  lavr
+ * CONNUTIL_GetUsername():  Pass pointer to buffer size in GetUserName()
+ *
+ * Revision 6.91  2006/01/31 20:29:24  lavr
+ * CONNUTIL_GetUsername():  use strncpy0 everywhere
+ *
+ * Revision 6.90  2006/01/31 20:26:46  lavr
+ * #include <pwd.h> only us getpwuid[_r]() is known to exist
+ *
+ * Revision 6.89  2006/01/31 19:32:55  lavr
+ * Use GetUserName() as provided by <winbase.h>
+ *
+ * Revision 6.88  2006/01/31 19:24:40  lavr
+ * CONNUTIL_GetUsername():  Clean ASCII username on MS-Windows
+ *
+ * Revision 6.87  2006/01/31 19:14:28  lavr
+ * CONNUTIL_GetUsername():  MS-Windows-specific username discovery
+ *
+ * Revision 6.86  2006/01/31 17:22:55  lavr
+ * CONNUTIL_GetUsername(): Consider USERNAME environment on Windows only
+ *
+ * Revision 6.85  2006/01/31 17:11:53  lavr
+ * MT-safe (as much as possible) implementation of CONNUTIL_GetUsername()
+ *
+ * Revision 6.84  2006/01/30 18:03:04  lavr
+ * Fix CONNUTIL_GetUsername() to return "buf" (instead of constant 0)
+ *
+ * Revision 6.83  2006/01/27 17:08:56  lavr
+ * Spell CONNUTIL_GetUsername() this way
+ *
+ * Revision 6.82  2006/01/27 17:00:52  lavr
+ * New CONNUTIL_GetUsername();  StringHostToPort() and HostPortToString()
+ * to remain obsoleted and upcall standardized SOCK_...() replacements
+ *
+ * Revision 6.81  2006/01/11 20:20:05  lavr
+ * -UTIL_ClientAddress()
+ * +ConnNetInfo_DeleteAllArgs()
+ * +ConnNetInfo_SetupStandardArgs()
+ *
+ * Revision 6.80  2006/01/11 16:25:02  lavr
+ * +UTIL_ClientAddress()
+ *
+ * Revision 6.79  2005/11/29 21:32:31  lavr
+ * Reserve SConnNetInfo::scheme, user, and pass for future use
+ *
+ * Revision 6.78  2005/11/29 19:54:49  lavr
+ * +CRC32 API
+ *
+ * Revision 6.77  2005/08/18 19:00:13  lavr
+ * Fix finishing-up bug in BASE64_Decode()
+ *
+ * Revision 6.76  2005/06/08 20:42:41  lavr
+ * Buffer size better adjustment in ConnNetInfo_AdjustForProxy()
+ *
+ * Revision 6.75  2005/06/08 16:59:38  lavr
+ * ConnNetInfo_ParseURL(): Use default port for absolute URL
+ *
+ * Revision 6.74  2005/05/13 21:12:23  lavr
+ * BASE64_Encode(): fix a critical encoding bug (stray ending char)
+ *
+ * Revision 6.73  2005/05/02 16:03:24  lavr
+ * Better assert() in BASE64_Encode()
+ *
  * Revision 6.72  2005/04/20 15:50:30  lavr
  * ConnNetInfo_Create(): Allow zero timeout
  * URL_Connect(): Automagically figure out ANY method based upon body size
