@@ -1,4 +1,4 @@
-/*  $Id: ncbi_file_connector.c,v 6.12 2003/05/31 05:14:56 lavr Exp $
+/*  $Id: ncbi_file_connector.c,v 6.14 2006/01/11 20:21:22 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 
 #include "ncbi_ansi_ext.h"
 #include <connect/ncbi_file_connector.h>
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -125,21 +126,20 @@ static EIO_Status s_VT_Open
         mode = "ab";  break;
     }
 
-    xxx->fout = fopen(xxx->out_file_name, mode);
-    if ( !xxx->fout ) {
+    if (!xxx->out_file_name  ||
+        !(xxx->fout = fopen(xxx->out_file_name, mode))) {
         return eIO_Unknown;
     }
 
     /* open file for input */
-    xxx->finp = fopen(xxx->inp_file_name, "rb");
-    if ( !xxx->finp ) {
+    if (!xxx->inp_file_name  ||
+        !(xxx->finp = fopen(xxx->inp_file_name, "rb"))) {
         fclose(xxx->fout);
         xxx->fout = 0;
         return eIO_Unknown;
     }
 
-    /* Due to the shortage of portable 'fseek' call
-     * ignore read/write positions so far;
+    /* Due to shortage of portable 'fseek' call ignore positioning for now;
      * only 0/EOF are in use for writing, and only 0 for reading
      */
     return eIO_Success;
@@ -299,8 +299,8 @@ extern CONNECTOR FILE_CreateConnectorEx
  const char*          out_file_name,
  const SFileConnAttr* attr)
 {
-    CONNECTOR       ccc = (SConnector*) malloc(sizeof(SConnector));
-    SFileConnector* xxx = (SFileConnector*) malloc(sizeof(SFileConnector));
+    CONNECTOR       ccc = (SConnector*)     malloc(sizeof(SConnector));
+    SFileConnector* xxx = (SFileConnector*) malloc(sizeof(*xxx));
 
     /* initialize internal data structures */
     xxx->inp_file_name = strdup(inp_file_name);
@@ -324,6 +324,12 @@ extern CONNECTOR FILE_CreateConnectorEx
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_file_connector.c,v $
+ * Revision 6.14  2006/01/11 20:21:22  lavr
+ * Uniform creation/fill-up of connector structures
+ *
+ * Revision 6.13  2005/04/20 18:15:59  lavr
+ * +<assert.h>
+ *
  * Revision 6.12  2003/05/31 05:14:56  lavr
  * Add ARGSUSED where args are meant to be unused
  *
