@@ -1,4 +1,4 @@
-static char const rcsid[] = "$Id: copymat.c,v 6.41 2005/02/14 14:11:55 camacho Exp $";
+static char const rcsid[] = "$Id: copymat.c,v 6.44 2005/12/22 14:22:19 papadopo Exp $";
 
 /*
 * ===========================================================================
@@ -36,6 +36,15 @@ Contents: main routines for copymatrices program to convert
 score matrices output by makematrices into a single byte-encoded file.
    
 $Log: copymat.c,v $
+Revision 6.44  2005/12/22 14:22:19  papadopo
+change signature of BLAST_FillLookupTableOptions
+
+Revision 6.43  2005/12/20 15:36:39  papadopo
+change name of structure field
+
+Revision 6.42  2005/05/20 18:57:51  camacho
+Update to use new signature to BLAST_FillLookupTableOptions
+
 Revision 6.41  2005/02/14 14:11:55  camacho
 Changes to use SBlastScoreMatrix
 
@@ -425,7 +434,7 @@ static Boolean RPSUpdateOffsets(BlastLookupTable *lookup)
     Int4 offset_diff;
 
     len = lookup->backbone_size;
-    offset_diff = lookup->wordsize - 1;
+    offset_diff = lookup->word_length - 1;
 
     /* Walk through table, copying info into mod_lt[] */
     for(index = 0; index < len; index++) {
@@ -608,8 +617,8 @@ Boolean RPSCreateLookupFile(ScoreRow *combinedMatrix, Int4 numProfiles,
     sbp = BlastScoreBlkNew(BLASTAA_SEQ_CODE, 1);
     RPSPsiMatrixAttach(sbp, posMatrix);
     LookupTableOptionsNew(eBlastTypeBlastp, &lookup_options);
-    BLAST_FillLookupTableOptions(lookup_options, eBlastTypeBlastp, FALSE, 
-	(Int4) (myargs[3].floatvalue*scalingFactor), myargs[4].intvalue, FALSE, TRUE);  /* add last arg for psi-blast?? */
+    BLAST_FillLookupTableOptions(lookup_options, eBlastTypePsiBlast, FALSE, 
+	(Int4) (myargs[3].floatvalue*scalingFactor), myargs[4].intvalue);
 
 
     BlastSeqLocNew(&lookup_segment, 0, all_length);
