@@ -1,6 +1,6 @@
-static char const rcsid[] = "$Id: rpsblast.c,v 6.75 2005/08/29 14:45:34 camacho Exp $";
+static char const rcsid[] = "$Id: rpsblast.c,v 6.82 2006/04/26 12:47:48 madden Exp $";
 
-/* $Id: rpsblast.c,v 6.75 2005/08/29 14:45:34 camacho Exp $
+/* $Id: rpsblast.c,v 6.82 2006/04/26 12:47:48 madden Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,12 +31,34 @@ static char const rcsid[] = "$Id: rpsblast.c,v 6.75 2005/08/29 14:45:34 camacho
 *
 * Initial Version Creation Date: 12/14/1999
 *
-* $Revision: 6.75 $
+* $Revision: 6.82 $
 *
 * File Description:
 *         Main file for RPS BLAST program
 *
 * $Log: rpsblast.c,v $
+* Revision 6.82  2006/04/26 12:47:48  madden
+* Use SBlastMessage in place of Blast_Message
+*
+* Revision 6.81  2006/04/21 14:34:50  madden
+* BLAST_GetQuerySeqLoc prototype change
+*
+* Revision 6.80  2006/04/20 15:32:37  papadopo
+* if query IDs are actually used, verify that there are no duplicate IDs
+*
+* Revision 6.79  2006/01/23 16:44:06  papadopo
+* change signature of FillHitSavingOptions
+*
+* Revision 6.78  2006/01/10 20:44:10  madden
+* Use SBlastSeqalignArray
+*
+* Revision 6.77  2006/01/06 16:36:14  papadopo
+* 1. By default, log messages go to stderr and not a logfile
+* 2. Do not continue execution after a fatal error is encountered
+*
+* Revision 6.76  2005/12/22 14:22:19  papadopo
+* change signature of BLAST_FillLookupTableOptions
+*
 * Revision 6.75  2005/08/29 14:45:34  camacho
 * From Ilya Dondoshansky:
 * Retrieve mask_at_hash option from the SBlastOptions structure instead of
@@ -284,6 +306,8 @@ static char const rcsid[] = "$Id: rpsblast.c,v 6.75 2005/08/29 14:45:34 camacho
 #include <algo/blast/api/blast_format.h>
 #include <algo/blast/api/blast_api.h>
 #include <algo/blast/api/blast_seq.h>
+#include <algo/blast/api/blast_seqalign.h>
+#include <algo/blast/api/blast_message_api.h>
 
 #ifdef PURIFY
 #include "/am/purew/solaris2/new/../purify/purify-4.5-solaris2/purify.h"
@@ -393,7 +417,7 @@ static Args myargs[] = {
       "F", NULL, NULL, FALSE, 'T', ARG_BOOLEAN, 0.0, 0, NULL},
     /* OPT_LOGIFLE */
     {"Logfile name ",
-     "rpsblast.log", NULL,NULL,TRUE,'l',ARG_FILE_OUT, 0.0,0,NULL},
+     "stderr", NULL,NULL,TRUE,'l',ARG_FILE_OUT, 0.0,0,NULL},
     /* OPT_LCASE */
     {"Use lower case filtering of FASTA sequence",
      "F", NULL,NULL,TRUE,'U',ARG_BOOLEAN, 0.0,0,NULL},
@@ -426,12 +450,12 @@ s_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
    BlastScoringOptions* score_options = options->score_options;
    BlastEffectiveLengthsOptions* eff_len_options = options->eff_len_options;
    BlastDatabaseOptions* db_options = options->db_options;
+   Blast_Message *core_msg = NULL;
 
    BLAST_FillLookupTableOptions(lookup_options, kProgram, 
                                 FALSE, /* megablast */
                                 0,     /* default threshold */
-                                0,     /* default wordsize */
-                                FALSE);/* no variable wordsize */
+                                0);    /* default wordsize */
 
    BLAST_FillQuerySetUpOptions(query_setup_options, kProgram, 
                               myargs[OPT_FILTER].strvalue, 0);
@@ -460,7 +484,8 @@ s_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
                       MAX(myargs[OPT_NUM_DESC].intvalue, 
                           myargs[OPT_NUM_RESULTS].intvalue),
                       TRUE,
-                      0);
+                      0,        /* turn off culling */
+                      0);       /* min diag separation */
 
    if (myargs[OPT_SEARCHSP].floatvalue != 0) {
       eff_len_options->searchsp_eff = (Int8) myargs[OPT_SEARCHSP].floatvalue; 
@@ -476,7 +501,8 @@ s_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
    /* Validate the options. */
    status = BLAST_ValidateOptions(kProgram, ext_options, score_options, 
                                   lookup_options, word_options, hit_options, 
-                                  &sum_returns->error);
+                                  &core_msg);
+   sum_returns->error = Blast_MessageToSBlastMessage(core_msg, NULL, NULL, FALSE);
 
    return status;
 }
@@ -518,19 +544,16 @@ Int2 Main_New(void)
    EBlastProgramType program_number;
    char* dbname = NULL;
    FILE *infp;
-   Int2 ctr = 1;
-   Int4 letters_read;
+   Int4 ctr = 1;
    Int4 query_from = 0, query_to = 0;
    SBlastOptions* options = NULL;
-   SeqLoc* query_slp = NULL;
-   SeqAlign* seqalign = NULL;
    BlastFormattingInfo* format_info = NULL;
    Blast_SummaryReturn* sum_returns=Blast_SummaryReturnNew();
    Int4 num_queries = 0;
    SeqLoc* lcase_mask = NULL;
-   SeqLoc* filter_loc = NULL;
    const int kMaxConcatLength = 40000;
    Blast_SummaryReturn* full_sum_returns = NULL;
+   Boolean believe_query = (Boolean) myargs[OPT_BELIEVE_QUERY].intvalue;
 
    /* select protein or nucleotide query, and choose
       the appropriate program type */
@@ -551,12 +574,14 @@ Int2 Main_New(void)
    if (status) {
        options = SBlastOptionsFree(options);
        if (sum_returns->error) {
-           Blast_SummaryReturnsPostError(sum_returns);
+           SBlastMessageErrPost(sum_returns->error);
            sum_returns = Blast_SummaryReturnFree(sum_returns);
        }
        return -1;
    }
 
+   SBlastOptionsSetBelieveQuery(options, believe_query);
+
    /* initialize the database and then the RPS_specific data files */
 
    dbname = myargs[OPT_DB].strvalue;
@@ -592,37 +617,47 @@ Int2 Main_New(void)
                                    myargs[OPT_NUM_RESULTS].intvalue,
                                    (Boolean) myargs[OPT_HTML].intvalue,
                                    FALSE, (Boolean) myargs[OPT_SHOW_GI].intvalue,
-                                   (Boolean) myargs[OPT_BELIEVE_QUERY].intvalue);
+                                   believe_query);
 
    BLAST_PrintOutputHeader(format_info);
 
    /* Loop over sets of queries. */
    while (1) {
+       SBlastSeqalignArray* seqalign_arr=NULL;
+       SeqLoc* query_slp = NULL;
+       SeqLoc* filter_loc = NULL;
+       Int4 letters_read;
 
        if ((Boolean)myargs[OPT_LCASE].intvalue) {
            letters_read = 
                BLAST_GetQuerySeqLoc(infp, query_is_na, 0, kMaxConcatLength, 
                                     query_from, query_to, &lcase_mask,
                                     &query_slp, &ctr, &num_queries, 
-                                    myargs[OPT_BELIEVE_QUERY].intvalue, 0);
+                                    believe_query, 0);
        } else {
            letters_read = 
                BLAST_GetQuerySeqLoc(infp, query_is_na, 0, kMaxConcatLength,
                                     query_from, query_to, NULL, 
                                     &query_slp, &ctr, &num_queries, 
-                                    myargs[OPT_BELIEVE_QUERY].intvalue, 0);
+                                    believe_query, 0);
        }
        
        if (letters_read <= 0)
            break;
        
+       if (believe_query && BlastSeqlocsHaveDuplicateIDs(query_slp)) {
+          ErrPostEx(SEV_FATAL, 1, 0, 
+                  "Duplicate IDs detected; please ensure that "
+                  "all query sequence identifiers are unique");
+       }
+
        /* Call database search function. Pass NULL for tabular formatting 
         * structure pointer, because on-the-fly tabular formatting is not
         * allowed for RPS BLAST.
         */
        status = 
            Blast_DatabaseSearch(query_slp, dbname, lcase_mask, options, NULL, 
-                                &seqalign, &filter_loc, sum_returns);
+                                &seqalign_arr, &filter_loc, sum_returns);
 
        /* Free the lower case mask in SeqLoc form. */
        lcase_mask = Blast_ValNodeMaskListFree(lcase_mask);
@@ -634,7 +669,7 @@ Int2 Main_New(void)
        
        /* Post warning or error messages, no matter what the search status 
           was. */
-       Blast_SummaryReturnsPostError(sum_returns);
+       SBlastMessageErrPost(sum_returns->error);
 
        if (status != 0) {
            ErrPostEx(SEV_FATAL, 1, 0, "BLAST search failed");
@@ -646,19 +681,33 @@ Int2 Main_New(void)
        /* format results */
        
        if (myargs[OPT_ASNOUT].strvalue) {
-           AsnIoPtr asnout = AsnIoOpen(myargs[OPT_ASNOUT].strvalue, (char*)"w");
-           GenericSeqAlignSetAsnWrite(seqalign, asnout);
-           asnout = AsnIoClose(asnout);
+                   /* This just prints out the ASN.1 to a secondary file. */
+                   BlastFormattingInfo* asn_format_info = NULL;
+                   BlastFormattingInfoNew(eAlignViewAsnText, options,
+                              blast_program, dbname,
+                              myargs[OPT_ASNOUT].strvalue, &asn_format_info);
+
+                   BlastFormattingInfoSetUpOptions(asn_format_info,
+                                       myargs[OPT_NUM_DESC].intvalue,
+                                       myargs[OPT_NUM_DESC].intvalue,
+                                       FALSE,
+                                       FALSE,
+                                       FALSE,
+                                       TRUE);
+                   status =
+                       BLAST_FormatResults(seqalign_arr, num_queries, query_slp,
+                                   NULL, asn_format_info, sum_returns);
+                   asn_format_info = BlastFormattingInfoFree(asn_format_info);
        }
        
        status = 
-           BLAST_FormatResults(seqalign, num_queries, query_slp, filter_loc,
+           BLAST_FormatResults(seqalign_arr, num_queries, query_slp, filter_loc,
                                format_info, sum_returns);
        
        /* finish cleanup */
        filter_loc = Blast_ValNodeMaskListFree(filter_loc);
-       seqalign = SeqAlignSetFree(seqalign);
        query_slp = SeqLocSetFree(query_slp);
+       seqalign_arr = SBlastSeqalignArrayFree(seqalign_arr);
        
        /* Update the cumulative summary returns structure and clean the returns
           substructures for the current search iteration. */
@@ -1187,11 +1236,11 @@ Int2 Main(void)
     if (!GetArgs(buf, NUM_ARGS, myargs))
         return 1;
     
-    if ( !ErrSetLog (myargs[OPT_LOGFILE].strvalue) ) { /* Logfile */
-        ErrShow();
-        return 1;
-    } else {
-        ErrSetOpts (ERR_CONTINUE, ERR_LOG_ON);
+    if (strcmp(myargs[OPT_LOGFILE].strvalue, "stderr") != 0) {
+       if ( !ErrSetLog (myargs[OPT_LOGFILE].strvalue) ) { /* Logfile */
+          ErrShow();
+          return 1;
+       }
     }
     
     UseLocalAsnloadDataAndErrMsg ();
