@@ -32,8 +32,14 @@ Contents: prototypes for "private" BLAST functions, these should not be called
 
 ******************************************************************************/
 
-/* $Revision: 6.120 $ 
+/* $Revision: 6.122 $ 
 * $Log: blastpri.h,v $
+* Revision 6.122  2006/04/26 12:42:36  madden
+* BlastSetUserErrorString and BlastDeleteUserErrorString moved from blastool.c to blfmtutl.c
+*
+* Revision 6.121  2005/12/29 19:56:06  madden
+* Moved functions to print tabular output to blfmtutl
+*
 * Revision 6.120  2005/12/01 15:10:23  madden
 * Gave BLASTCheckHSPInclusion external linkage (i.e. removed the static specifier).
 *
@@ -939,11 +945,6 @@ SeqIdPtr BlastGetAllowedGis PROTO((BlastSearchBlkPtr search, Int4 ordinal_id, Se
 
 Int4 BlastDeleteHeap PROTO((BLASTHeapPtr which_heap, Int4 position));
 
-/* Functions used for printing error messages. */
-
-Uint1 BlastSetUserErrorString(CharPtr string, SeqIdPtr sip, Boolean use_id);
-void BlastDeleteUserErrorString(Uint1 err_id);
-
 
 
 /* Functions used in OOF calculations */
@@ -990,14 +991,6 @@ StdSegPtr BLASTHspToStdSeg PROTO((BlastSearchBlkPtr search, Int4 subject_length,
 
 int LIBCALLBACK BlastPrintAlignInfo PROTO((VoidPtr srch));
 int LIBCALLBACK MegaBlastPrintAlignInfo PROTO((VoidPtr srch));
-void BlastPrintTabulatedResults PROTO((SeqAlignPtr seqalign, BioseqPtr query_bsp, SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, Boolean is_ungapped, Boolean believe_query, Int4 q_shift, Int4 s_shift, FILE *fp, Boolean print_query_info));
-void BlastPrintTabulatedResultsEx PROTO((SeqAlignPtr seqalign, BioseqPtr query_bsp, SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, Boolean is_ungapped, Boolean believe_query, Int4 q_shift, Int4 s_shift, FILE *fp, int *num_formatted, Boolean print_query_info));
-
-void BlastPrintTabularResults(SeqAlignPtr seqalign, BioseqPtr query_bsp,
-        SeqLocPtr query_slp, Int4 num_alignments, CharPtr blast_program, 
-        Boolean is_ungapped, Boolean is_ooframe, Boolean believe_query, 
-        Int4 q_shift, Int4 s_shift, FILE *fp, int *num_formatted, 
-        Boolean print_query_info);
 
 void
 BlastProcessGiLists PROTO((BlastSearchBlkPtr search, 
