@@ -1,6 +1,6 @@
-static char const rcsid[] = "$Id: pseed3.c,v 6.42 2004/04/01 13:43:08 lavr Exp $";
+static char const rcsid[] = "$Id: pseed3.c,v 6.47 2006/03/10 15:22:30 papadopo Exp $";
 
-/* $Id: pseed3.c,v 6.42 2004/04/01 13:43:08 lavr Exp $ */
+/* $Id: pseed3.c,v 6.47 2006/03/10 15:22:30 papadopo Exp $ */
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -35,9 +35,24 @@ Maintainer: Alejandro Schaffer
  
 Contents: high-level routines for PHI-BLAST and pseed3
 
-$Revision: 6.42 $
+$Revision: 6.47 $
 
 $Log: pseed3.c,v $
+Revision 6.47  2006/03/10 15:22:30  papadopo
+when reading multi-line PHI patterns, strip trailing newlines from each line
+
+Revision 6.46  2006/03/01 13:44:28  coulouri
+From Alejandro Schaffer: handle patterns in a case-insensitive fashion. Fixes rt#15144173
+
+Revision 6.45  2005/07/28 14:57:10  coulouri
+remove dead code
+
+Revision 6.44  2005/07/22 14:24:13  coulouri
+Incremented positions printed by the -p pattern option so that the first position is 1
+
+Revision 6.43  2005/05/02 16:03:14  coulouri
+refactor code to set db_chunk_size
+
 Revision 6.42  2004/04/01 13:43:08  lavr
 Spell "occurred", "occurrence", and "occurring"
 
@@ -1145,11 +1160,6 @@ SeqAlignPtr LIBCALL output_hits(ReadDBFILEPtr rdpt,
     SeqAlignPtr nextSeqAlign; /*new one to add*/
     SeqAlignPtr lastSeqAlign = NULL; /*last one on list*/
     GapXEditBlockPtr nextEditBlock;  /*intermediate structure towards seqlign*/
-    Int4 p; /*loop index*/
-    Int4 *tempPosThreshSequences, *tempPosResultSequences; /*temporary
-                       arrays for copying over old posSearch structure*/
-    Int4 totalNumMatches; /*number of <threshold matches from previous calls
-                            + numMatches*/
     Int4 posBaseIndex; /*loop index for copying pos results arrays*/
 
     Char tmpbuf[512];
@@ -1360,7 +1370,6 @@ void LIBCALL quicksort_hits(Int4 no_of_seq, seedResultItems *seedResults)
 {
     Int4 i;
     store_ptr sp;
-    store_node sentinel;
     store_ptr *qs; /*local array for sorting*/
 
     /*Copy the list starting from seedResults->listOfMatchingSequences 
@@ -1435,6 +1444,8 @@ static Int4 get_pat(FILE *fp, Char *stringForPattern, Char *pname)
     Char line[BUF_SIZE];  /*Line read in from the file*/
     Char *name;  /*name of pattern, if there is a name*/
     Char  *rp;   /*pattern string read in from file*/
+    Int4 length; /*length of pattern string*/
+    Int4 i; /*loop index*/
 
     name = NULL; 
     stringForPattern[0] = '\0';
@@ -1452,13 +1463,26 @@ static Int4 get_pat(FILE *fp, Char *stringForPattern, Char *pname)
         }
 	/*recognize pattern content by line starting 'PA'*/
 	if (line[0] == 'P' && line[1] == 'A') {
+
+          /* remove newlines */
+          rp = strchr(line, '\n');
+          if (rp)
+            *rp = NULLB;
+
 	  /*skip over spaces*/
 	  for (rp = &line[2]; *rp == ' '; rp++);
 	  /*copy rest of pattern to return parameter stringForPattern*/
 	  strcat(stringForPattern, rp);
 	  while (FileGets(line, BUF_SIZE, fp)) {
 	    if (line[0] == 'P' && line[1] == 'A') {
-	      for (rp = &line[2]; *rp == ' '; rp++);
+
+              /* remove newlines */
+              rp = strchr(line, '\n');
+              if (rp)
+                *rp = NULLB;
+
+	      for (rp = &line[2]; *rp == ' '; rp++)
+                ;
 	      strcat(stringForPattern, rp); 
 	    } 
 	    else {
@@ -1472,6 +1496,10 @@ static Int4 get_pat(FILE *fp, Char *stringForPattern, Char *pname)
 	      } */
 	    }
 	  }
+	  length = strlen(stringForPattern);
+	  for(i = 0; i < length; i++)
+	    if ('x' !=  stringForPattern[i])
+	      stringForPattern[i] = toupper(stringForPattern[i]);
 	  if (name) {
 	    return 1;
 	  }
@@ -1536,7 +1564,7 @@ void LIBCALL search_pat(ReadDBFILEPtr rdpt, Char *patternFileName, Boolean is_dn
 	  }
 	  for (i = 0; i < numMatches; i+=2) {
               if (is_dna) {
-                  sprintf(buffer,"HI (%ld %ld)\n", (long) hitArray[i+1], (long) hitArray[i]);
+                  sprintf(buffer,"HI (%ld %ld)\n", (long) (1 + hitArray[i+1]), (long) (1 + hitArray[i]));
                   ValNodeCopyStr(info_vnp, 0, buffer);
               } else {
                   pat_output(seq, hitArray[i+1], hitArray[i], 
@@ -1576,12 +1604,10 @@ static void do_the_seed_search(BlastSearchBlkPtr search, Int4 num_seq,
   seedResultItems * seedResults)
 {
     Int2 threadIndex, secondIndex; /*indices over threads*/
-    Int4 number_of_entries;  /*number of entries in the database*/
     seedParallelItems **seedParallelArray; /*keeps search information for each thread*/
     TNlmThread PNTR thread_array;
     VoidPtr status=NULL;
     store_ptr endOfList; /*end of list of matching seuqnecs*/
-    ReadDBFILEPtr local_rdfp; /*local copy for indexing down a list*/
     
     if (search == NULL)
         return;
@@ -1592,8 +1618,6 @@ static void do_the_seed_search(BlastSearchBlkPtr search, Int4 num_seq,
     search->thr_info->number_seqs_done = 
         search->pbp->first_db_seq;  /* The 1st sequence to compare against. */
 
-    search->thr_info->db_incr = num_seq / BLAST_NTICKS;
-    
     /*readjustment of final_db_seq needs to be after initThreadInfo*/
     if (search->pbp->final_db_seq > 0)
         search->thr_info->final_db_seq = search->pbp->final_db_seq;
@@ -1605,22 +1629,10 @@ static void do_the_seed_search(BlastSearchBlkPtr search, Int4 num_seq,
         search->thr_info->realdb_done = TRUE;
     else
         search->thr_info->realdb_done = FALSE;
+
+    ConfigureDbChunkSize(search, num_seq);
     
     if (NlmThreadsAvailable() && search->pbp->process_num > 1) {
-        number_of_entries = INT4_MAX;
-        /* Look for smallest database. */
-        /* local_rdfp = readdb_attach(search->rdfp); */
-        local_rdfp = search->rdfp;
-
-        while (local_rdfp) {
-            number_of_entries = MIN(number_of_entries, readdb_get_num_entries(local_rdfp));
-            local_rdfp = local_rdfp->next;
-        }
-        /* Divide up the chunks differently if db is small. */
-        if (search->thr_info->db_chunk_size*(search->pbp->process_num) > number_of_entries) {
-            /* check that it is at least one. */
-            search->thr_info->db_chunk_size = MAX(number_of_entries/(search->pbp->process_num), 1);
-        }
         NlmMutexInit(&(search->thr_info->db_mutex));
         NlmMutexInit(&(search->thr_info->results_mutex));
         
