@@ -44,4 +44,23 @@ NLM_EXTERN Boolean ValidateSeqAlignandACCInSeqEntry (SeqEntryPtr sep,
 
 NLM_EXTERN void    CalculateAlignmentOffsets (SeqEntryPtr sepnew, SeqEntryPtr sepold);
 
+NLM_EXTERN Boolean TruncateAlignment (SeqAlignPtr salp, Int4 num_aln_pos, Boolean from_left);
+NLM_EXTERN SeqAlignPtr MakeDiscontiguousAlignments (SeqAlignPtr salp);
+
+#define ALNMGR_GAP           -2
+#define ALNMGR_ROW_UNDEFINED -1
+
+extern void 
+AlignmentIntervalToString 
+(SeqAlignPtr salp,
+ Int4        row,
+ Int4        start,
+ Int4        stop,
+ Int4        target_row,
+ Boolean     view_whole_entity,
+ Uint1Ptr    seqbuf,
+ Uint1Ptr    alnbuf,
+ Int4 PNTR   alnbuffer_len,
+ Boolean     show_substitutions);
+
 #endif
